package rocketBugs;
/* 
	The Rod Object... like a stick but accepts input forces and calculates motion accordingly
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.io.*;
import java.text.*;

public class A_Rod extends Player {
	// static variables... only one copy needed for the class
	static final double minBranchGap = 10*Env.lengthPerActin;	// minimum distance between branch locations
	static final double mxIndividualCrossLinkForce = 1.0e-14;		// force at which a cross link breaks
	static final double mxTotalCrossLinkForce = 1.0e-14;				// cap on crosslink force acting on a single rod
	static final int minMonomers = 20;
	static final double aParallel = -0.20;  		// approx to constant in damping for parallel motion
	static final double aOrthog = 0.84;		// ...for orthogonal motion
	static final double aTurning = -0.662; 	// ...for rotational motion
	static final double bendSpringFac = (4*Math.PI/3)*(Env.actinElasticity*1e-12)*Math.pow(Env.lengthPerActin,4);	// (N-m^2)
	static final double longSpringFac = (Math.PI*Math.pow(Env.lengthPerActin,2))*(Env.actinElasticity*1e-12);		// (N)
	static double radius, diam;
	// variables for each instance
	double birthTime = 0;	// simulation time at which born
	double length;			// will be defined as number of monomers * lengthPerActin
	double ADPPiLength = 0; 	// length of the filament that has ADP-Pi bound nucleotide-- hydrolized but not dissociated
	double ADPLength = 0;		// length of completely hydrolyzed nucleotide.
	int numOfMonomers;		// number of actin monomers in this rod.
	int numADPPiMonomers; 	// number of ADP-Pi actin monomers in this rod.
	int numADPMonomers;		// number of hydrolyzed (ADP) actin monomers in this filament.
	Pt3D end1 = new Pt3D();	// end points of the rod... end1 is pointed, end2 is barbed
	Pt3D end2 = new Pt3D();		
	Pt3D end1InLMFrame = new Pt3D();	// end1 in Listeria's coordinate system with listeria.coord as origin, i.e. XTox(end1-listeria.coord)
	Pt3D end2InLMFrame = new Pt3D();	// ditto for end2
	Pt3D tip1 = new Pt3D();		// very tip of rod... end point + .5 actin monomer size ... decide whether to use this later
	Pt3D tip2 = new Pt3D();		
	double helixAng = 0;		// defines start of helix relative to body-fixed y-axis
	double tipClearance = 100;
	boolean plusCapped = false;				// flag to indicate that the plus end of this rod is capped
	boolean minusCapped = false;			// flag to indicate that the minus end of this rod is capped
	boolean changedLength = false;			// flag to indicate a change in length.
	boolean aBranch = false;				// flag to indicate that this rod is branched off of another
	// *** declarations for linking to other rods ***
	int linkerCount = 0;					// number of rods this rod is linked with.
	A_Rod [] linkers = new A_Rod [1];		// array of rods to which this rod is linked
	double [] myLinkDist = new double [1];	// array of length to link from end1 for this rod
	double [] linkLinkDist = new double [1];	// array of length to link from end1 for linked rod
	Pt3D [] linkOrientation = new Pt3D [1];	// array of vectors defining link orientations
	double [] linkAng = new double [1];		// array of angle between linked rods
	Pt3D crossLinkForces = new Pt3D();		// sum of crosslink forces
	//*** some declarations for ARP2/3 binding ***
	Object arpLock = new Object();		// synchronization lock for adding arps
	A_Arp [] arps = new A_Arp [10];		// array to keep track of side-bound ARP2/3
	int arpCount = 0;						// counter of total ARP2/3 bound
	int branches = 0;						// counter of branches = # of active ARP2/3
	//*** declarations for ActA binding
	double [] actALocs = new double [10];
	int actACt = 0;
	//*** some declarations for ADF/Cofilin binding ***
	double [] adfLoc = new double [10];		// array to keep track of bound ADF/Cofilin
	int adfCount = 0;						// counter of ADF/Cofilin bound to this filament
	//*** declarations for tethering of rod to Listeria
	Pt3D tetherForceVec, tetherTorqueVec;	// recycled for tether force/torque calculations
	Pt3D [] tetherEffects = new Pt3D [2];	// array to hold tetherForceVec and tetherTorqueVec
	Pt3D tetherForceSum = new Pt3D();
	Pt3D tetherTorqueSum = new Pt3D();
	
	// severing related
	boolean justSevered = false;	// flag to let us know that this rod was severed last step	
	double severLocation;
	A_Rod rodFromSever;				// pointer to new rod created by sever event
	
//	 fluid coupling to listeria
	boolean fluidCoupleToBug = false;
	boolean end2Coupled = true;
	Pt3D oldEnd1 = new Pt3D();
	Pt3D oldEnd2 = new Pt3D();
	Pt3D end1Delta = new Pt3D();
	Pt3D end2Delta = new Pt3D();
	Pt3D end1VelocInLisFrame = new Pt3D();
	Pt3D end2VelocInLisFrame = new Pt3D();
	ValueTracker aveEnd1VelocInLisFrame = new ValueTracker (5, ValueTracker.PT3D_TYPE);
	ValueTracker aveEnd2VelocInLisFrame = new ValueTracker (5, ValueTracker.PT3D_TYPE);
	
	static HistogramPlus filLife = new HistogramPlus (100,0,20,Env.datFilePath,"filLife",null);

	// for testing of tipClearance and nucleation probs
	double unModAddProb, modAddProb;
	boolean polymPossible = true;
	double arpProb = 0;
	boolean arpPossible = false;
	double plusCapProb;
	boolean plusCapPossible = true;
	boolean aNewNuc = false;	// is this a newly nucleated rod

	// specialized function flags
	boolean linked = false;		// flag says if something is keeping this rod is linked to Listeria
	
	// *****************************************
	static DecimalFormat toScreenFormat = new DecimalFormat ("#0.000000#;#0.000000#");
	Pt3D aPt = new Pt3D();	// a blank Pt3D to use whenever.
	// a few placeholders only used when saving config to file
	int [] linkerIds = new int [30];
	// some declarations related to painting of the rod
	Pt3D ptA = new Pt3D(); 
	Pt3D ptB = new Pt3D();
	Pt3D dR = new Pt3D();
	Pixel pixA = new Pixel();
	Pixel pixB = new Pixel();
	Pixel pixAp = new Pixel();
	Pixel pixBp = new Pixel();
	double thMon, thLine;			// angles for helix blob and line painting, respectively
	double bMonLoc, bLineLoc;		// location along rod used for helix blob and line painting, repectively
	boolean oddMon;				// boolean to specify odd or even monomer number
	
	public A_Rod (Pt3D initCoord,Pt3D initUVec,int numOfMonomers,double diam) {
		super();
		this.numOfMonomers = numOfMonomers;
		this.birthTime = Env.simulationTime;
		A_Rod.diam = diam;
		length = numOfMonomers*Env.lengthPerActin;
		radius = diam/2;
		// initialize positions, angles, and all velocities
		coord.copy(initCoord);
		uVec.copy(initUVec);
		playerColor = Env.ATPActinColor;
		initialize();
		calculateProperties();
	}
	
	public static A_Rod makeNewRod(Pt3D xyzCoords, Pt3D uVec, int numMonomers) {
		A_Rod newRod = new A_Rod (xyzCoords, uVec, numMonomers, Env.fActinDiameter);
		return newRod;
	}
	
	public static void shiftRod(A_Rod rod, int numToShift) {
		rod.coord.inc(numToShift*Env.lengthPerActin,rod.uVec);
		rod.end1.add(rod.coord, -rod.length/2, rod.uVec);
		rod.end2.add(rod.coord, rod.length/2, rod.uVec);
	}
	
	public void sepaku () {
		//System.out.println("In A_Rod.sepaku()");
		// get rid of all links to this player
		end1 = null;
		end2 = null;
		end1InLMFrame = null;
		end2InLMFrame = null;
		tip1 = null;
		tip2 = null;	
		linkers = null;
		myLinkDist = null;
		linkLinkDist = null;
		linkOrientation = null;
		linkAng = null;
		crossLinkForces = null;
		arps = null;
		arpLock = null;
		actALocs = null;
		adfLoc = null;
		tetherForceVec = null; 
		tetherTorqueVec = null;	
		tetherEffects = null;
		tetherForceSum = null;
		tetherTorqueSum = null;
		rodFromSever = null;
		
		oldEnd1 = null;
		oldEnd2 = null;
		end1Delta = null;
		end2Delta = null;
		end1VelocInLisFrame = null;
		end2VelocInLisFrame = null;
		aveEnd1VelocInLisFrame = null;
		aveEnd2VelocInLisFrame = null;
		
		aPt = null;
		linkerIds = null;
		ptA = null;
		ptB = null;
		dR = null;
		pixA = null;
		pixB = null;
		pixAp = null;
		pixBp = null;
		
		
		super.sepaku();
		justRemoved = true;
	}
	
	public void initialize () {
		// this method assumes the uVec has been set, or it unchanged
		// store old end1 and end2 values
		oldEnd1.copy(end1);
		oldEnd2.copy(end2);
		// euler angles from uVec
		angle.eulerAnglesFromUVec(uVec);
		// calculate cos and sin of Euler angles only once during each time step
		cosAng.cosine(angle);
		sinAng.sine(angle);
		// find the transformation matrices at this time step
		transMat ();
		// find the transverse unit vectors that defines the orientation of the rod
		t1Vec.xToX(this, yUnitVector); 
		t2Vec.xToX(this, zUnitVector);
		// find the end points of the rod
		if (end1 == null) { myStructure.report(); }
		end1.add(coord, -length/2, uVec);
		end2.add(coord, length/2, uVec);
		// subtract end positions and calculate end velocities in listeria frame
		end1VelocInLisFrame.sub(1/Env.deltaT,end1,oldEnd1);
		end2VelocInLisFrame.sub(1/Env.deltaT,end2,oldEnd2);
		end1VelocInLisFrame.XTox(Player.listeria);
		end2VelocInLisFrame.XTox(Player.listeria);
		aveEnd1VelocInLisFrame.registerValue(end1VelocInLisFrame);
		aveEnd2VelocInLisFrame.registerValue(end2VelocInLisFrame);
		// calculate the positions of end1 and end2 in the listeria frame with listeria.coord as origin
		end1InLMFrame.sub(end1,Player.listeria.coord);
		end2InLMFrame.sub(end2,Player.listeria.coord);
		end1InLMFrame.XTox(Player.listeria);
		end2InLMFrame.XTox(Player.listeria);
	}
	
	public void calculateProperties () {
		// define the constants for motion of this rod in viscous medium
		// Remember that the dimensions we've been using are in micrometers so....
		length = numOfMonomers*Env.lengthPerActin;
		ADPPiLength = numADPPiMonomers*Env.lengthPerActin;
		ADPLength = numADPMonomers*Env.lengthPerActin;
		if (!structureMember) { 	// only calculate individual viscous drags for solo filaments
			// **WARNING** below a certain number of monomers, depending on values like aParallel, etc
			// the rod approximation will give NaN... hence the IF statement below
			if (numOfMonomers >= 6) {
				double lengthM = length*1.0e-6;  	// length in meters
				double radiusM = radius*1.0e-6;
				double denomLogTerm = Math.log(length/(2*radius));	//dimensionless
				bTransGam.x = (2*Math.PI*Env.aeta*lengthM)/(denomLogTerm + aParallel);
				bTransGam.y = (4*Math.PI*Env.aeta*lengthM)/(denomLogTerm + aOrthog);
				bTransGam.z = bTransGam.y;
				bRotGam.x = 4*Math.PI*Env.aeta*radiusM*radiusM*lengthM;	// drag for turning about x
				bRotGam.y = (Math.PI*Env.aeta*Math.pow(lengthM,3))/(3*(denomLogTerm + aTurning));
				bRotGam.z = bRotGam.y;
			} else {
				// use values for 6 monomer filament
				double lengthM = 6*Env.lengthPerActin*1.0e-6;  	// length in meters
				double radiusM = radius*1.0e-6;
				double denomLogTerm = Math.log(6*Env.lengthPerActin/(2*radius));	//dimensionless
				bTransGam.x = (2*Math.PI*Env.aeta*lengthM)/(denomLogTerm + aParallel);
				bTransGam.y = (4*Math.PI*Env.aeta*lengthM)/(denomLogTerm + aOrthog);
				bTransGam.z = bTransGam.y;
				bRotGam.x = 4*Math.PI*Env.aeta*radiusM*radiusM*lengthM;	// drag for turning about x
				bRotGam.y = (Math.PI*Env.aeta*Math.pow(lengthM,3))/(3*(denomLogTerm + aTurning));
				bRotGam.z = bRotGam.y;
			}
			
			//*** INSERT: scale factor for aeta
			
			// check NaN
			if (!Pt3D.CheckPt3D(bTransGam)) { talkln ("bTransGam NaN NaN NaN"); }
			if (!Pt3D.CheckPt3D(bRotGam)) { talkln ("bRotGam NaN NaN NaN"); }
			
			// call method to modify drags based on different physical realities
			//dragModifications(length);
			//dragModifications(this.length, numOfMonomers*numOfMonomers*Env.lengthPerActin);
			
			//********
			//dragModifications based on functions to increase drag artificially and try turning off any fixation
			//********
			// x represents numOfMonomers, y represents the scale-factor, ie EffectiveNumMon
			double x1 = 10;	//these functions start at numOfMonomers = 10
			double y1 = Env.lubyPhelpsScaleAt10Mon; //the 1/(D/D0) for 10 monomers, ie where LubyPhelps method leaves off
			double x2 = Env.numMonomersAtGoalHighDrag;
			double y2 = Env.goalHighDrag;
			
			if (Env.useOldDrag){
				dragModifications(length, numOfMonomers*numOfMonomers*Env.lengthPerActin);
			}else if (Env.adhesionsByLin){
				double aLin = (y2-y1)/(x2-x1);
				double bLin = y1-(aLin*x1);
				double linearEffectiveNumMon = (aLin*numOfMonomers) + bLin;
				double effectiveLength = linearEffectiveNumMon * Env.lengthPerActin;
				dragModifications(length, effectiveLength);
				
			}else if(Env.adhesionsByQuad){
				double aQuad = ((x2*x2)-(x1*x1))/(y2-y1);
				double bQuad = y1-(aQuad*x1*x1);
				double quadEffectiveNumMon = (aQuad*numOfMonomers*numOfMonomers) + bQuad;
				double effectiveLength = quadEffectiveNumMon * Env.lengthPerActin;
				dragModifications(length, effectiveLength);
				
			}else if (Env.adhesionsByExp){
				double bExp = (Math.log(y2/y1))/(x2-x1);
				double aExp = y1/(Math.exp(bExp*x1));
				double exponentEffectiveNumMon = aExp*Math.exp(bExp*numOfMonomers);	
				double effectiveLength = exponentEffectiveNumMon * Env.lengthPerActin;
				dragModifications(length, effectiveLength);
				
			}else{
				double effectiveLength = numOfMonomers*numOfMonomers*Env.lengthPerActin;
				dragModifications(length, effectiveLength);
			}
			

			
			bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
			bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
			
		}
	}
	
	public void movePlayer () {
		// update age
		age += Env.deltaT;
		if (linkerCt > 0) { linked = true; }
		
		sumForces();
		
		// calculate forces, velocities, etc and move, unless actin is fixed
		if (!structureMember) {
			// explicit Euler approximation to ODE solution
			// first work in coordinates aligned with the rod....
			// transform forces and torques into body-fixed axis....
			bForceSum.XTox(this, forceSum);
			bTorqueSum.XTox(this, torqueSum); 
			// add random forces, given in body-fixed frame
			if (Env.shakeActin) {
				calcRandomForces();
				// scale these forces down if tethered
				if (linked) {
					double scaleFac = linkerCt*((Env.tetherBrownianScale/length)-((Env.tetherBrownianScale/Env.cutoffLength)-(1.0)));
					//double scaleFac = linkerCt*Env.tetherBrownianScale/length;	//old (pre050106) way
					//**remove scalefac and make it based on length (ie if >Env.thetherBrownianScale then scaleFac=1
					if (length > linkerCt * Env.cutoffLength) { scaleFac = 1.0; }
					//if (scaleFac < 1.0) { scaleFac = 1.0; }
					randForces.scale(1.0/scaleFac);
					randTorques.scale(1.0/scaleFac);
				}
				// then add to force and torque sums
				bForceSum.inc(randForces);
				bTorqueSum.inc(randTorques);
			}
			// now that the forces and torques are in the body fixed frame, we apply the eoms....
				bVeloc.div(1.0e6, bForceSum, bTransGam);		// in micron/sec
				bAngVeloc.div(bTorqueSum, bRotGam);			// in radians/sec
						
			// ....then transform back into fixed coordinate frame
			veloc.xToX(this, bVeloc);
			
			// to apply the body-fixed angular velocities, approximate new unit vector from arc of rotations
			double endTransInZ = -bAngVeloc.y * Env.deltaT;	// arclength out at 1 micron
			double endTransInY = bAngVeloc.z * Env.deltaT;
			uVec.setVals(1,endTransInY,endTransInZ);	// in body-fixed, not a unit vector yet
			uVec.xToX(this);	// make in fixed-frame, not a unit vector yet
			uVec.unitVec();		// make a unit vector
			
			// and finally move everything
			coord.inc(Env.deltaT, veloc);		// just position = velocity*time
			
			// body-fixed x angular velocity is same as euler angle phi rotation
			angle.z += bAngVeloc.x*Env.deltaT;
		}
		
		initialize();	// convert center of mass position and uVec into end positions and Euler angles
		
		if (!structureMember) { fixAngles(); }	// call the local override of fixAngles AFTER end1 and end2 are established

		
		// reset counters, force sums, velocity vectors
		resetCounters();				// call to reset collision counters and force sums
	
	}
	
	public void resetCounters() {
		super.resetCounters();
		justSevered = false;
		actACt = 0;
	}
	
	public void interactionSim (double dT) {
		
		calculateTipClearance();	// figure room for monomer addition and distance from bacteria					
				
		growRod(dT);				// check both plus and minus ends for potential growth
		
		checkCaps(dT);				// plus and minus ends cap on/off
		
		nucleotideHydrolysis(dT);		// advance the hydrolyzed fuse, so to speak
		
		//checkCofilinBinding(dT);		// check for ADF/Cofilin binding
		
		//cofilinSevering(dT);			// check each bound cofilin for severing
		
		checkRemove();			// check for possible 'death' of this filament
		
		checkAdhesions(dT);			// check for addition of adhesions... a hackeola
		
		viscousBlobSim(length,dT);	// check binding/unbinding of viscous blobs
		
		checkFixing ();			// check if this rod should be fixed in space
		
		if (changedLength) { 
			calculateProperties (); 
			changedLength = false;
		}
		
		
		// some flags that must be reset AFTER interactionSim is over
		linked = false;			// reset to false each time-step... ActA will set to true if attached
	}
	
	public void calculateTipClearance () {
		if (Env.listeriaPresent) {
			double normalDist, cosAngTween;
			Pt3D end2Inx = Player.listeria.tox(end2);
			if ((end2Inx.x > 0) & (end2Inx.x < Listeria.cylLength)) {	// if end2 is up cylindrical side
				Pt3D cenPt = Player.listeria.toX(new Pt3D(end2Inx.x,0,0));
				normalDist = Pt3D.ptDist(end2, cenPt) - Listeria.cylDiam/2;
				cosAngTween = Pt3D.Dot(uVec, Pt3D.UnitVec(cenPt, end2));
			} else {
				if (end2Inx.x > Listeria.cylLength) {  // end2 is closest
					normalDist = Pt3D.ptDist(end2, Player.listeria.rec2) - Listeria.capRad; 
					cosAngTween = Math.abs(Pt3D.Dot(uVec, Pt3D.UnitVec(Player.listeria.rec2, end2)));		// cos(angle) between unit vec of rod and normal
				} else {  // end1 is closest
					normalDist = Pt3D.ptDist(end2, Player.listeria.rec1) - Listeria.capRad; 
					cosAngTween = Math.abs(Pt3D.Dot(uVec, Pt3D.UnitVec(Player.listeria.rec1, end2)));		// cos(angle) between unit vec of rod and normal
				}
			}
			//if ((cosAngTween < 0.001) | (normalDist > Env.lengthPerActin)) { 	// don't divide by zero
			if (cosAngTween < 0.001) {
				tipClearance = 1e2;	// a big number
			} else {
				tipClearance = normalDist/cosAngTween;
				if (tipClearance < Env.actinAdd1Dist) { 
					Env.registerCloseTip(); 
					if (end2Inx.x < 0) { Env.registerTipOnLisSphere(); }
				}
			}
		} else {
			tipClearance =  1e2;	// a big number
		}
	}
	
	public boolean inCylinderViscousBox () {
		double end2OffCylAxisSqrd = 1e6;
		double end1OffCylAxisSqrd = 1e6;
		boolean end2In = false;
		boolean end1In = false;
		
		//		 ** fast checks to see if rod ends are in simple bounding shape **
		if ((end2InLMFrame.x > -Listeria.halfCylLength) & (end2InLMFrame.x < Listeria.halfCylLength)) { 
			end2OffCylAxisSqrd = end2InLMFrame.y*end2InLMFrame.y + end2InLMFrame.z*end2InLMFrame.z;
			if (end2OffCylAxisSqrd < Listeria.capRadSqrd + Env.viscousShellThickness) {
				end2In = true;  // end2 on cylinder
			}
		} 
		
		if ((end1InLMFrame.x > -Listeria.halfCylLength) & (end1InLMFrame.x < Listeria.halfCylLength)) { 
			end1OffCylAxisSqrd = end1InLMFrame.y*end1InLMFrame.y + end1InLMFrame.z*end1InLMFrame.z;
			if (end1OffCylAxisSqrd < Listeria.capRadSqrd + Env.viscousShellThickness) {
				end1In = true;  // end1 on cylinder
			}
		} 
		
		if ((end2In) | (end1In)) { 
			fluidCoupleToBug = true;
			end2Coupled = (end2OffCylAxisSqrd <= end1OffCylAxisSqrd);
			return true;
		} else {
			fluidCoupleToBug = false;
			return false;
		}
	}
	
	public Pt3D filamentLocToPt3D (double filamentLoc) {
		Pt3D ptOnFilament = Pt3D.Scale(filamentLoc, uVec);
		ptOnFilament.inc(end1);
		return ptOnFilament;
	}
	
	public void subsumedMove (Pt3D centroid, Pt3D cOff, Pt3D newuVec, Pt3D newt1Vec) {
		coord.add(centroid, cOff);
		uVec.copy(newuVec);
		t1Vec.copy(newt1Vec);
		// find Euler angles from the unit vector
		angle.x = Math.atan2(uVec.y, uVec.x);
		angle.y = Math.asin(-uVec.z);
		angle.z = 0;
		movePlayer();		// now go on and do the rest of the important stuff besides dynamics
	}
	
	public void fixAngles () {
		// If psi and theta are out of bounds then fix... note that phi is set to zero
		if ((Math.abs(angle.x) > 2*Math.PI) | (Math.abs(angle.y) > Math.PI/2)) {
			angle.x = Math.atan2(end2.y-coord.y, end2.x-coord.x);
			angle.y = Math.asin((end1.z-end2.z)/length);
			//angle.z = 0;
		}
	}
	
	public double getStiffness (Pt3D tetherVec) {
		// calculates the stiffness of this rod for tethering spring (N/m)
		// two springs in series... one from the rod and one from the stuff it's crosslinked to (cytoskeletal mesh)
		double k1 = Env.actinBaseSpring;		// this value represents some sort of compliance of the vague cytoskeletal mesh
		double kbend = bendSpringFac/Math.pow(length,3);	// (N/m) bending stiffness of this filament assuming fixed at end1
		double klong = longSpringFac/length;		// (N/m) longitudinal stiffness of this filament
		double theDot = Pt3D.Dot(tetherVec,uVec);
		double tetherAng = Math.acos(theDot);
		double k2 = kbend*Math.abs(Math.sin(tetherAng))+klong*Math.abs(theDot);
		double stiffness = k1*k2/(k1+k2);
		//talkln ("Stiffness:  kbend = " + String.valueOf(kbend) + ", klong = " + String.valueOf(klong) + ", k2 = " + String.valueOf(k2));
		return stiffness;
	}
	
	public double getMaxTetherStretch() {
		return Env.maxTetherStretch;
	}
	
	public double getMaxAllowTetherForce() {
		return Env.maxAllowTetherForce;
	}
	
	public double getBuckleForce() {
		return 1E-12;
	}
	
	public void checkBuckling (double colForce) {
		double buckleForce = getBuckleForce();
		if (colForce > buckleForce) {
			buckleAtten = buckleForce/colForce;
		} else {
			buckleAtten = 1;
		}
	}
	
	private void growRod (double dT) {
		if (! plusCapped) {
			// Determine whether or not to add an actin monomer to the plus end of the rod at this time step...
			// here we've only included the ATP-actin and Profilin-ATP-actin pools in the calculation.
			
			//*** Profilin-ATP monomer addition **REMOVE**
			//double profilin_ActinConc = impP.getProfilin_ATP_ACTINValue(end2);
			polymPossible = true;	// set flag that, at least right now, we think it's possible to polymerize
			double profilin_ActinConc = Env.PROFILIN_ATP_ACTINConc;
			double addProfilinMonProb = (Env.kPlusProfilin_On*profilin_ActinConc - Env.kPlusProfilin_Off)*dT;
			unModAddProb = addProfilinMonProb;
			double addProfilinProb = tipClearanceModifiedAddProb(addProfilinMonProb);
			modAddProb = addProfilinProb;
			double dicethrow = Rdm.uRdm(this);
			if (dicethrow < addProfilinProb) { // add Profilin-ATP-actin
				end2.inc(Env.lengthPerActin, uVec);
				coord.add(.5, end1,end2);
				numOfMonomers ++;
				//impP.seedProfilin_ATP_ACTIN(end2, -1);	// subtract a Profilin-actin monomer from the pool in this grid.
				changedLength = true;
				Env.registerPlusMon(this);
			}
			
			// *** ATP monomer addition/subtraction  **REMOVE**
			//double ATP_ActinConc = impP.getActin_ATPValue(end2);
			double ATP_ActinConc = Env.ACTIN_ATPConc;
			double addATPMonProb = (Env.kPlusATP_On*ATP_ActinConc - Env.kPlusATP_Off)*dT; 
			dicethrow = Rdm.uRdm(this);
			if (addATPMonProb >= 0) {
				if ((dicethrow < addATPMonProb) & (tipClearance > Env.lengthPerActin)) { // add ATP-actin
					end2.inc(Env.lengthPerActin, uVec);
					coord.add(.5, end1,end2);
					numOfMonomers ++;
					//impP.seedActin_ATP(end2, -1);	// subtract an actin monomer from the pool in this grid.
					changedLength = true;
					Env.registerPlusMon(this);
				}
			} else {
				if ((dicethrow < -addATPMonProb) & (numOfMonomers > 1)) {	// remove monomer from filament, but not last one
					end2.inc(-Env.lengthPerActin, uVec);
					coord.add(.5, end1, end2);
					numOfMonomers += -1;
					//impP.seedActin_ATP(end2, 1);	// add actin monomer back to pool... assume always ATP because of rapid nucleotide exchange on tip
					changedLength = true;
				}
			}
		}
		
		if (! minusCapped) {  //***REMOVE THIS****
			// Determine whether or not to add an actin monomer to the minus end of the filament
			// We've only included ATP-actin as polymerizable and assumed that it'd be an ADP coming off.
			//double actinConc = impP.getActin_ATPValue(end1);
			double actinConc = Env.ACTIN_ATPConc;  //**REMOVE
			double addATPMonProb = (Env.kMinusATP_On*actinConc - Env.kMinusADP_Off)*dT;
			double dicethrow = Rdm.uRdm(this);
			if (addATPMonProb >= 0) {
				if (dicethrow < addATPMonProb) {	// grow end1 by one actin monomer
					end1.inc(-Env.lengthPerActin, uVec);
					coord.add(.5, end1, end2);
					numOfMonomers++;
					numADPMonomers++;			// assume for now that minus end is essentially ADP-actin
					//impP.seedActin_ATP (end1, -1);	// subtract an ATP-actin monomer from pool in this grid
					changedLength = true;
					for (int i = 0; i < arpCount; i++) { arps[i].loc += Env.lengthPerActin; }
					for (int i = 0; i < adfCount; i++) { adfLoc[i] += Env.lengthPerActin; }
					helixAng += -Env.helixAngleInc;	// change helix angle 
				}
			}else {
				// don't remove the last monomer this way... fixes problem with tethered rods shrinking to nothing
				if ((dicethrow < -addATPMonProb) & (numOfMonomers > 1)) {	// shrink end1 by one actin monomer
					end1.inc(Env.lengthPerActin, uVec);
					coord.add(.5, end1, end2);
					numOfMonomers += -1;
					if (numADPMonomers > 0) {
						numADPMonomers += -1;
					} else {
						if (numADPPiMonomers > 0) { numADPPiMonomers += -1; } 
					}
					//impP.seedActin_ADP(end1, 1);	// add ADP-actin monomer back to pool in this grid
					changedLength = true;
					for (int i = 0; i < arpCount; i++) { 
						arps[i].loc += -Env.lengthPerActin; 
						if (arps[i].loc < 0) { removeArp(i); }	// the rod has shrunk to this branch point -> remove
					}
					for (int i = 0; i < adfCount; i++) { adfLoc[i] += -Env.lengthPerActin; }
					helixAng += Env.helixAngleInc;
				}
			}	
		}	
	}
	
	/*private double tipClearanceModifiedAddProb(double curAddProb) {
		if (tipClearance > Env.lengthPerActin) { 
			return curAddProb;     // don't change prob if plenty of room
		} else {
			if (Env.modifyPolymProb) {
				double L1 = length*length/(3*Env.actinPersistenceLength*Env.lengthPerActin);
				double normFact = Math.exp(1000*Env.lengthPerActin/L1);
				double eFact = Math.exp(1000*tipClearance/L1);
				if (selected) {
					talkln ("tipClearance < actinGap... prob is " + String.valueOf(eFact/normFact));
					talkln ("	L1 = " + String.valueOf(L1));
					talkln ("	normFact = " + String.valueOf(normFact));
					talkln ("	eFact = " + String.valueOf(eFact));
				}
				return (Env.actinFlexFactor*eFact/normFact)*curAddProb;
			} else {
				return 0;
			} 
		}
	}*/
	
	private double tipClearanceModifiedAddProb(double curAddProb) {
		if (tipClearance > 2*Env.lengthPerActin) { return curAddProb; }   // don't change prob if plenty of room
		if (tipClearance < 0) {
			polymPossible = false; 
			return 0; 
		}
		
		if (Env.modifyPolymProb) {
			double linSqVar = Math.pow(tipClearance/(2*Env.lengthPerActin),2);
			return linSqVar*curAddProb;
		} else {
			return 0; 
		}
	}
	
	public void checkCaps (double dT)  {
		if (!plusCapped) { checkPlusCapOn(dT); } else { checkPlusCapOff(dT); }
		if (!minusCapped) { checkMinusCapOn(dT); } else { checkMinusCapOff(dT); } 
	}
		
	public void checkPlusCapOn (double dT) {
		// Conditions for capping
		boolean maybeCap = true;
		if (tipClearance < 2*Env.radOfCap) { maybeCap = false; }			// don't cap if not enough room
		if (maybeCap) {
			//double capProb = Env.kCap_On * impP.getCapValue(end2)*dT;
			double capProb = Env.kCap_On * Env.CAPConc * dT;
			plusCapPossible = true;
			plusCapProb = capProb;
			if (Rdm.uRdm(this) < capProb) { plusCapOn (); }
		} else {
			plusCapPossible = false;
			plusCapProb = 0;
		}
	}
	
	public void checkPlusCapOff (double dT) {
		// Check to see if a capped rod should come undone
		//	UncappingProb = k_Off 
		double unCapProb = Env.kCap_Off * dT;
		double dicethrow = Rdm.uRdm(this);
		if (dicethrow < unCapProb) { 
			plusCapOff ();
		}
	}
	
	public void plusCapOn () {
		//impP.seedCap (end2, -1);	// remove one capping protein from pool
		plusCapped = true; 
	}
	
	public void plusCapOff () {
		//impP.seedCap (end2,1);	// add one capping protein to the pool
		plusCapped = false;
	}
	
	public void checkMinusCapOn (double dT) {
		// Assign minus end cap (ARP2/3) to this rod according to the equation:
		// 	CappingProb = k_On * [Arp2/3]
		double capRate;
		if (ADPLength > 0) { capRate = Env.kADPFactinArp_On; } else { capRate = Env.kATPFactinArp_On; }
		double capProb = capRate*impP.getARPValue(end1)*dT;
		double dicethrow = Rdm.uRdm(this);
		if (dicethrow < capProb) { 
			//impP.seedARP(end1, -1);		// remove one ARP2/3 from pool
			minusCapped = true; 
		}
	}
	
	public void checkMinusCapOff (double dT) {
		if (!aBranch) {
			// Check to see if a minus capping should come undone
			// 	UncappingProb = k_Off
			double unCapRate;
			if (ADPLength > 0) { unCapRate = Env.kADPFactinArp_Off; } else { unCapRate = Env.kATPFactinArp_Off; }
			double unCapProb = unCapRate*dT;
			double dicethrow = Rdm.uRdm(this);
			if (dicethrow < unCapProb) {
				//impP.seedARP(end1, 1);		// add one capping protein to the pool 
				minusCapped = false; 
			}
		}
	}

	
	public void linkerOn (Linker newLinker) {
		arpPossible = true;
		arpProb = ActA.arpOnProb;
	}
	
	public void linkerOff (Linker newLinker) {
		arpPossible = false;
		arpProb = 0;
	}
	
	public boolean validActASite (double bindLoc) {
		if ((bindLoc < 0) | (bindLoc > length)) { return false; }	// not valid if off of rod
		if (bindLoc < ADPLength) { return false; }					// only bind to ATP or ATP-Pi actin
		if (bindLoc < Env.minActAgapLength) { return false; }		// not valid if too close to end1
		for (int i=0; i < actACt; i++) {
			if (Math.abs(bindLoc-actALocs[i]) < Env.minActAgapLength) { return false; }
		}
		return true;
	}
	
	public void registerActA (double bindLoc) {
		// first check if we need to grow array
		if (actACt >= actALocs.length) {
			double [] tempActALocs = actALocs;
			actALocs = new double [actACt+10];
			for (int i=0;i<actACt;i++) {
				actALocs[i] = tempActALocs[i];
			}
			tempActALocs = null;
		}
		actALocs[actACt] = bindLoc;
		actACt++;
	}
	
	public boolean tooCloseARP (double bindLoc) {
		if (bindLoc < Env.minARPgapLength) { return true; }
		for (int i = 0; i < arpCount; i++) {
			if (Math.abs(bindLoc-arps[i].loc) < Env.minARPgapLength) { return true; }
		}
		return false;
	}
	
	public A_Arp addArp (double bindLoc) {
		synchronized (arpLock) {
			// make a new Arp2/3 formed branch at the given location
			if (! tooCloseARP(bindLoc)) {
				// check if we need to expand the Arp array
				if (arpCount >= arps.length) {
					A_Arp [] tempArpArray = arps;
					arps = new A_Arp [arpCount + 10];
					for (int i=0;i<arpCount;i++) {
						arps[i] = tempArpArray[i];
					}
					tempArpArray = null;
				}
				arps[arpCount] = A_Arp.makeActiveArp(bindLoc, this);
				arpCount ++;
				branches ++;
				Env.registerArp(this);
				return arps[arpCount-1];
			} else {
				return null;
			}
		}
	}
	
	public void removeArp (int i) {
		synchronized (arpLock) {
			if (arps[i].active) {
				branches += -1;	// one less branch on this filament
				arps[i].branch.aBranch = false;	// this filament is no longer a branch off another.
				myStructure.removeSubBranch(arps[i].branch);
				if (arps[i].branch.branches > 0) { 	// create a new ActinBranch with this sub-tree
					ActinBranch newBranch = ActinBranch.makeNewActinBranch();
					newBranch.addSubBranch(arps[i].branch);
					newBranch.calculateProperties();
				}
			} else {
				// add only inactive ARP2/3 back to pool... active still bound to dissociated branch
				aPt.add(end1, arps[i].loc, uVec);	// calculate location of inactive ARP2/3 to remove
				//impP.seedARP(aPt, 1);			// add inactive ARP2/3 back to pool
			}
			arpCount += -1;	// one less ARP2/3 on this filament
			arps[i] = arps[arpCount]; 	// fill in array gap
			arps[arpCount] = null;	
		}
	}
	
	public void nucleotideHydrolysis (double dT) {
		int numATP = numOfMonomers - (numADPPiMonomers + numADPMonomers);
		if (numATP > 1) {
			double ATPtoADPPiProb = Env.kATPtoADPPi*(numOfMonomers-numADPPiMonomers-numADPMonomers)*dT;
			double dicethrow = Rdm.uRdm(this);
			if (dicethrow < ATPtoADPPiProb) {
				numADPPiMonomers ++;
				changedLength = true;
			}
		}
		if (numADPPiMonomers > 0) {
			double ADPPitoADPProb = Env.kADPPitoADP*(numADPPiMonomers)*dT;
			double dicethrow = Rdm.uRdm(this);
			if (dicethrow < ADPPitoADPProb) {
				numADPPiMonomers += -1;
				numADPMonomers ++;
				changedLength = true;
			}
		}
	}
	
	public void checkCofilinBinding (double dT) {
		// Allow ADF/Cofilin proteins to bind...
		// Only binding on ADP actin for now...
		// 	bindProb = k * [ADF] 
		double ADFconc = impP.getADFValue(end1);
		double bindProb = Env.kADPFactinCofilin_On*numADPMonomers*ADFconc*dT;
		double dicethrow = Rdm.uRdm(this);
		if (dicethrow < bindProb) {
			//talkln ("adf/cofilin has bound");
			// check if we need to grow array
			if (adfCount >= adfLoc.length) {
				double [] tempAdfLoc = adfLoc;
				adfLoc = new double [adfCount + 10];
				for (int i=0;i<adfCount;i++) {
					adfLoc[i] = tempAdfLoc[i];
				}
				tempAdfLoc = null;
			}
			adfLoc[adfCount] = Rdm.uRdm(this)*ADPLength;
			aPt.add(end1, adfLoc[adfCount], uVec);	// define location of binding
			//impP.seedADF(aPt, -1);					// subtract a free ADF from implicit pool
			adfCount++;
		}
	}
	
	public void cofilinSevering (double dT) {
		// this just removes adf/cofilin beyond end1
		for (int i=0; i<adfCount; i++) {
			if (adfLoc[i] < 0) {
				removeCofilin(i);
			}
		}
		/*if (numOfMonomers > 1) {	// only sever filaments that have at least 2 monomers
			double severProb = Env.kCofilinSever;	
			for (int i = 0; i < adfCount; i++) {
				if (true) { //(Rdm.uRdm(this) < severProb) {
					severFilament (i);
					return;
				}
			}
		}*/
	} 
	
	public void severFilament (int i) {
		int numMon1 = (int)(Math.rint(adfLoc[i]/Env.lengthPerActin));		// find closest actin monomer cleavage pt
		if (numMon1 == numOfMonomers) { numMon1 += -1; } 	// upper limit... must be at least one monomer left for rod2.
		if (numMon1 < 1) { numMon1 = 1; }					// lower limit on number of monomers left with rod... 1 monomer
		int numMon2 = numOfMonomers - numMon1;
		double severLoc = numMon1*Env.lengthPerActin;	// define sever location on rod
		removeCofilin(i);								// remove this ADF/cofilin from the list
		aPt.add(end1, severLoc, uVec);					// define sever pt
		Pt3D coord1 = Pt3D.Add(.5, end1, aPt)	;		// define new coord for this filament
		Pt3D coord2 = Pt3D.Add(.5, aPt, end2);			// define coord for rod2
		A_Rod rod2 = A_Rod.makeNewRod(coord2, this.uVec, numMon2);
		// now define the number of different actin types for each rod
		if (numMon1 < numADPMonomers) {
			rod2.numADPMonomers = numADPMonomers - numMon1;
			rod2.numADPPiMonomers = numADPPiMonomers;
		} else {
			rod2.numADPMonomers = 0;
			if (numMon1 < numADPMonomers + numADPPiMonomers) {
				rod2.numADPPiMonomers = numADPMonomers + numADPPiMonomers - numMon1;
			} else {
				numADPPiMonomers = 0;
			}
		}
		numOfMonomers = numMon1;	
		numADPMonomers = numOfMonomers - 1;	// subtract 1 to account for rapid nucleotide exchange on plus end
		numADPPiMonomers = 0;
		
		// take care of branches on both sides of the filament sever location
		//  transfer ARP2/3 past sever point to rod2
		int j = 0;
		while (j < arpCount ) {
			if (arps[j].loc > severLoc) {	
				if (arps[j].active) { 	// if active then remove branch from this structure
					myStructure.removeSubBranch(arps[j].branch);
					rod2.branches++; 	// increment branches if active ARP2/3 bound
					branches+= -1;	// decrement this filaments branches
				}
				// add arp to rod2
				rod2.arps[rod2.arpCount] = arps[j];
				rod2.arps[rod2.arpCount].loc = arps[j].loc - severLoc;		// adjust binding location due to severing
				rod2.arpCount++;		
				// remove same arp from this rod
				arpCount += -1;
				arps[j] = arps[arpCount];
				arps[arpCount] = null;
				j += -1;		// cancel upcoming increment
			}
			j ++;
		}
		// next transfer ADF past sever point to rod2
		j = 0;
		while (j < adfCount ) {
			if (adfLoc[j] > severLoc) {	
				// add adf to rod2
				rod2.adfLoc[rod2.adfCount] = adfLoc[j] - severLoc;
				rod2.adfCount++;
				// remove same adf from this rod
				adfCount += -1;
				adfLoc[j] = adfLoc[adfCount];
				j += -1;		// cancel upcoming increment
			}
			j ++;
		}
		
		// initialize and calculate properties for this filament and rod2
		coord = coord1;	// reset coordinates for this filament to center of severed filament
		calculateProperties();
		initialize();
		if (structureMember) { myStructure.calculateProperties(); }
		rod2.calculateProperties();
		rod2.initialize();
	
		// create a new actin branch for rod2 if necessary...
		if (rod2.branches > 0) {
			ActinBranch newBranch = ActinBranch.makeNewActinBranch();
			newBranch.addSubBranch(rod2);
			newBranch.calculateProperties();
		}
		
		// give the new rod the same number of adhesions as the old rod had....
		rod2.adhesionCount = adhesionCount;

		// for debugging... REMOVE
		justSevered = true;
		rodFromSever = rod2;
		severLocation = severLoc;
	}
	
	public void removeCofilin (int i) {
		adfCount += -1;	// one less ADF/Cofilin on this filament
		aPt.add(end1, adfLoc[i], uVec);		// define the point where this ADF exists
		//impP.seedADF_ADP_ACTIN (aPt, 1);	// add an ADF-ADP-actin into implicit pool
		if (adfCount != 0) {
			adfLoc[i] = adfLoc[adfCount];	// fill in array gap 
		}
	}
	
	public void checkRemove () {
		if ((numOfMonomers < 2) & (! minusCapped) & (!linked)) {	// if only a monomer and not stabilized by a minus end cap and not ActA bound
			remove = true;	// mark this rod for removal from player list
			double lifeTime = Env.simulationTime - birthTime;
			filLife.addValue(lifeTime);
			//impP.seedActin_ADP(coord, numOfMonomers);	// add these monomers back into the implicit pool
			if (branches > 0) {
				for (int i = 0; i < arpCount; i ++) { removeArp(i); }
			}
		}
	}
	
	private void checkAdhesions (double dT) {
		if (Env.applyAdhesions) {
			double adProb = length*Env.adhesionRate*dT;
			if (Rdm.uRdm(this) < adProb) { adhesionCount ++; }
		}
	}
	
	public void viscousBlobSim (double effectiveLength, double dT) {
		// simulate addition of blobs
		double blobAddProb = effectiveLength*Env.vBlobOnRate*dT;
		if (Rdm.uRdm(this) < blobAddProb) { numViscBlobs++; }
		
		// simulate detaching blobs
		double blobRemoveProb = Env.vBlobOffRate*dT;
		if (Rdm.uRdm(this) < blobRemoveProb) { 
			numViscBlobs--; 
			if (numViscBlobs < 0) { numViscBlobs = 0; }
		}
	}
	
	/*private void checkFixing () {
		if (! fixedInSpace) {
			if (age > Env.ageToFix) { fixedInSpace = true; return; }
			if (numOfMonomers > 40) { fixedInSpace = true; return; }
			//if (adhesionCount > 1) { fixedInSpace = true; return; }
			if (bTransGam.y > Env.dragToFix) { fixedInSpace = true; return; }
			if (Env.fixAllActin) { fixedInSpace = true; return;}
		}
	}*/
	
	private void checkFixing () {
		if ((! fixedInSpace) & (Env.fixFilaments)) {
			if ((age > Env.ageToFix) & (Env.fixByAge)) { fixedInSpace = true; return; }
			if ((numOfMonomers > Env.monCtToFix) & (Env.fixByMonCt)) { fixedInSpace = true; return; }
			//if (adhesionCount > 1) { fixedInSpace = true; return; }
			if (bTransGam.y > Env.dragToFix) { fixedInSpace = true; return; }
			if (Env.fixAllActin) { fixedInSpace = true; return;}
		}
	}
			
	public Pt3D crossLinkerForces () {
		Pt3D crossForces = new Pt3D();
		Pt3D linkPt1 = new Pt3D();
		Pt3D linkPt2 = new Pt3D();
		double linkerG, thisG;
		Pt3D forceComp = new Pt3D();
		double forceMag = 0;
		A_Rod curL;
		thisG = getEffectiveDrag();
		for (int i=0; i<linkerCount; i++) {
			curL = linkers[i];
			linkerG = curL.getEffectiveDrag();
			// find the {X,Y,Z} of the link locations
			linkPt1.add(end1, myLinkDist[i], uVec);
			linkPt2.add(curL.end1, linkLinkDist[i], curL.uVec);
			// calculate the forces based on link location differences
			forceComp.sub(1.0e-6/((1/linkerG + 1/thisG)*Env.deltaT*3), linkPt2,linkPt1);
			forceMag = Pt3D.vecMag(forceComp);
			crossForces.inc(forceComp);
		}
		return crossForces;
	}		
	
	public static int countArpsInRegion(){
		int arpCt = 0;
		arpCt ++;
		return arpCt;
		/*for (int i=0; i < Player.playerCt; i++) {
			if(!(Player.thePlayers[i] instanceof A_Rod)){}else {
				A_Rod rod = (A_Rod) Player.thePlayers[i];
				arpCt += rod.arpCount;
			}
		}
		return arpCt;*/
	}
	
	public static double aveLengthBetweenBranches (){
		int branchDistCt = 0;
		double branchDistSum = 0;
		for (int i=0; i < Player.playerCt; i++) {
			if(Player.thePlayers[i] instanceof A_Rod){
				A_Rod rod = (A_Rod) Player.thePlayers[i];
				for (int j=0;j<rod.arpCount;j++) {
					if (j+1 < rod.arpCount) { 
						branchDistSum += rod.arps[j+1].loc - rod.arps[j].loc;
						branchDistCt ++;
					}
				}
			}
		}
		if (branchDistCt > 0) {
			return branchDistSum/branchDistCt;
		} else {
			return 0;
		}
	}
	
	public static void countArpAndActMonsInRegion(){
		//to count Arps, actin monomers and ratio in region behind bug
		Env.monCt= 0;
		Env.arpCt = 0;
		Env.arpBranchCt =0;
		double lengthInRegion; //= 0;
		double aLengthOutOfRegion, bLengthOutOfRegion;
		double end1ToEnd2VertDist, end1ToFrontVertDist, end1ToBackVertDist, end2ToFrontVertDist, end2ToBackVertDist;
		Pt3D end1inListx = new Pt3D();
		Pt3D end2inListx = new Pt3D();
		for (int i=0; i < Player.playerCt; i++) {
			if(Player.thePlayers[i] instanceof A_Rod){
				A_Rod rod = (A_Rod) Player.thePlayers[i];
				end1inListx = Player.listeria.tox(rod.end1);
				end2inListx = Player.listeria.tox(rod.end2);
				//end2inListx.XTox(listeria, rod.end2);
				//end2inListx.XTox(listeria, Pt3D.sub(rod.end2 - listeria.rec1));
				//talk("end1.x = " + String.valueOf(end1inListx.x) + ", end1.y = " + String.valueOf(end1inListx.y) + ", end1.z = " + String.valueOf(end1inListx.z));
				// if both ends are outside of the region, do nothing
				if((end1inListx.x< Env.backBorder) & (end2inListx.x < Env.backBorder)){}
				if((end1inListx.x> Env.frontBorder) & (end2inListx.x > Env.frontBorder)){}
				//some values needed to determine amount of filament in region
				end1ToEnd2VertDist = Math.abs(end1inListx.x - end2inListx.x);
				end1ToFrontVertDist = Math.abs(end1inListx.x - Env.frontBorder);
				end1ToBackVertDist = Math.abs(end1inListx.x - Env.backBorder);
				end2ToFrontVertDist = Math.abs(end2inListx.x - Env.frontBorder);
				end2ToBackVertDist = Math.abs(end2inListx.x - Env.backBorder);
				//talk ("rodlength " + String.valueOf(rod.length));
				
				// if end1(pointed end) inside region...
				if((end1inListx.x <= Env.frontBorder) & (end1inListx.x >= Env.backBorder)){
					if (rod.minusCapped){Env.arpCt ++;} //if end1capped (ie at minus end) then count as arp
					if (rod.aBranch){Env.arpBranchCt ++;}	// if rod with end1 is a branch then count as a branched(active) arp
					//talk("am in region");
					
					if((end2inListx.x <= Env.frontBorder) & (end2inListx.x >= Env.backBorder)){//and end2 inside region
						lengthInRegion = rod.length;
						Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end2inListx.x < Env.backBorder){ //and end2 behind region
						lengthInRegion = (end1ToBackVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end2inListx.x > Env.frontBorder){ //and end2 in front of region
						lengthInRegion = (end1ToFrontVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
				}
				// if end2(plus end) inside region...
				if((end2inListx.x <= Env.frontBorder) & (end2inListx.x >= Env.backBorder)){
					if(end1inListx.x < Env.backBorder){ //and end1 behind region
						lengthInRegion = (end2ToBackVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end1inListx.x > Env.frontBorder){ //and end1 in front of region
						lengthInRegion = (end2ToFrontVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
				}
				// if end1 and end 2 on either side of region
				if ((end1inListx.x > Env.frontBorder) & (end2inListx.x < Env.backBorder)){
					aLengthOutOfRegion = (end1ToFrontVertDist*rod.length)/end1ToEnd2VertDist;
					bLengthOutOfRegion = (end2ToBackVertDist*rod.length)/end1ToEnd2VertDist;
					lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
					Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				if ((end1inListx.x < Env.backBorder) & (end2inListx.x > Env.frontBorder)){
					aLengthOutOfRegion = (end1ToBackVertDist*rod.length)/end1ToEnd2VertDist;
					bLengthOutOfRegion = (end2ToFrontVertDist*rod.length)/end1ToEnd2VertDist;
					lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
					Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
			}
			//Env.monCt += (int) Math.round(lengthInRegion/Env.lengthPerActin);
		}
		//talk ("monCt " + String.valueOf(Env.monCt));
		if (Env.monCt == 0){ Env.arpBranchActMonRatio = -0.01;} else{
			Env.arpBranchActMonRatio = ((double)Env.arpBranchCt)/((double)Env.monCt);
		}
		
	}
	
	public static void countRegionArpActin(){
		//to count Arps, actin monomers and ratio in regions A to D along bug
		Env.monCtInA= 0;
		Env.monCtInB= 0;
		Env.monCtInC= 0;
		Env.monCtInD= 0;
		Env.arpBranchCtInA =0;
		Env.arpBranchCtInB =0;
		Env.arpBranchCtInC =0;
		Env.arpBranchCtInD =0;
		double lengthInRegion; //= 0;
		double aLengthOutOfRegion, bLengthOutOfRegion;
		double end1ToEnd2VertDist; 
		double end1ToAFrontVertDist, end1ToABackVertDist, end2ToAFrontVertDist, end2ToABackVertDist;
		double end1ToBFrontVertDist, end1ToBBackVertDist, end2ToBFrontVertDist, end2ToBBackVertDist;
		double end1ToCFrontVertDist, end1ToCBackVertDist, end2ToCFrontVertDist, end2ToCBackVertDist;
		double end1ToDFrontVertDist, end1ToDBackVertDist, end2ToDFrontVertDist, end2ToDBackVertDist;
		//xcoords of regionsA to D front and backs (back is smaller x)
		double xABack = -1.0 * Env.listeriaCapRad;
		double xAFront =  (Env.listeriaLength/4.0) - Env.listeriaCapRad;
		double xBBack =  (Env.listeriaLength/4.0) - Env.listeriaCapRad;
		double xBFront = (Env.listeriaLength/2.0) - Env.listeriaCapRad;
		double xCBack = (Env.listeriaLength/2.0) - Env.listeriaCapRad;
		double xCFront = ((3.0 * Env.listeriaLength)/4.0) - Env.listeriaCapRad;
		double xDFront = ((3.0 * Env.listeriaLength)/4.0) - Env.listeriaCapRad;
		double xDBack = Env.listeriaLength - Env.listeriaCapRad;
		Pt3D end1inListx = new Pt3D();
		Pt3D end2inListx = new Pt3D();
		for (int i=0; i < Player.playerCt; i++) {
			if(Player.thePlayers[i] instanceof A_Rod){
				A_Rod rod = (A_Rod) Player.thePlayers[i];
				end1inListx = Player.listeria.tox(rod.end1);
				end2inListx = Player.listeria.tox(rod.end2);
				//some values needed to determine amount of filament in each region A to D
				end1ToEnd2VertDist = Math.abs(end1inListx.x - end2inListx.x);
				end1ToAFrontVertDist = Math.abs(end1inListx.x - xAFront);
				end1ToABackVertDist = Math.abs(end1inListx.x - xABack);
				end2ToAFrontVertDist = Math.abs(end2inListx.x - xAFront);
				end2ToABackVertDist = Math.abs(end2inListx.x - xABack);
				end1ToBFrontVertDist = Math.abs(end1inListx.x - xBFront);
				end1ToBBackVertDist = Math.abs(end1inListx.x - xBBack);
				end2ToBFrontVertDist = Math.abs(end2inListx.x - xBFront);
				end2ToBBackVertDist = Math.abs(end2inListx.x - xBBack);
				end1ToCFrontVertDist = Math.abs(end1inListx.x - xCFront);
				end1ToCBackVertDist = Math.abs(end1inListx.x - xCBack);
				end2ToCFrontVertDist = Math.abs(end2inListx.x - xCFront);
				end2ToCBackVertDist = Math.abs(end2inListx.x - xCBack);
				end1ToDFrontVertDist = Math.abs(end1inListx.x - xDFront);
				end1ToDBackVertDist = Math.abs(end1inListx.x - xDBack);
				end2ToDFrontVertDist = Math.abs(end2inListx.x - xDFront);
				end2ToDBackVertDist = Math.abs(end2inListx.x - xDBack);
				//talk ("rodlength " + String.valueOf(rod.length));
				
				// if both ends are outside of the regions A to D, do nothing
				if((end1inListx.x< xABack) & (end2inListx.x < xABack)){}
				if((end1inListx.x> xDFront) & (end2inListx.x > xDFront)){}
				//********RegionA*********
				// if end1(pointed end) inside regionA...
				if((end1inListx.x < xAFront) & (end1inListx.x >= xABack)){
					if (rod.aBranch){Env.arpBranchCtInA ++;}	// if rod with end1 is a branch then count as a branched(active) arp
					//talk("am in region");
					
					if((end2inListx.x < xAFront) & (end2inListx.x >= xABack)){//and end2 inside region
						lengthInRegion = rod.length;
						Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end2inListx.x < xABack){ //and end2 behind region
						lengthInRegion = (end1ToABackVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end2inListx.x >= xAFront){ //and end2 in front of region
						lengthInRegion = (end1ToAFrontVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
				}
				// if end2(plus end) inside region...and end1 isn't
				if((end2inListx.x < xAFront) & (end2inListx.x >= xABack)){
					if(end1inListx.x < xABack){ //and end1 behind region
						lengthInRegion = (end2ToABackVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end1inListx.x >= xAFront){ //and end1 in front of region
						lengthInRegion = (end2ToAFrontVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
				}
				// if end1 and end 2 on either side of region
				if ((end1inListx.x >= xAFront) & (end2inListx.x < xABack)){
					aLengthOutOfRegion = (end1ToAFrontVertDist*rod.length)/end1ToEnd2VertDist;
					bLengthOutOfRegion = (end2ToABackVertDist*rod.length)/end1ToEnd2VertDist;
					lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
					Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				if ((end1inListx.x < xABack) & (end2inListx.x >= xAFront)){
					aLengthOutOfRegion = (end1ToABackVertDist*rod.length)/end1ToEnd2VertDist;
					bLengthOutOfRegion = (end2ToAFrontVertDist*rod.length)/end1ToEnd2VertDist;
					lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
					Env.monCtInA += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				//********RegionB*********
				// if end1(pointed end) inside regionB...
				if((end1inListx.x < xBFront) & (end1inListx.x >= xBBack)){
					if (rod.aBranch){Env.arpBranchCtInB ++;}	// if rod with end1 is a branch then count as a branched(active) arp
					//talk("am in region");
					
					if((end2inListx.x < xBFront) & (end2inListx.x >= xBBack)){//and end2 inside region
						lengthInRegion = rod.length;
						Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end2inListx.x < xBBack){ //and end2 behind region
						lengthInRegion = (end1ToBBackVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end2inListx.x >= xBFront){ //and end2 in front of region
						lengthInRegion = (end1ToBFrontVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
				}
				// if end2(plus end) inside region...and end1 isn't
				if((end2inListx.x < xBFront) & (end2inListx.x >= xBBack)){
					if(end1inListx.x < xBBack){ //and end1 behind region
						lengthInRegion = (end2ToBBackVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
					if(end1inListx.x >= xBFront){ //and end1 in front of region
						lengthInRegion = (end2ToBFrontVertDist*rod.length)/end1ToEnd2VertDist;
						Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
					}
				}
				// if end1 and end 2 on either side of region
				if ((end1inListx.x >= xBFront) & (end2inListx.x < xBBack)){
					aLengthOutOfRegion = (end1ToBFrontVertDist*rod.length)/end1ToEnd2VertDist;
					bLengthOutOfRegion = (end2ToBBackVertDist*rod.length)/end1ToEnd2VertDist;
					lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
					Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				if ((end1inListx.x < xBBack) & (end2inListx.x >= xBFront)){
					aLengthOutOfRegion = (end1ToBBackVertDist*rod.length)/end1ToEnd2VertDist;
					bLengthOutOfRegion = (end2ToBFrontVertDist*rod.length)/end1ToEnd2VertDist;
					lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
					Env.monCtInB += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}

			//********RegionC*********
			// if end1(pointed end) inside regionC...
			if((end1inListx.x < xCFront) & (end1inListx.x >= xCBack)){
				if (rod.aBranch){Env.arpBranchCtInC ++;}	// if rod with end1 is a branch then count as a branched(active) arp
				//talk("am in region");
				
				if((end2inListx.x < xCFront) & (end2inListx.x >= xCBack)){//and end2 inside region
					lengthInRegion = rod.length;
					Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				if(end2inListx.x < xCBack){ //and end2 behind region
					lengthInRegion = (end1ToCBackVertDist*rod.length)/end1ToEnd2VertDist;
					Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				if(end2inListx.x >= xCFront){ //and end2 in front of region
					lengthInRegion = (end1ToCFrontVertDist*rod.length)/end1ToEnd2VertDist;
					Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
			}
			// if end2(plus end) inside region...and end1 isn't
			if((end2inListx.x < xCFront) & (end2inListx.x >= xCBack)){
				if(end1inListx.x < xCBack){ //and end1 behind region
					lengthInRegion = (end2ToCBackVertDist*rod.length)/end1ToEnd2VertDist;
					Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
				if(end1inListx.x >= xCFront){ //and end1 in front of region
					lengthInRegion = (end2ToCFrontVertDist*rod.length)/end1ToEnd2VertDist;
					Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
				}
			}
			// if end1 and end 2 on either side of region
			if ((end1inListx.x >= xCFront) & (end2inListx.x < xCBack)){
				aLengthOutOfRegion = (end1ToCFrontVertDist*rod.length)/end1ToEnd2VertDist;
				bLengthOutOfRegion = (end2ToCBackVertDist*rod.length)/end1ToEnd2VertDist;
				lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
				Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}
			if ((end1inListx.x < xCBack) & (end2inListx.x >= xCFront)){
				aLengthOutOfRegion = (end1ToCBackVertDist*rod.length)/end1ToEnd2VertDist;
				bLengthOutOfRegion = (end2ToCFrontVertDist*rod.length)/end1ToEnd2VertDist;
				lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
				Env.monCtInC += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}

//		********RegionD*********
		// if end1(pointed end) inside regionD...
		if((end1inListx.x < xDFront) & (end1inListx.x >= xDBack)){
			if (rod.aBranch){Env.arpBranchCtInD ++;}	// if rod with end1 is a branch then count as a branched(active) arp
			//talk("am in region");
			
			if((end2inListx.x < xDFront) & (end2inListx.x >= xDBack)){//and end2 inside region
				lengthInRegion = rod.length;
				Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}
			if(end2inListx.x < xDBack){ //and end2 behind region
				lengthInRegion = (end1ToDBackVertDist*rod.length)/end1ToEnd2VertDist;
				Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}
			if(end2inListx.x >= xDFront){ //and end2 in front of region
				lengthInRegion = (end1ToDFrontVertDist*rod.length)/end1ToEnd2VertDist;
				Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}
		}
		// if end2(plus end) inside region...and end1 isn't
		if((end2inListx.x < xDFront) & (end2inListx.x >= xDBack)){
			if(end1inListx.x < xDBack){ //and end1 behind region
				lengthInRegion = (end2ToDBackVertDist*rod.length)/end1ToEnd2VertDist;
				Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}
			if(end1inListx.x >= xDFront){ //and end1 in front of region
				lengthInRegion = (end2ToDFrontVertDist*rod.length)/end1ToEnd2VertDist;
				Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
			}
		}
		// if end1 and end 2 on either side of region
		if ((end1inListx.x >= xDFront) & (end2inListx.x < xDBack)){
			aLengthOutOfRegion = (end1ToDFrontVertDist*rod.length)/end1ToEnd2VertDist;
			bLengthOutOfRegion = (end2ToDBackVertDist*rod.length)/end1ToEnd2VertDist;
			lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
			Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
		}
		if ((end1inListx.x < xDBack) & (end2inListx.x >= xDFront)){
			aLengthOutOfRegion = (end1ToDBackVertDist*rod.length)/end1ToEnd2VertDist;
			bLengthOutOfRegion = (end2ToDFrontVertDist*rod.length)/end1ToEnd2VertDist;
			lengthInRegion = rod.length - (aLengthOutOfRegion + bLengthOutOfRegion);
			Env.monCtInD += (int) Math.round(lengthInRegion/Env.lengthPerActin);
		}
	}
}	
		//talk ("monCt " + String.valueOf(Env.monCt));
		if (Env.monCtInA == 0){ Env.arpBranchRatioInA = -0.01;} else{
			Env.arpBranchRatioInA = ((double)Env.arpBranchCtInA)/((double)Env.monCtInA);
		}
		if (Env.monCtInB == 0){ Env.arpBranchRatioInB = -0.01;} else{
			Env.arpBranchRatioInB = ((double)Env.arpBranchCtInB)/((double)Env.monCtInB);
		}
		if (Env.monCtInC == 0){ Env.arpBranchRatioInC = -0.01;} else{
			Env.arpBranchRatioInC = ((double)Env.arpBranchCtInC)/((double)Env.monCtInC);
		}
		if (Env.monCtInD == 0){ Env.arpBranchRatioInD = -0.01;} else{
			Env.arpBranchRatioInD = ((double)Env.arpBranchCtInD)/((double)Env.monCtInD);
		}

	}
	
	public Pt3D crossLinkerTorques () {
		Pt3D crossTorques = new Pt3D();
		A_Rod curL;
		Pt3D vecO = new Pt3D();
		Pt3D diffVec = new Pt3D();
		for (int i=0; i < linkerCount; i++) {
			curL = linkers[i];
			// find the unit vector in current position and compare to stored unit vector defining link orientation
			vecO.sub(Pt3D.XToNewx(this, curL.uVec), xUnitVector);		// orientation vector in rod's frame
			diffVec.sub(linkOrientation[i], vecO);
		}
		return crossTorques;
	}
	
	public void checkListeriaCollision (Listeria LM) { 
		// *******  ONE COLLISION ALLOWED FOR EACH END1 AND END2... MODIFIED TO CHECK POSITIONS OF ROD RELATIVE TO LISTERIA
		// There are several distinct types of collisions that could occur... assume 
		// that more than one collisions can occur in any given time step.
		// Treat the listeria as a thick rod and two spheres to detect collisions with rods.
		//*********************
		
		// check collisions of the rod with listeria spherical caps... four possibilities.  Of these four only a rod.end2 collision
		// with Listeria.end1 has the possibility of an ActA binding event, forced uncapping, or whatever
		// This first possibility is most important... interesting things happen for this case: uncapping, tethering, etc
		// Rod.end2 with spherical cap of Listeria at end1
		
		// ** fast checks to see if rod ends are out of simple bounding shape **
		if ((end1InLMFrame.x < -Listeria.halfLength) & (end2InLMFrame.x < -Listeria.halfLength)) { return; }  // behind bug
		if ((end1InLMFrame.x > Listeria.halfLength) & (end2InLMFrame.x > Listeria.halfLength)) { return; }  // before bug
		
		double end1OffCylAxisSqrd = end1InLMFrame.y*end1InLMFrame.y + end1InLMFrame.z*end1InLMFrame.z;
		double end2OffCylAxisSqrd = end2InLMFrame.y*end2InLMFrame.y + end2InLMFrame.z*end2InLMFrame.z;
		if ((end1OffCylAxisSqrd > Listeria.capRadSqrd) & (end2OffCylAxisSqrd > Listeria.capRadSqrd)) { return; }  // outside infinite cylinder
		// ** end fast checks **
		
		double conDist;
		boolean end2Collided = false;
		boolean end1Collided = false;
		
		// Possible collisions
		// *** END2 ***
		if (!end2Collided & (end2InLMFrame.x > -Listeria.halfLength) & (end2InLMFrame.x < -Listeria.halfCylLength)) {  // possible listeria end1 collision
			// rod.end2 with spherical cap of Listeria at end1
			conDist = Pt3D.ptDist(end2, LM.rec1); 
			if (conDist < Listeria.capRad + Env.actATetherLength) {	// if rod.end2 is close enough then go to method to check tethering
				LM.checkRodTether(this, end2);
				if (conDist < Listeria.capRad) { 
					ForceResolver.newRodListeriaCollision(this, LM, Pt3D.zeroPt, Pt3D.zeroPt, conDist, 21); 
					if ((Env.collisionForcesCapOff) & (plusCapped)) { plusCapOff (); }
					end2Collided = true;
				}
			}
		}
		
		if (!end2Collided & (end2InLMFrame.x > -Listeria.halfCylLength) & (end2InLMFrame.x < Listeria.halfCylLength)) {  // possible listeria cylinder collision
			// Rod.end2 with cylinder sides
			pointAndLineIntersectTest (end2, LM.rec1, LM.rec2, retObj);
			//retObj = this.pointAndLMCylSlowWay(rod.end2,LM);
			if ((retObj.collision) & (retObj.conDist < Listeria.cylDiam/2 + Env.actATetherLength)) {
				LM.checkRodTether(this, end2);
				if (retObj.conDist < Listeria.cylDiam/2) { 
					ForceResolver.newRodListeriaCollision(this, LM, retObj.conPt1, retObj.conPt2, retObj.conDist, 20);
					if ((Env.collisionForcesCapOff) & (plusCapped)) { plusCapOff (); }
					end2Collided = true;
				}
			}
		} 
		
		if (!end2Collided & (end2InLMFrame.x > Listeria.halfCylLength) & (end2InLMFrame.x < Listeria.halfLength)) {  // possible listeria end2 collision
			// rod.end2 with spherical cap of Listeria at end2
			conDist = Pt3D.ptDist(end2, LM.rec2);
			if (conDist < Listeria.capRad + Env.actATetherLength) {	// if rod.end2 is close enough then go to method to check tethering
				LM.checkRodTether(this, end2);
				if (conDist < Listeria.capRad) { 
					ForceResolver.newRodListeriaCollision(this, LM, Pt3D.zeroPt, Pt3D.zeroPt, conDist, 22); 
					if ((Env.collisionForcesCapOff) & (plusCapped)) { plusCapOff (); }
					end2Collided = true;
				}
			}
		}
		
		
		// *** END1 ***
		if ((end1InLMFrame.x > -Listeria.halfLength) & (end1InLMFrame.x < -Listeria.halfCylLength)) {  // possible listeria end1 collision
			// rod.end1 with spherical cap of Listeria at end1
			conDist = Pt3D.ptDist( end1, LM.rec1);
			if (conDist < Listeria.capRad) { 
				ForceResolver.newRodListeriaCollision(this, LM, Pt3D.zeroPt, Pt3D.zeroPt, conDist, 11);
				end1Collided = true;	
			}
		}
		
		if (!end1Collided & (end1InLMFrame.x > -Listeria.halfCylLength) & (end1InLMFrame.x < Listeria.halfCylLength)) {  // possible listeria cylinder collision
			// rod.end1 with cylinder sides
			pointAndLineIntersectTest (end1, LM.rec1, LM.rec2, retObj);
			//retObj = this.pointAndLMCylSlowWay(rod.end1,LM);
			if ((retObj.collision) & (retObj.conDist < Listeria.cylDiam/2)) { 
				ForceResolver.newRodListeriaCollision(this, LM, retObj.conPt1, retObj.conPt2, retObj.conDist, 10); 
				end1Collided = true;	
			}
		} 
		
		if (!end1Collided & (end1InLMFrame.x > Listeria.halfCylLength) & (end1InLMFrame.x < Listeria.halfLength)) {  // possible listeria end2 collision
			// rod.end1 with spherical cap of Listeria at end2
			conDist = Pt3D.ptDist(end1, LM.rec2);
			if (conDist < Listeria.capRad) { 
				ForceResolver.newRodListeriaCollision(this, LM, Pt3D.zeroPt, Pt3D.zeroPt, conDist, 12); 
				end1Collided = true;	
			}
		}

	
		// ** CHECK FOR ROD SIDES WITH PARTS OF BUG, BUT ONLY IF NO OTHER COLLISIONS YET **
		if (!end1Collided & !end2Collided) {
			// Side of Rod with cylinder sides
			lineSegmentIntersectTest (end1,end2, LM.rec1, LM.rec2, retObj);
			if ((retObj.collision) & (retObj.conDist < Listeria.cylDiam/2 + radius)) {	// can be skew if close enough
				ForceResolver.newRodListeriaCollision(this, LM, retObj.conPt1, retObj.conPt2, retObj.conDist, 0);
				return;			
			}
			
			// this next part checks a side collision of a rod with one of the spherical cap ends of listeria
			pointAndLineIntersectTest (LM.rec1,end1, end2, retObj);
			if ((retObj.collision) & (retObj.conDist < Listeria.capRad + radius)) {
				ForceResolver.newRodListeriaCollision(this, LM, retObj.conPt1, retObj.conPt2, retObj.conDist, 1); 
				return;		
			} else {
				pointAndLineIntersectTest (LM.rec2,end1, end2, retObj);
				if ((retObj.collision) & (retObj.conDist < Listeria.capRad + radius)) {
					ForceResolver.newRodListeriaCollision(this, LM, retObj.conPt1, retObj.conPt2, retObj.conDist, 2); 
					return;		
				}
			}
		}
	} 
			
	
	public void reportStats () {
		talkln ("Rod: Id #" + String.valueOf(idnum));
		talkln ("	Coords = " + coord.reportCoords() + ", Angles = " + angle.reportCoords());
		talkln ("	plusCapped = " + String.valueOf(plusCapped));
		talkln ("	minusCapped = " + String.valueOf(minusCapped));
		talkln ("	aBranch = " + String.valueOf(aBranch));
		talkln ("	number of Branches = " + String.valueOf(branches));
		talkln ("	my structure is " + String.valueOf(myStructure));
		if (myStructure != null) { 
			talkln ("		fixedInSpace for the branch is " + String.valueOf(myStructure.fixedInSpace)); 
			talkln ("		numMems = " + String.valueOf(myStructure.numMems) + ", oldNumMems = " + String.valueOf(myStructure.oldNumMems));
			talkln ("		forceSum = " + myStructure.forceSum.reportCoords());
			talkln ("		torqueSum = " + myStructure.torqueSum.reportCoords());
		} 
		talkln ("	fixedInSpace for this rod is " + String.valueOf(fixedInSpace));
		talkln ("	Total actin = " + String.valueOf(numOfMonomers) + "  ADP-Pi = " + String.valueOf(numADPPiMonomers) + "  ADP = " + String.valueOf(numADPMonomers));
		talkln ("	end1 = " + end1.reportCoords() + "    end2 = " + end2.reportCoords());
		talkln ("	linkerCt = " + String.valueOf(linkerCt));
		//talkln (" uVec = " + uVec.reportCoords());
		/*talk (" Hi!  I'm a rod, id# " + String.valueOf(idnum));
		talk ("	age = " + String.valueOf(age));
		talk ("	catastrophe = " + String.valueOf(catastrophe));
		talk (" 	subsumed = " + String.valueOf(subsumed));
		talk ("	linkerCount = " + String.valueOf(linkerCount));
		talk ("	thermal forces last time step = " + String.valueOf(Util.vecMag(randForce)));
		talk ("	crosslinkforce last time step = " + String.valueOf(Util.vecMag(crossLinkForces))); */
	}
	
	public void showPtToPt (Graphics g, double scale, PinCam cam, Pt3D ptA, Pt3D ptB) {
		if (Env.usePinCam) {
			pixA = cam.ptToPix(ptA, scale);
			pixB = cam.ptToPix(ptB, scale);
		} else {
			pixA.ptToPix(ptA, scale, cam.offset, cam.e);
			pixB.ptToPix(ptB, scale, cam.offset, cam.e);
		}
		if ((pixA.show) | (pixB.show)) { Pixel.drawLine(g,pixA,pixB); }
	}
	
	public void showHelixMonomers (Graphics g, double scale, PinCam cam, int monomers) {
		double R = Env.helicalBlobOffset;
		double dth = Env.helixAngleInc;
		double dx = Env.helixXInc;
		Pt3D bLinePt = new Pt3D ();
		int blobSize = (int)(Env.helixBlobDiam*scale);
		int halfBlobSize = (int)(blobSize/2);
		Color curColor = g.getColor();
		for (int i = 0; i < monomers; i++) {
			bLinePt.add(end1, bMonLoc, uVec);
			dR.add(R*Math.cos(thMon), t1Vec, R*Math.sin(thMon), t2Vec);
			ptA.add(bLinePt, dR);
			if (Env.usePinCam) {
				pixA = cam.ptToPix(ptA, scale);
			} else {
				pixA.ptToPix(ptA, scale, cam.offset, cam.e);
			}
			g.fillOval(pixA.x - halfBlobSize, pixA.y - halfBlobSize, blobSize, blobSize);
			g.setColor(Env.actinOutlineColor);
			g.drawOval(pixA.x - halfBlobSize, pixA.y - halfBlobSize, blobSize, blobSize);
			g.setColor(curColor);
			thMon += dth;
			bMonLoc += dx;
		} 
	}
	
	public void showHelixLines (Graphics g, double scale, PinCam cam, int monomers) {
		double R = Env.fActinDiameter/2.0;
		double dth = Env.helixAngleInc;
		double dx = Env.helixXInc;
		Pt3D bLinePt = new Pt3D ();
		for (int i = 0; i < monomers; i++) {
			bLinePt.add(end1, bLineLoc, uVec);
			dR.add(R*Math.cos(thLine), t1Vec, R*Math.sin(thLine), t2Vec);
			if (oddMon) {
				ptA.add(bLinePt, dR);
				pixA.ptToPix(ptA, scale, cam.offset, cam.e);
				if (bLineLoc > dx) { Pixel.drawLine (g, pixAp, pixA); }
				pixAp.copy(pixA);
				oddMon = false;
			} else {
				ptB.add(bLinePt, dR);
				pixB.ptToPix(ptB, scale, cam.offset, cam.e);
				if (bLineLoc > dx) { Pixel.drawLine (g, pixBp, pixB); }
				pixBp.copy(pixB);
				oddMon = true;
			}
			thLine += dth;
			bLineLoc += dx;
		}		
	}
	
	public void showHelix (Graphics g, double scale, PinCam cam, int monomers) {
		if (Env.drawHelicalLines) { showHelixLines (g, scale, cam, monomers); }
		if (Env.drawEachMonomer) { showHelixMonomers (g, scale, cam, monomers); }
	}
	
	public void showYourself (Graphics g, double scale, PinCam cam) {
		thMon = helixAng;		// set initial value for helical angles
		thLine = helixAng;
		bMonLoc = 0;			// reset filament location to zero
		bLineLoc = 0;			
		oddMon = true;		// set odd monomer to true
		if ((Env.drawActinHelix) & (scale > Env.helixScaleCutOff)){
			if (Env.showHydrolysisStages) {	// show ATP, ADP-Pi, and ADP sections of actin
				// *** ADP section
				g.setColor(Env.ADPActinColor);
				showHelix (g, scale, cam, numADPMonomers);
				// *** ADP-Pi section
				g.setColor(Env.ADPPiActinColor);
				showHelix (g, scale, cam, numADPPiMonomers);
				// *** ATP section
				g.setColor(Env.ATPActinColor);
				showHelix (g, scale, cam, numOfMonomers - numADPMonomers - numADPPiMonomers);
			} else {
				g.setColor(Env.ATPActinColor);
				showHelix (g, scale, cam, numOfMonomers);
			}
		} else {
			if (Env.showHydrolysisStages) {	// show ATP, ADP-Pi, and ADP sections of actin
				//*** ADP section
				g.setColor(Env.ADPActinColor);
				Pt3D ADPPt = Pt3D.Add(end1, ADPLength, uVec);
				showPtToPt (g, scale, cam, end1, ADPPt);
				//*** ADP-Pi section
				g.setColor(Env.ADPPiActinColor);
				Pt3D ADPPiPt = Pt3D.Add(ADPPt, ADPPiLength, uVec);
				showPtToPt (g, scale, cam, ADPPt, ADPPiPt);
				//*** ATP section
				g.setColor(Env.ATPActinColor);
				showPtToPt (g, scale, cam, ADPPiPt, end2);
			} else {
				g.setColor(Env.ATPActinColor);
				showPtToPt (g, scale, cam, end1, end2);
			}
		}
		
		/*if (Env.showARPLocations) {	// show ARP2/3 on filament
			int arpSize = (int)(0.005*scale);
			int halfSize = (int)(arpSize/2);
			for (int i = 0; i < arpCount; i++) {
				ptA.add(end1, arps[i].loc, uVec);
				pixA.ptToPix(ptA, scale, cam.offset, cam.e);
				if (arps[i].active) { g.setColor(Env.activeARPColor); } else { g.setColor(Env.inactiveARPColor); }
				g.fillOval(pixA.x-halfSize, pixA.y-halfSize, arpSize, arpSize);
			}
		}*/
		
		if (Env.showARPLocations) { 
			for (int i = 0; i < arpCount; i++) {
				ptA.add(end1, arps[i].loc, uVec);
				arps[i].showYourself(g,scale,cam,ptA);
			}
		}
		
		if (Env.showADFLocations) { // show ADF locations
			int adfSize = (int)(0.003*scale);
			int halfSize = (int)(adfSize/2);
			for (int i = 0; i < adfCount; i++) {
				ptA.add(end1, adfLoc[i], uVec);
				pixA.ptToPix(ptA, scale, cam.offset, cam.e);
				g.setColor(Env.adfColor);
				g.fillOval(pixA.x-halfSize, pixA.y-halfSize, adfSize, adfSize);
				g.setColor(Env.actinOutlineColor);
				g.drawOval(pixA.x-halfSize, pixA.y-halfSize, adfSize, adfSize);
			}
		}
		
		if (plusCapped) {
			g.setColor(Env.plusCapColor);
			int capSize = (int)(0.006*scale);
			int halfSize = (int)(capSize/2);
			pixA.ptToPix(end2, scale, cam.offset, cam.e);
			g.fillOval(pixA.x-halfSize, pixA.y-halfSize, capSize, capSize);
		}
		
		if ((minusCapped) & (!aBranch)) {	
			g.setColor(Env.minusCapColor);
			int capSize = (int)(0.005*scale);
			int halfSize = (int)(capSize/2);
			pixA.ptToPix(end1, scale, cam.offset, cam.e);
			g.fillOval(pixA.x-halfSize, pixA.y-halfSize, capSize, capSize);
		}	
		
		if ((Env.showProbsOnScreen) & (selected)) {
			String sOut;
			// show tip clearance on screen
			g.setColor(Color.cyan);
			pixA.ptToPix(end2, scale, cam.offset, cam.e);
			ptB.add(end2, tipClearance, uVec);
			pixB.ptToPix(ptB, scale, cam.offset, cam.e);
			Pixel.drawLine(g,pixA,pixB);
			
			// display collision status
			if (lastColliderCt > 0) { 
				g.setColor(Color.red);
				sOut = "Collision";
				Player.stringToScreen(g, sOut, 310,395);
			}
			sOut = String.valueOf("Tip Clearance = " + String.valueOf(toScreenFormat.format(tipClearance)));
			Player.stringToScreen(g, sOut, 310,410);
			
			// display polym prob
			if (polymPossible) { g.setColor(Color.cyan); } else { g.setColor(Color.red); }
			sOut = String.valueOf("Polym prob = " + String.valueOf(toScreenFormat.format(modAddProb)));
			Player.stringToScreen(g, sOut, 310,425);
			
			// display cap prob
			if (plusCapPossible) { g.setColor(Color.cyan); } else { g.setColor(Color.red); }
			sOut = String.valueOf("Cap prob = " + String.valueOf(toScreenFormat.format(plusCapProb)));
			Player.stringToScreen(g, sOut, 310,440);
			
			// display arp prob
			if (arpPossible) { g.setColor(Color.cyan); } else { g.setColor(Color.red); }
			sOut = String.valueOf("Arp prob = " + String.valueOf(toScreenFormat.format(arpProb)));
			Player.stringToScreen(g, sOut, 310,455);
		}
		
		if (selected) { // indicate that this rod is the GUI tweezer selected player
			g.setColor(new Color(200,175,255));
			int markSize = (int)(0.02*scale);
			int halfSize = (int)(markSize/2);
			pixA.ptToPix(coord, scale, cam.offset, cam.e);
			g.drawOval(pixA.x-halfSize, pixA.y-halfSize, markSize, markSize);
		}
	}
	
	public void stereoVision (Graphics g, double scale, PinCam camL, PinCam camR) {
	/*	Pixel [][] pixL = new Pixel [2][2];
		Pixel [][] pixR = new Pixel [2][2];
		for (int i = 0; i < 2; i ++) { pixL[i][0] = new Pixel(); pixL[i][1] = new Pixel(); }
		for (int i = 0; i < 2; i ++) { pixR[i][0] = new Pixel(); pixR[i][1] = new Pixel(); }
		
		g.setColor(playerColor);
		for (int i = 0; i <= numLines; i++) {
			dR.xToX(this, rodPaintPts[i]);
			ptA.add(end1, dR);
			ptB.add(end2, dR);
			pixL[0][1] = camL.ptToPix(ptA, scale);
			pixL[1][1] = camL.ptToPix(ptB, scale);
			pixR[0][1] = camR.ptToPix(ptA, scale);
			pixR[1][1] = camR.ptToPix(ptB, scale);
			if ((pixL[0][1].show) | (pixL[1][1].show)) { Pixel.drawLine(g,pixL[0][1],pixL[1][1]); }
			if ((pixR[0][1].show) | (pixR[1][1].show)) { Pixel.drawLine(g,pixR[0][1],pixR[1][1]); }
			if (i != 0) { 
				if ((pixL[0][0].show) | (pixL[0][1].show)) { Pixel.drawLine(g, pixL[0][0], pixL[0][1]); }
				if ((pixL[1][0].show) | (pixL[1][1].show)) { Pixel.drawLine(g, pixL[1][0], pixL[1][1]); }
				if ((pixR[0][0].show) | (pixR[0][1].show)) { Pixel.drawLine(g, pixR[0][0], pixR[0][1]); }
				if ((pixR[1][0].show) | (pixR[1][1].show)) { Pixel.drawLine(g, pixR[1][0], pixR[1][1]); }
			}
			pixL[0][0] = pixL[0][1];
			pixL[1][0] = pixL[1][1];
			pixR[0][0] = pixR[0][1];
			pixR[1][0] = pixR[1][1];
		} */
	}
			
	public void saveYourself (DataOutputStream ds) {
		try {
			ds.writeByte(1);			// the A_Rod identifier
			ds.writeInt(idnum);											// integer #1
			Pt3D.writePt3D(ds, coord);									// Pt3D #1
			Pt3D.writePt3D(ds, uVec);									// Pt3D #2
			ds.writeInt(numOfMonomers);								// integer #2
			ds.writeInt(numADPPiMonomers);								// integer #3
			ds.writeInt(numADPMonomers);								// integer #4
			if (plusCapped) { ds.writeByte(1); } else { ds.writeByte(0); }		
			if (minusCapped) { ds.writeByte(1); } else { ds.writeByte(0); }
			if (aBranch) { ds.writeByte(1); } else { ds.writeByte(0); } 
			if (structureMember) { 	//branchMember info
				ds.writeByte(1); 
				if (thePlayers[myStructure.idnum] != myStructure) { talkln ("actinbranch id screw-up"); }
				ds.writeInt(myStructure.idnum);
				ds.writeInt(structureId);
			} else { ds.writeByte(0); }
			ds.writeFloat((float)helixAng);
			ds.writeInt(arpCount);
			ds.writeInt(branches);
			for (int i = 0; i < arpCount; i++) { A_Arp.writeArp(ds, arps[i]); }
			ds.writeInt(adfCount);
			for (int i = 0; i < adfCount; i++) { ds.writeFloat((float)adfLoc[i]); }	
		} catch (IOException ioe) { talkln ("some sort of error in saving rod info"); }	
	}
	
	public static Player readYourself (DataInputStream ds, RocketBugs MC) {
		try {
			int idnum = ds.readInt();
			Pt3D coord = Pt3D.readPt3D(ds);
			Pt3D uVec = Pt3D.readPt3D(ds);
			int numMons = ds.readInt();
			A_Rod newRod = new A_Rod (coord, uVec, numMons, Env.fActinDiameter);
			//talk ("... made new rod: ");
			if (newRod.idnum != idnum) {
				talk (" rod idnum = " + String.valueOf(newRod.idnum) + " and should be " + String.valueOf(idnum));
			} 
			//newRod.idnum = (short)idnum;
			newRod.numADPPiMonomers = ds.readInt();
			newRod.numADPMonomers = ds.readInt();
			if (ds.readByte() == 1) { newRod.plusCapped = true; } else { newRod.plusCapped = false; }
			if (Env.loadedRodsUncapped) { newRod.plusCapped = false; } // override saved value of plusCapped
			if (ds.readByte() == 1) { newRod.minusCapped = true; } else { newRod.minusCapped = false; }
			if (ds.readByte() == 1) { newRod.aBranch = true; } else { newRod.aBranch = false; }
			if (ds.readByte() == 1) { 	// branchMember info
				newRod.structureMember = true;
				newRod.myStructuresIdNumber = ds.readInt(); 
				newRod.structureId = ds.readInt(); 
			} else { newRod.structureMember = false; }
			newRod.helixAng = ds.readFloat();
			newRod.arpCount = ds.readInt();
			newRod.branches = ds.readInt();
			newRod.arps = new A_Arp [newRod.arpCount];
			for (int i=0; i < newRod.arpCount; i++) { newRod.arps[i] = A_Arp.readArp(ds); }
			newRod.adfCount = ds.readInt();
			newRod.adfLoc = new double [newRod.adfCount];
			for (int i=0; i < newRod.adfCount; i++) { newRod.adfLoc[i] = ds.readFloat(); }
			return newRod;
		} catch (IOException ioe) { talkln ("error in reading rod"); }
		return null;
	}
	
	public void initializeAfterRead () {
		// set some lengths
		ADPPiLength = numADPPiMonomers*Env.lengthPerActin;
		ADPLength = numADPMonomers*Env.lengthPerActin;
		// superstructure pointers
		if (structureMember) { 
			if (thePlayers[myStructuresIdNumber] instanceof SuperStructure) {} else { talkln ("supposed actinbranch is " + String.valueOf(thePlayers[structureId])); }
			myStructure = (SuperStructure)thePlayers[myStructuresIdNumber]; 
		}
		// initialize Arps
		for (int i=0; i < arpCount; i++) { arps[i].initializeArpAfterRead(thePlayers); }
	}
		

}
	
	
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		