package rocketBugs;
/*
	ActinBranch... this class, extending Player, holds together actin filaments that comprise a single branching 
	network, i.e. all the filaments that have originated from the same parent.  An ActinBranch can in turn be
	part of another superstructure, such as a comet tail.
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math.*;
import java.awt.*;
import java.io.*;

public class ActinBranch extends SuperStructure {
		
	public ActinBranch () { 
		super();
	}
	
	public void addSubBranch (A_Rod baseP) {
		addMember (baseP);
		for (int i = 0; i < baseP.arpCount; i++) {
			if (baseP.arps[i].active) { addSubBranch (baseP.arps[i].branch); }
		}
	}
	
	public void removeSubBranch (A_Rod baseP) {
		removeMember (baseP);
		for (int i = 0; i < baseP.arpCount; i++) {
			if (baseP.arps[i].active) { removeSubBranch (baseP.arps[i].branch); }
		}
	}
	
	public void calculateProperties () {
		A_Rod rod;
		// find the centroid
		Pt3D posSums = new Pt3D();
		double lsum = 0;
		for (int i=0; i < numMems; i++) {
			if (members[i] instanceof A_Rod) { } else { talkln (" members[" + String.valueOf(i) + "] is " + String.valueOf(members[i])); }
			rod = (A_Rod) members[i];
			posSums.inc(rod.length, rod.coord);
			lsum += rod.length;
		}
		coord.scale(1/lsum, posSums);
		// find the major axis 
		Pt3D xPt = new Pt3D();						// store the furthest point from centroid
		majorAxis = 0;
		double cenDist1, cenDist2;
		for (int i=0; i < numMems; i++) {
			rod = (A_Rod) members[i];
			cenDist1 = Pt3D.ptDistSqrd(rod.end1, coord);
			cenDist2 = Pt3D.ptDistSqrd(rod.end2, coord);
			if (cenDist1 > majorAxis) {
				majorAxis = cenDist1;
				xPt.copy(rod.end1);
			}
			if (cenDist2 > majorAxis) {
				majorAxis = cenDist2;
				xPt.copy(rod.end2); 
			}
		}
		majorAxis = Math.sqrt(majorAxis);
		// calculate the angles of the ActinBranch based on this major axis
		angle.x = Math.atan2(xPt.y-coord.y, xPt.x-coord.x);
		angle.y = Math.asin((coord.z-xPt.z)/majorAxis);
		angle.z = 0;
		
		// initialize all the cosine and sin vectors, transformation matrices, etc
		initialize();		
		
		// with the major axis established, find the smallest ellipse, within a tolerance, to contain all rod centers
		smallestEllipse();
		
		// now figure out the drag coefficients based on minor and major axis
		double majorAxisM = 1.0e-6*majorAxis;	// convert to meters from microns
		double minorAxisM = 1.0e-6*minorAxis;	// ditto
		double logTerm = Math.log(2*majorAxis/minorAxis);
		bTransGam.x = 4*Math.PI*Env.aeta*majorAxisM/(logTerm-.5);						// see Berg p. 57
		bTransGam.y = 8*Math.PI*Env.aeta*majorAxisM/(logTerm+.5);						// ditto Berg
		bTransGam.z = bTransGam.y;
		bRotGam.x = 16*Math.PI*Env.aeta*majorAxisM*minorAxisM*minorAxisM/3;			// see Berg p. 84
		bRotGam.y = 8*Math.PI*Env.aeta*Math.pow(majorAxisM,3)/(3*(logTerm-.5));		// ditto Berg
		bRotGam.z= bRotGam.y;
		
		//*** INSERT: scale factor for aeta
		
		
		double effectiveNumOfMon = 2*majorAxis/Env.lengthPerActin; //a value for numberofmonomers if assume that the 
		//max length of the branch has the same numberofmonomers as a filament(A_rod) of that length
		//2*majorAxis is the longest diameter of the branch ellipse
		
		// call method to modify drag values based on different physical processes
		//dragModifications(2*majorAxis, effectiveNumOfMon*effectiveNumOfMon*Env.lengthPerActin);
		
		//********
		//dragModifications based on functions to increase drag artificially and try turning off any fixation
		//********
		//x represents numOfMonomers, y represents the scale-factor, ie EffectiveNumMon
		double x1 = 10;	//these functions start at numOfMonomers = 10
		double y1 = Env.lubyPhelpsScaleAt10Mon; //the 1/(D/D0) for 10 monomers, ie where LubyPhelps method leaves off
		double x2 = Env.numMonomersAtGoalHighDrag;
		double y2 = Env.goalHighDrag;
		
		if (Env.useOldDrag){
			//sum viscous blobs
			numViscBlobs = 0;
			for (int i=0;i<numMems;i++) {
				numViscBlobs += members[i].numViscBlobs;
			}
			//I think this is what it was before 110105 - ie using the "length" of branch. Only A_Rod had the scaling by monCt^2.
			dragModifications(2*majorAxis, 2*majorAxis);
			//if want to match A_Rod then do below
			//dragModifications(2*majorAxis, effectiveNumOfMon*effectiveNumOfMon*Env.lengthPerActin);
			
		}else if (Env.adhesionsByLin){
			double aLin = (y2-y1)/(x2-x1);
			double bLin = y1-(aLin*x1);
			double linearEffectiveNumMon = (aLin*effectiveNumOfMon) + bLin;
			double effectiveLength = linearEffectiveNumMon * Env.lengthPerActin;
			dragModifications(2*majorAxis, effectiveLength);
			
		}else if(Env.adhesionsByQuad){
			double aQuad = ((x2*x2)-(x1*x1))/(y2-y1);
			double bQuad = y1-(aQuad*x1*x1);
			double quadEffectiveNumMon = (aQuad*effectiveNumOfMon*effectiveNumOfMon) + bQuad;
			double effectiveLength = quadEffectiveNumMon * Env.lengthPerActin;
			dragModifications(2*majorAxis, effectiveLength);
			
		}else if (Env.adhesionsByExp){
			double bExp = (Math.log(y2/y1))/(x2-x1);
			double aExp = y1/(Math.exp(bExp*x1));
			double exponentEffectiveNumMon = aExp*Math.exp(bExp*effectiveNumOfMon);	
			double effectiveLength = exponentEffectiveNumMon * Env.lengthPerActin;
			dragModifications(2*majorAxis, effectiveLength);
			
		}else{
			double effectiveLength = 2*majorAxis;
			dragModifications(2*majorAxis, effectiveLength);
		}
		
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
		
	}
	
	public void smallestEllipse() {
		// calculate centroid position in body-fixed coordinate frame
		Pt3D cmp = Pt3D.XToNewx(this,coord);
		minorAxis = majorAxis/2;
		double delta = majorAxis/2;	
		double tol = 0.01;	
		while (delta > tol) {
			delta = delta/2;
			if (fitsEllipse(cmp)) {
				minorAxis += -delta;
			} else {
				minorAxis += delta;
			}
		}
	}
	
	public boolean fitsEllipse (Pt3D cmp) {
		Player curP;
		Pt3D Pp = new Pt3D();
		Pt3D diffVec = new Pt3D();
		Pt3D diffSqrd = new Pt3D();
		double distCM;
		for (int i=0; i < numMems; i++) {
			curP = members[i];
			// use primed coordinate system so ellipse major axis is aligned with x' axis.
			Pp.XTox(this, curP.coord);
			diffVec.sub(Pp, cmp);
			diffSqrd.mult(diffVec,diffVec);
			distCM = diffSqrd.x/Math.pow(majorAxis,2)+diffSqrd.y/Math.pow(minorAxis,2)+diffSqrd.z/Math.pow(minorAxis,2);
			if (distCM > 1.01) { return false; }
		}
		return true;
	}
	
	private void checkFixing () {
		if (! fixedInSpace) {
			if (bTransGam.y > Env.dragToFix) { fixedInSpace = true; }
			if (Env.fixAllActin) { fixedInSpace = true; }
		}
	}
	public static ActinBranch makeNewActinBranch () {
		ActinBranch newBranch = new ActinBranch ();
		return newBranch;
	}
	
	public static A_Rod addFActinBranch (A_Rod parent, Pt3D bloc) {
		Pt3D coord = new Pt3D(bloc.x, bloc.y, bloc.z);
		Pt3D bUVec = makeBranchUVec(parent.end1, coord, parent.uVec, parent.t1Vec);
		A_Rod child = A_Rod.makeNewRod (coord, bUVec, 1);
		A_Rod.shiftRod(child,2);	// shift rod along its own uVec by this many lengthPerActin
		if (parent.structureMember) {
			parent.myStructure.addMember(child);
		} else {
			ActinBranch newBranch = makeNewActinBranch();
			newBranch.addMember (parent);
			newBranch.addMember (child);
			newBranch.calculateProperties();
		}
		return child;
	}
	
	public static Pt3D makeBranchUVec (Pt3D endMark, Pt3D coord, Pt3D uVec1, Pt3D tVec1) {
		// first find the magnitude of the branch angle for this particular branch
		double bAng, tAng;
		double bAngVar = Rdm.uRdm()*Env.branchAngleVar;
		if (Rdm.uRdm() < 0.5) { bAng = Env.branchAngle+bAngVar; } else { bAng = Env.branchAngle-bAngVar; }
		if (Env.branchRdm) {
			tAng = Rdm.uRdm()*2*Math.PI;
		} else {
			double fDist = Pt3D.ptDist(endMark, coord);
			double divNum = (int) (fDist/Env.helixWavelength);
			tAng = ((fDist - divNum*Env.helixWavelength)/Env.helixWavelength)*2*Math.PI;
		} 
		Pt3D zVec1 = Pt3D.Cross(uVec1, tVec1);	// third orthogonal vector
		Pt3D baseVec = Pt3D.Add(Math.cos(tAng), tVec1, Math.sin(tAng), zVec1);
		Pt3D uVec2 = Pt3D.Add(Math.cos(bAng), uVec1, Math.sin(bAng), baseVec);
		if (Env.qkMovie) { FileOps.saveNucInfo(uVec2); }
		return uVec2;
	}
	
	public static void ckBranchAng () {
		Pt3D rdmVec = Pt3D.RandomUnitVec();
		Pt3D branchAngles = Pt3D.EulerAnglesFromUVec(rdmVec);
		if (Env.qkMovie) { FileOps.saveNucInfo(branchAngles); }
	}
	
	
	public void showYourself (Graphics g, double scale, PinCam cam) {
		if (Env.showDragEllipses) {
			Pixel cenPix = Pixel.ptToNewPix(coord, scale, cam.offset, cam.e);
			int cenRad = (int) Math.ceil(.01*scale);
			showCentroid(g, cenPix, cenRad);
			Pt3D majDRx = new Pt3D (majorAxis,0,0);
			Pt3D majDRX = Pt3D.xToNewX(this,majDRx);
			Pt3D end1 = Pt3D.Sub(coord, majDRX);
			Pt3D end2 = Pt3D.Add(coord, majDRX);
			Pixel e1 = Pixel.ptToNewPix(end1,scale,cam.offset,cam.e);
			Pixel e2 = Pixel.ptToNewPix(end2,scale,cam.offset,cam.e);
			g.setColor(Color.cyan);
			Pixel.drawLine(g, e1, e2);
			double theta, comp1, comp2;
			Pt3D dR = new Pt3D();
			Pt3D curPt = new Pt3D();
			Pixel pixL = new Pixel();
			Pixel pixN = new Pixel();
			for (int i = 0; i <= 20; i ++) {
				theta = 2*i*Math.PI/20;
				comp1 = minorAxis*Math.cos(theta);
				comp2 = minorAxis*Math.sin(theta);
				dR.xToX(this, new Pt3D(0,comp1,comp2));
				curPt.add(coord, dR);
				pixN.ptToPix(curPt,scale,cam.offset,cam.e);
				if (i != 0) { Pixel.drawLine(g, pixL, pixN); }
				pixL.copy(pixN);	// copy value of pixN to pixL
			}	
		}
	}
	
	public void saveYourself (DataOutputStream ds) {
		try {
			ds.writeByte(5);	// the ActinBranch identifier
			ds.writeInt(idnum);			// this ActinBranch's idnum
			ds.writeInt(strucId);		// this ActinBranch's positions in superstructure array
			ds.writeInt(numMems);		// number of players to follow
			for (int i=0; i < numMems; i++) {
				ds.writeInt(members[i].idnum);
			}
		} catch (IOException ioe) { talkln ("some sort of error in saving actin branch info"); }
	}
	
	public static Player readYourself (DataInputStream ds, RocketBugs MC) {
		try {
			int idnum = ds.readInt();
			int strucId = ds.readInt();
			ActinBranch newBranch = new ActinBranch();
			//newBranch.idnum = (short)idnum;
			//newBranch.strucId = (short)strucId;
			newBranch.numMems = ds.readInt();
			for (int i=0; i < newBranch.numMems; i++) {
				newBranch.memIds[i] = ds.readInt();
			}
			return newBranch;
		} catch (IOException ioe) { talkln ("some sort of error in reading actin branch info"); }
		return null;
	}
	
	public void initializeAfterRead () {
		for (int i=0; i < numMems; i++) { members[i] = thePlayers[memIds[i]]; }
		calculateProperties();
	}
	
	
}
