package rocketBugs;
/* 	Env.java	... the class with lots of static definitions defining the cellular environment. 
	Includes information on how values were estimated or from where they were taken.
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.text.*;
import java.util.Date;

import edu.cornell.lassp.houle.RngPack.RanMT;

public class Env {	
	//  **** psuedo-random number generator (PRNG) ****
	static final RanMT mtRNG = new RanMT(new Date());
	
	//*** BASIC RUNTIME MODEL OPTIONS ***
	static final boolean branchRdm = true;	// flag to dictate actin filament branching model... random or not
	
	// ***** PHYSICAL CONSTANTS *****
	static final double Boltz = 1.380662e-23;			// Boltzman's constant in J/K
	static final double Avogadro = 6.0221367e17;		// Avogadro's number in #/M
	static final double cubicMicroMetersPerLiter = 1.0e15;	// m^3/liter
	static final double microMolarPerOne = Avogadro/cubicMicroMetersPerLiter;	// number of M/Single unit added to one m^3
	static final double degToRad = Math.PI/180;		// conversion of degrees to radians
	
	// **** BASIC PROPERTIES OF THE ENVIRONMENT ****
	static final double pH = 7.5;		// pH of buffer
	static final double tempK = 298.15;	// 25 deg Celsius in Kelvin
	static final double aeta = 3.00e-3;	 // viscosity of cytosol w/o cytoskeletal effect -- 3.0 mPa-s (Dembo '89)
	
	// **** FRICTION AND VISCOSITY EFFECTS ON/OFF BUG
	static final boolean tetherForceOff = false;		// turn off tether forces completely
	static final boolean useBugFriction = true;	// avoid unnecessary calcs by using this flag
	static final double bugFriction = 0.2;	// sliding friction coefficient for filaments on listeria surface
	static final int LINEAR_VISC = 1;
	static final int SQUARE_VISC = 2;
	static final int RATIONAL_VISC = 3;
	static final int viscType = Env.LINEAR_VISC; 	// choose visc. shell function type
	static final double lisViscDragSlope = 1;	// rate of inrease of drag on bug as function of filament number in visc. shell
	static final double sqViscCoeff = 0.01;	// tuning of square of filaments visc. drag increase
	static final double baseViscFactor = 25;
	static final double coeffViscFactor = 10000;		// make zero to have constant viscosity of base value
	static final double halfMaxViscFactor = 500;
	static final double coopViscFactor = 1;
	
	//*** assign integer values to protein names for clarity in programming ****
	static final int ACTIN_ATP = 0;			// free ATP-actin monomers
	static final int ACTIN_ADP = 1;			// free ADP-actin
	static final int DIMER = 2;				// Dimer of actin
	static final int TRIMER = 3;				// Trimer of actin
	static final int WASP = 4;				// WASP/Scar protein
	static final int ARP = 5;					// regular old ARP2/3
	static final int ARPa = 6;					// ARP2/3 in the active state
	static final int ADF = 7;					// actin depolymerizing factor
	static final int ADF_ATP_ACTIN = 8;		// ADF bound ATP-actin
	static final int ADF_ADP_ACTIN = 9;		// ADF bound ADP-actin
	static final int CAP = 10;				// capping protein
	static final int PROFILIN = 11;			// profilin
	static final int PROFILIN_ATP_ACTIN = 12;	// profilin bound ATP-actin
	static final int PROFILIN_ADP_ACTIN = 13;	// profilin bound ADP-actin
	static final int ACTA = 14;				// ActA
	static final int aACTININ = 15;			// alpha-Actinin
	static final int VASP = 16;				// VASP
	
	//*** INITIAL CONCENTRATIONS FOR AMBIENT ENVIRONMENT **** (M)
	static final int numEls = 17;						// number of implicit elements
	static final double ACTIN_ATPConc = 0.0;			// initial molarity of actin monomers
	static final double ACTIN_ADPConc = 0.0;			// initial molarity of actin monomers
	static final double DIMERConc = 0.0;				// initial molarity of actin dimers
	static final double TRIMERConc = 0.0;				// initial molarity of actin trimers
	static final double WASPConc = 0.00;				// initial molarity of WASP/Scar proteins
	static final double ARPConc = 0.3;//0.10;				// initial molarity of ARP2/3
	static final double ARPaConc = 0.00;				// initial molarity of active ARP2/3
	static final double ADFConc = 3.0;				// initial molarity of actin depolymerizing factor ADF
	static final double ADF_ATP_ACTINConc = 0.0;		// initial molarity of ADF bound ATP actin
	static final double ADF_ADP_ACTINConc = 0.0;		// initial molarity of ADF bound ADP actin
	static final double CAPConc = 2.0;				// initial molarity of capping proteins
	static final double PROFILINConc = 2.50;			// initial molarity of the profilin
	static final double PROFILIN_ATP_ACTINConc = 12;//7.5;	// initial molarity of profilin bound ATP actin
	static final double PROFILIN_ADP_ACTINConc = 0.0;	// initial molarity of profilin bound ADP actin
	static final double ACTAConc = 0.00;				// initial molarity of ActA
	static final double aACTININConc = 0.25;			// initial molarity of alpha-Actinin
	static final double VASPConc = 0.50;				// initial molarity of VASP
	//************************************
	
	// **** MASS OF PROTEINS ****  (kiloDaltons) kD
	static final double massOfActin = 42;
	static final double massOfAlphaActinin = 190;
	static final double massOfProfilin = 125;
	static final double massOfCap = 68;
	static final double massOfADF = 15;
	static final double massOfARP = 220;
	
	// **** EFFECTIVE RADIUS OF PROTEINS *** (m)
	// Use the following rules of thumb from Howard:
	// 		~1.2 nm^3 for each kD of protein (1.2e-9 m^3)
	//		~7 amino acids per nm^3
	// and assume globular so that volume = (4/3)*PI*r^3
	static final double reffCoeff = 3*(1.2e-9)/(4*Math.PI);
	static final double radOfActin = Math.pow(reffCoeff*massOfActin, 1.0/3.0);
	static final double radOfAlphaActinin = Math.pow(reffCoeff*massOfAlphaActinin, 1.0/3.0);
	static final double radOfProfilin = Math.pow(reffCoeff*massOfAlphaActinin, 1.0/3.0);
	static final double radOfCap = Math.pow(reffCoeff*massOfCap, 1.0/3.0);
	static final double radOfADF = Math.pow(reffCoeff*massOfADF, 1.0/3.0);
	static final double radOfARP = Math.pow(reffCoeff*massOfARP, 1.0/3.0);
	
	// **** DIFFUSIVITIES ****		(m^2/s)
	// Replace estimate based on globular nature with better values eventually....
	// If globular then use Stokes Law:  gamma = 6*PI*aeta*r  
	// Einstein relation:  D = kT/gamma
	// So....  D = kT/(6*PI*aeta*reff);
	static final double diffCoeff = 1e18*Boltz*tempK/(6*Math.PI*aeta);  // to get proper units
	static final double alphaACTIN = diffCoeff/radOfActin;	// diffusivity of Actin monomers.
	static final double alphaDIMER = .5*alphaACTIN;		// diffusivity of Actin dimer
	static final double alphaTRIMER = .3*alphaACTIN;		// diffusivity of Actin trimer
	static final double alphaWASP = 0.0;					// diffusivity of WASP/Scar protein.
	static final double alphaARP = diffCoeff/radOfARP; 		// diffusivity of ARP complex.
	static final double alphaARPa = alphaARP;				// diffusivity of the ActiveARP
	static final double alphaADF = diffCoeff/radOfADF;		// diffusivity of ADF/cofilin.
	static final double alphaADF_ACTIN = alphaACTIN;		// diffusivity of ADF-Actin
	static final double alphaCap = diffCoeff/radOfCap;		// diffusivity of the Cap proteins.
	static final double alphaPROFILIN = diffCoeff/radOfProfilin;// diffusivity of Profilin.
	static final double alphaPROFILIN_ACTIN = alphaPROFILIN;	// diffusivity of Profilin-Actin
	static final double alphaActA = 0.0;					// diffusivity of ActA ( a surface bound non-diffuser)
	static final double alphaActinin = diffCoeff/radOfAlphaActinin;	// diffusivity of alpha-actinin
	static final double alphaVASP = 0.0;					// diffusivity of VASP
	
	//**** DIFFUSION FLAGS... WHICH THINGS DIFFUSE *****
	static final boolean ACTIN_ATPDiffuse = false;
	static final boolean ACTIN_ADPDiffuse = false;
	static final boolean DIMERDiffuse = false;
	static final boolean TRIMERDiffuse = false;
	static final boolean WASPDiffuse = false;
	static final boolean ARPDiffuse = false;
	static final boolean ARPaDiffuse = false;
	static final boolean ADFDiffuse = false;
	static final boolean ADF_ATP_ACTINDiffuse = false;
	static final boolean ADF_ADP_ACTINDiffuse = false;
	static  boolean [] diffuse = new boolean [numEls];
	
	// **** ACTIN PROPERTIES ****
	static final double actinElasticity = 2.3e9;				// Young's modulus (GPa) which is (G Newtons/m^2)
	static final double actinEI = 60e-27;					// (N-m^2) Elasticity * Second Moment for Actin filament (Howard Table 6.4)
	static final double actinPersistenceLength = 15.0;		// (m) persistence length of actin
	static final double gActinDiameter = 0.0054;			// (m) diameter of G-actin
	static final double lengthPerActin = 0.5*gActinDiameter;	// (m) length added to filament by one actin monomer
	static final double helixWavelength = 0.037;			// wavelength of actin filament helix = 37 nm. (m)
	static final double actinPerTurn = helixWavelength/lengthPerActin; 	// number of monomers per helical turn
	static final double branchAngle = 70*Math.PI/180;  		// mean branching angle.. from Pollard et al review 2000.
	static final double branchAngleVar = 0;//7*Math.PI/180;	// branch angle variance
	static final double widthToDiam = 0.007/0.0054;			// use measured values of f-actin & g-actin diameter to find ratio between
	static final double fActinDiameter = widthToDiam*gActinDiameter;				// diameter of actin filament
	static final double helixAngPerActin = Math.PI/actinPerTurn; // helical turn per actin monomer...only 1/2 actual turn per wavelength
	
	// **** ARP2/3 BINDING
	static final double minARPbindLength = 2*lengthPerActin;	// minimum length needed for binding of ARP2/3
	static final double minARPgapLength = 4*lengthPerActin;	// minimum gap between ARPs
	
	//*** IMPLICIT OBJECT DEFINITIONS **** (idNum, ambientConc, diffusivity, diffuser)
	static ImplicitPlayer actin_atp = new ImplicitPlayer(ACTIN_ATP, ACTIN_ATPConc , alphaACTIN ,false);
	
	//*** DISTANCE TO WHICH WE CALCULATE DIFFUSION AND REACTIONS ****
	static final double rDiffusion = 3.0;			// (m) in micrometers
	
	//*** COLLISION RELATED ***
	static final double factinCollCushion = 0;		// some extra cushioning used for filament collisions
	
	static final double viscousShellThickness = 10*gActinDiameter;	// distance off cylindrical part of bug within which filaments contribute to increased viscous drag
	
	//**** LISTERIA-RELATED PARAMETERS ****
	
	// *** ActA DISTRIBUTION HISTOGRAM VALUES
	//**wild-type linescan (092805)
	/*static final double listeriaLength = 1.66;
	static final boolean actAFromHist = true; //use the uncommented histogram below for ActA distribution
	static final double [] actAMeasure = new double [] {0.005,0.016,0.028,0.041,0.048,0.056,0.058,0.059,0.061,0.061,0.059,0.054,0.051,0.047,0.044,0.041,0.040,0.039,0.037,0.036,0.034,0.028,0.023,0.018,0.011,0.004,0.000};
	static final double [] actAProb = new double [] {0.085,0.254,0.461,0.671,0.778,0.904,0.950,0.956,1.000,0.985,0.956,0.883,0.834,0.773,0.720,0.668,0.656,0.630,0.603,0.580,0.548,0.461,0.373,0.289,0.178,0.073,0.000};
	static final double equilActAAdjust = 1;	// wtActA fudge factor to adjust ActA amounts in "Listeria"
	static final boolean poleOnlyActA = false; //just a placeholder so that variable is defined //calls bothpolesActA method (in Listeria)
	*/
	
	//**ultra-polar linescan(092805)
	/*static final double listeriaLength = 1.66;
	static final boolean actAFromHist = true; //use the uncommented histogram below for ActA distribution
	static final double [] actAMeasure = new double [] {0.037,0.055,0.074,0.083,0.082,0.080,0.073,0.067,0.061,0.052,0.044,0.035,0.029,0.027,0.025,0.023,0.021,0.019,0.019,0.017,0.017,0.016,0.016,0.012,0.009,0.006,0.002};
	static final double [] actAProb = new double [] {0.449,0.659,0.893,1.000,0.995,0.961,0.883,0.805,0.737,0.629,0.532,0.424,0.356,0.322,0.298,0.283,0.249,0.224,0.229,0.205,0.210,0.190,0.195,0.141,0.112,0.068,0.024};
	static final double equilActAAdjust = 1/1.347;	//ultrapolarActA fudge factor to adjust ActA amounts in "Listeria"
	static final boolean poleOnlyActA = false; //just a placeholder so that variable is defined //calls bothpolesActA method (in Listeria)
	*/
	
	//**unipolar linescan (BiophysJ paper Fig. 2)
	//static final double listeriaLength = 1.51;
	static final boolean actAFromHist = true; //use the uncommented histogram below for ActA distribution
	//static final double [] actAMeasure = new double [] {0.076,0.095,0.106,0.101,0.096,0.088,0.080,0.076,0.072,0.068,0.062,0.046,0.025,0.009,0.000};
	//maxnormto1 ie +actAProb
	//static final double [] actAProb = new double [] {0.718,0.902,1.000,0.956,0.910,0.835,0.753,0.719,0.684,0.642,0.591,0.433,0.240,0.087,0.000};
	//medActA actAProbCeiling (see linescanforzoom file in excel)
	//static final double [] actAProb = new double [] {1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.987,0.939,0.881,0.811,0.594,0.330,0.120,0.000};
	//medActA HALFactAProbCeiling (see linescanforzoom file in excel)
	//static final double [] actAProb = new double [] {0.500,0.500,0.500,0.500,0.500,0.500,0.500,0.494,0.470,0.440,0.405,0.297,0.165,0.060,0.000};
	//lowActA actAProbCeiling (see linescanforzoom file in excel)
	//static final double [] actAProb = new double [] {0.530,0.667,0.739,0.707,0.673,0.617,0.557,0.532,0.506,0.474,0.437,0.320,0.178,0.065,0.000};
	//lowActA HALFactAProbCeiling (see linescanforzoom file in excel)
	//static final double [] actAProb = new double [] {0.265,0.333,0.370,0.353,0.336,0.309,0.278,0.266,0.253,0.237,0.218,0.160,0.089,0.032,0.000};
	//highActA actAProbCeiling (see linescanforzoom file in excel)
	//static final double [] actAProb = new double [] {1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.915,0.507,0.185,0.000};
	//highActA HALFactAProbCeiling (see linescanforzoom file in excel)
	//static final double [] actAProb = new double [] {0.500,0.500,0.500,0.500,0.500,0.500,0.500,0.500,0.500,0.500,0.500,0.457,0.254,0.092,0.000};
	
	//**ID3 bugexamples distributions (012006)
	static final double listeriaLength = 1.7;	// 1.7 for most, or 2.26 for bipolar, 1.51 for cartoon distributions
	//31710	*** normal	... length is 1.7, numActAs is 10K
	static final double [] actAMeasure = new double [] {0.038,0.064,0.084,0.093,0.091,0.090,0.086,0.080,0.080,0.074,0.064,0.057,0.044,0.032,0.018,0.006,0.000};
	static final double [] actAProb = new double [] {0.854,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.726,0.403,0.142,0.000};
	//32110
	//static final double [] actAMeasure = new double [] {0.023,0.062,0.075,0.091,0.092,0.091,0.087,0.077,0.074,0.064,0.062,0.059,0.055,0.044,0.033,0.011,0.000};
	//static final double [] actAProb = new double [] {0.269,0.711,0.869,1.000,1.000,1.000,1.000,0.893,0.855,0.741,0.721,0.677,0.636,0.507,0.381,0.126,0.000};
	//31904	*** ultapolar... length is 1.7, change numActAs to 7890
	//static final double [] actAMeasure = new double [] {0.074,0.109,0.119,0.108,0.092,0.077,0.067,0.061,0.058,0.053,0.050,0.044,0.037,0.029,0.017,0.007,0.000};
	//static final double [] actAProb = new double [] {1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.946,0.888,0.787,0.660,0.519,0.302,0.127,0.000};
	//31506
	//static final double [] actAMeasure = new double [] {0.039,0.093,0.119,0.118,0.094,0.084,0.067,0.058,0.051,0.052,0.051,0.050,0.049,0.041,0.020,0.014,0.000};
	//static final double [] actAProb = new double [] {0.335,0.794,1.000,1.000,0.799,0.716,0.576,0.495,0.438,0.446,0.437,0.423,0.420,0.353,0.170,0.117,0.000};
	
	//bipolar (Fig2 BiophysJ) adjusted for same conditions as these ID3 bugs. Length is 2.26, numActAs is 10K
	//static final double [] actAMeasure = new double [] {0.047,0.069,0.089,0.087,0.087,0.078,0.069,0.068,0.043,0.033,0.021,0.006,0.015,0.034,0.034,0.041,0.035,0.038,0.042,0.033,0.023};
	//static final double [] actAProb = new double [] {1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,0.980,0.754,0.488,0.126,0.347,0.771,0.784,0.934,0.792,0.874,0.964,0.759,0.522};

	
	//**poleonly ActA's by histogram (111905 and on)
	//static final double [] actAMeasure = new double [] {1.0,0.0,0.0,0.0};		// pole-only
	//static final double [] actAProb = new double [] {1.0,0.0,0.0,0.0};		// pole-only
	//static final double [] actAProb = new double [] {0.5,0.0,0.0,0.0};
	//static final double [] actAMeasure = new double [] {0.5,0.5,0.0,0.0};
	//static final double [] actAProb = new double [] {1.0,1.0,0.0,0.0};
	//static final double [] actAProb = new double [] {0.5,0.5,0.0,0.0};
	//static final double [] actAMeasure = new double [] {0.34,0.33,0.33,0.0};	// 	3/4
	//static final double [] actAProb = new double [] {1.0,1.0,1.0,0.0};			//	3/4
	
	//static final double [] actAProb = new double [] {0.5,0.5,0.5,0.0};
	//**poleonly ActA's histograms (012006)
	//static final double [] actAMeasure = new double [] {0.8,0.1,0.1,0.0};		// sprinkle
	//static final double [] actAProb = new double [] {1.0,0.53,0.53,0.0};		// sprinkle
	//static final double [] actAMeasure = new double [] {0.45,0.45,0.1,0.0};
	//static final double [] actAProb = new double [] {1.0,1.0,0.53,0.0};
	//static final double [] actAMeasure = new double [] {0.8,0.0,0.2,0.0};	
	//static final double [] actAProb = new double [] {1.0,0.0,0.53,0.0};
	//static final double [] actAMeasure = new double [] {0.3,0.25,0.2,0.14,0.08,0.03,0.0,0.0};
	//static final double [] actAProb = new double [] {1.0,1.0,1.0,1.0,0.85,0.32,0.0,0.0};
	
	
	
	static final double equilActAAdjust = 1;//1/1.367;	//ultrapolarActA fudge factor to adjust ActA amounts in "Listeria"
	static final boolean poleOnlyActA = false; //just a placeholder so that variable is defined //calls bothpolesActA method (in Listeria)
	
	//other parameters from elsewhere in env file that i'm moving here temporarily for rapid screening
	//static final double numberActAs = 5000; // change to 7890 for ultrapolar or norm 7890, 10,000 for all others
	static final double numberActAs = 8000;//4000 + Env.mtRNG.raw()*(8000 - 4000);
	static final boolean useActAThereProb = true; 		// if true then use the actAProb histogram to effect probability of binding. If FALSE then do as before - distribution no effect on binding
	static final double poleOnlyAdjustFactor = 1.0; //value that Listeria.bindprob is multiplied by s.t. less total ActA means less prob of binding
	static final double arpOnFudgeFactor = 8;//0.5;		// adjust arp on prob in ActA.java and hence side-branching
	static final double kMinusADP_Off = 3.6;//8; //2.7;//0.27;	// minus-end dissociation rate of ADP-actin 
	static final double kATPtoADPPi = 12.3;		// hydrolysis rate of ATP to ADP-Pi (s^-1)
	static final double kADPPitoADP = 0.1; //0.01;//0.0026;		// dissociation of Pi from ADP-Pi  (s^-1)
	static final boolean fixByMonCt = false;	//if true then use monCtToFix (see A_Rod)
	static final int monCtToFix = 40;
	static final boolean fixByAge = false;		//if true then use ageToFix (see A_Rod)
	static final double ageToFix = 5.0; 					// seconds till firmly fixed
	static final boolean noActALinkForce = false; 		// if true then in ActA.calculateLinkForce return a force of Zero for links
	
	//**new cartoon distributions with new definition for actAProb = (numberActAs/10K)*actAMeasure
	static boolean useNewToonDist = false;
	//static double offSet = Env.mtRNG.raw() - 0.5;  // range it from -0.5 to 0.5
	static double minAmp = 0.3;
	static double maxAmp = 1.0;
	static double rdmAmp = minAmp + (maxAmp-minAmp)*Env.mtRNG.raw(); // range from minAmp to maxAmp;
	static double AF = numberActAs/500;
	static int numMeasures = 17;
	//static final double [] actAMeasure = new double [numMeasures];
	//static final double [] actAProb = new double [numMeasures];
	
	//**bipolar linescan (BiophysJ paper Fig. 2)
	/*static final double listeriaLength = 2.37;
	static final boolean actAFromHist = true; //use the uncommented histogram below for ActA distribution
	static final double [] actAMeasure = new double [] {0.007,0.047,0.069,0.089,0.087,0.087,0.078,0.069,0.068,0.043,0.033,0.021,0.006,0.015,0.034,0.034,0.041,0.035,0.038,0.042,0.033,0.023,0.000};
	static final double [] actAProb = new double [] {0.081,0.526,0.780,1.000,0.979,0.979,0.879,0.777,0.772,0.487,0.374,0.242,0.063,0.172,0.383,0.390,0.464,0.393,0.434,0.479,0.377,0.260,0.000};
	static final double equilActAAdjust = 1/1.146;	//ultrapolarActA fudge factor to adjust ActA amounts in "Listeria"
	static final boolean poleOnlyActA = false; //just a placeholder so that variable is defined //calls bothpolesActA method (in Listeria)
	*/
	
	//**bipolar fighter linescan (BiophysJ paper Fig. 3)
	/*static final double listeriaLength = 2.37;
	static final boolean actAFromHist = true; //use the uncommented histogram below for ActA distribution
	static final double [] actAMeasure = new double [] {0.016,0.035,0.055,0.073,0.077,0.073,0.066,0.060,0.052,0.039,0.028,0.019,0.022,0.027,0.035,0.042,0.052,0.056,0.058,0.051,0.041,0.025,0.000};
	static final double [] actAProb = new double [] {0.202,0.458,0.718,0.938,1.000,0.940,0.850,0.779,0.670,0.509,0.360,0.248,0.280,0.348,0.456,0.544,0.669,0.730,0.745,0.656,0.525,0.318,0.000};
	static final double equilActAAdjust = 1;	//ultrapolarActA fudge factor to adjust ActA amounts in "Listeria"
	static final boolean poleOnlyActA = false; //just a placeholder so that variable is defined //calls bothpolesActA method (in Listeria)
	*/
	
	//**PoleOnlyActA distributions
	/*//static final double listeriaLength = 1.66;
	static final double listeriaLength = 1.51;
	static final boolean actAFromHist = false; //calls other method (in Listeria) for poleonlyActA or bothpolesActA
	static final boolean poleOnlyActA = true; //calls poleonlyActA method (in Listeria)  
	//static final boolean poleOnlyActA = false; //calls bothpolesActA method (in Listeria)
	static final double listeriaCylDiam = 0.7;		// diameter of Listeria cylinder, m.
	static final double listeriaCylLength = listeriaLength - listeriaCylDiam; 	// length of cylindrical part of listeria
	static final double listeriaCapRad = listeriaCylDiam/2;		// radius of spherical Listeria cap, m.
	static final double capSurfaceArea = 2.0*Math.PI*(listeriaCapRad*listeriaCapRad); // surface area of one spherical cap (ie (4pi()r^2)/2)
	static final double cylSurfaceArea = 2.0*Math.PI*(listeriaCylDiam/2.0)*listeriaCylLength; // surface area of cylinder
	//placeholders so program can run
	static final double [] actAMeasure = new double [] {1.0,0.0,0.0,0.0};
	static final double [] actAProb = new double [] {1.0,0.0,0.0,0.0};
	//static final double [] actAMeasure = new double [] {1.0,0.0,0.0,0.0,0.0};
	//static final double [] actAProb = new double [] {1.0,0.0,0.0,0.0,0.0};
	//poleonly ActA amount adjustment factor
	static final double equilActAAdjust =  (capSurfaceArea) / ((2*capSurfaceArea) + cylSurfaceArea);	//poleonlyActA fudge factor to adjust ActA amounts in "Listeria"
	//static final double equilActAAdjust =  ((2*capSurfaceArea) / ((2*capSurfaceArea) + cylSurfaceArea));	//bothpolesActA fudge factor to adjust ActA amounts in "Listeria"
	*/
	//*** LISTERIA PARAMETERS ***
	//**comment out this section if PoleOnlyActA or other ActA dist not using a histogram 
	//static final double listeriaLength = 1.66;			// length of Listeria in m. **is now determined in specific ActA distribution above
	static final double listeriaCylDiam = 0.7;		// diameter of Listeria cylinder, m.
	static final double listeriaCylLength = listeriaLength - listeriaCylDiam; 	// length of cylindrical part of listeria
	static final double listeriaCapRad = listeriaCylDiam/2;		// radius of spherical Listeria cap, m.
	static final double capSurfaceArea = 2.0*Math.PI*(listeriaCapRad*listeriaCapRad); // surface area of one spherical cap (ie (4pi()r^2)/2)
	static final double cylSurfaceArea = 2.0*Math.PI*(listeriaCylDiam/2.0)*listeriaCylLength; // surface area of cylinder
	
	//**** FLUXES TO LISTERIA SURFACE ****
	static final double arpToSpherePerMicroMolar = 1.3e4;	// how many Arp2/3 "stick" to sphere per M Arp2/3 conc... from Heat Eq calculation
	static final double actinToSpherePerMicroMolar = 6.875e4;	// how many actin monomers stick to sphere per M actin conc... see notebook
	static final double kArpFluxIn = arpToSpherePerMicroMolar*ARPConc;	// (/s) Calculated with Diffusion eq to sticky sphere... see notebook
	static final double kActinFluxIn = actinToSpherePerMicroMolar*PROFILIN_ATP_ACTINConc; // (/s) Calculated with 
	
	// *** ActA DISTRIBUTION, POLARITY, ETC
	//static final boolean useActAThereProb = true; 		// if true then use the actAProb histogram to effect probability of binding. If FALSE then do as before - distribution no effect on binding
	static final boolean useSurfaceGrids = false;		// should we use a 2-D mapping to Listeria surface for interaction sim.
	static final boolean swapActiveActAs = false;		// flag indicates whether we simulate the activation of ActAs or just have a set of actives
	static final double maxActALocAngle = 90*Math.PI/180;	// max angle of distribution of ActA on cap (*Must be 90 deg if lengthUpCyl != 0)
	static final double equivCylArea = 0.65;			// length of cylinder that would be covered by ActA if it were distributed with same density as on hemispherical cap
	static final double lengthUpCyl = listeriaCylLength;		// (m)  length up the side of Listeria that ActA is distributed (with some gradient)
	static final double actADensity = 6.4e17;	// (#/m^2) assuming 1e6 ActA on 0.5m diameter latex bead (from J. Theriot)
	static final double capActAArea = 2.0*Math.PI*(listeriaCapRad*listeriaCapRad);		// area of ActA distribution on cap
	static final double cylActAArea = 2.0*Math.PI*(listeriaCylDiam/2.0)*equivCylArea;	// area of ActA distribution on cylinder
	
	
	// *** ACTIN TETHERING PARAMETERS ***
	static final int maxActAsOnFilament = 50;//20;				// the number of bound ActAs on a given filament
	static final double minActAgapLength = 5*lengthPerActin;		// min distance between ActA binding sites
	static double tetherInGActin = 2;					// length of tether zone in terms of G-actin diameter
	static double actATetherLength = tetherInGActin*gActinDiameter;		// (m) distance from Listeria surface in which tethering occurs/length of ActA strand outside bacteria
	static double actAProtectsCappingDist = tetherInGActin*gActinDiameter;	// distance from end2 of filament to which a bound ActA protects from capping
	static final double actinBaseSpring = 1.0e-10;			// (N/m) hack stiffness of actin meshwork spring
	static double maxTetherStrain = 1.05;				// strain of assumed relaxed position before breakage
	static double maxTetherStretch = maxTetherStrain*actATetherLength;		// max allowable stretch of tether
	static double maxStretchDistFromActA = maxTetherStretch-actATetherLength; //distance between ActA length and maxTetherLength
	static final double maxAllowTetherForce = 1.0e-11;			// (N) max allowable force in tether
	static final double ActA_ArpGrabDist = .5*radOfARP;		// how far away can ActA_Arp2/3 complex "grab" a filament  
	static final double ActA_ArpGrabArea = Math.PI*Math.pow(ActA_ArpGrabDist,2);	// area on surface for binding Arp (pi*r^2)
	static final double ActA_ArpGrabAreaRatio = ActA_ArpGrabArea/(capActAArea+cylActAArea);	// ratio of grab area to total ActA distribution
	//static final double arpOnFudgeFactor = 1.5;//0.5;		// adjust arp on prob in ActA.java and hence side-branching
	static final double tetherBrownianScale = 500.0*lengthPerActin;		// scale factor to reduce thermal forcing for tethered objects per tether
	static final double cutoffLength = 100.0*lengthPerActin;		//cutoff length at which being linked no longer reduces brownian motion of filament
	static final boolean smallFilLargerStrain = false; 	// if true then smaller filaments allowed to stretch much further before breaking (see ActA.checkTetherDetatch)
	static final boolean tethersIncreaseDrag = true;	// flag for hack drag increase if tethered
	static final boolean ActARebinds = false;			// binding site slides
	static final boolean ArpBranchLink = false;			// if true, then actin-ActA link has a chance to immediately be bound to an Arp (see Listeria.chkRodTether)
	static final double percentNucleators = 0.1;		// if ArpBranchLink=true, this is percent of ActA's on surface that are nucleators, ie mediates ArpBound in List.chkRodTether
	
	static final boolean oldTetherProperties = true; 	// if true then use tether as before, with max tether strain/stretch etc. and make smallFilLargerStrain true
	static final boolean mechanicalTether = false;		// if true, then tether is a spring with Fmax and dmax
	static final boolean constantMechanical = false;
	static final boolean chemicalTether = false;		// if true, then tether has constant force if attached, and different off rates if strained or loose
	static final boolean constantChemical = false;		//if true, then a constant off rate, not adjusted by distance of tether from surface
	static final double chemTetherOffRate = 1.0;		//tethers/s * deltaT NEED TO ADJUST
	static final double maxTetherForce = 7.0e-6;		//for mechanical is maximum force of tether, for chemical is constant force of tether
	static double tetherSpringK = maxTetherForce/maxStretchDistFromActA; //spring constant for mechanical tether to calculate force of tether dependant on stretch
	static final boolean attenuateLinks = true;
	
	static final boolean keepTrackActAs=false;
	
	// *** HACKEOLAS ***
	static final double nucProbFactor = 3e-5;//0.00005;		// hack to adjust nucleation freq.
	static final double actinFlexFactor = 0.5;//1;			// hack to adjust flexibility related monomer addition to filament
	
	// *** EXTERNAL FORCES ***
	static final boolean opposingForcePresent = false;		// is there a force resisting bacterial motion forward?
	static final double opposingForce = -26e-12;				// value of opposing force in Newtons
	
	// *** SIMPLE ATTENUATION FACTORS ***
	static final double collisionAtten = 0.01;
	static final double linkAtten = 0.01;
	
	// *** FIXING IN SPACE, ADHESIONS, ETC
	static final boolean applyLubyPhelpsFactor = true;		// use a curve fit to Luby-Phelps data to modify viscous drags
	static final double lubyPhelpsScaleAt10Mon = 12.55783;	//value for 1/(D/D0) for 10 monomers - length in nm is 27
	static final boolean applyAdhesions = false;				// use an adhesion simulation to fix filaments in space
	static final double adhesionRate = 50;				// adhesions per second/m of filament length ... a coarse hack
	static final double adhesionFactor = 100;				// multiply drag of an object by this much per adhesion
	static double dragToFix = 1e6;						// value of drag above which the object is considered fixed... reset by Listeria
	static final boolean fixFilaments = false;				// should we fix in space based on criteria here?
	static final boolean fixAllActin = false;					// overrides everything else... actin filaments and branches fixed in space
	//static final boolean fixByMonCt = false;	//if true then use monCtToFix (see A_Rod)
	//static final int monCtToFix = 40;
	static final boolean ageBasedAdhesions = false;
	static final double ageAdhesionFactor = 50;
	//static final boolean fixByAge = false;		//if true then use ageToFix (see A_Rod)
	//static final double ageToFix = 3.0; 					// seconds till firmly fixed
	static final boolean lengthBasedAdhesions = false;
	static final double lengthAdhesionFactor = 100;
	static final boolean lengthBasedAdhesionsByFunction = false; 	//if true then use functions to determine how adhesions scale (see Player.DragModifications)
	
	static final boolean viscousBlobs = true;				// blobs glom onto the filaments
	static final double lengthForOneBlobPerSecond = 0.25*lengthPerActin;	// reference length... one viscous blob per second will add to this length
	static final double vBlobOnRate = 1/lengthForOneBlobPerSecond;		// (blobs/s-micron) viscous blob on rate per second-micron (of effective length)
	static final double vBlobOffRate = 0.5;				// (blobs/s)
	static final double numVBlobEqualBug = 25;//50;					// number of viscous blob that equals one bug/bead drag wise
	static final double blobGamScaleFactor = 1000; 	//based on fact that blobs need drag scaled up by a standard value (1000) simialr to lisTransGamScaleFactor but does not change
	static final double blobRotGamScaleFactor = 1000; 	//default is 1000; like blobGamScaleFactor but can adjust the rotGam for the viscous blobs separately (to test fil alignment, March2007)
	static final double bTransGamViscBlob = 6*Math.PI*Env.aeta*1.0e-6*0.5*blobGamScaleFactor;			//viscous drag based on a sphere of 1um diameter - the default for viscous blobs
	static final double bRotGamViscBlob = 8*Math.PI*Env.aeta*Math.pow(1.0e-6*0.5,3)*blobRotGamScaleFactor;	//viscous drag based on a sphere of 1um diameter - the default for viscous blobs
	static final double N = 1.0/Env.numVBlobEqualBug;
	static final Pt3D blobTransGam = new Pt3D(N*bTransGamViscBlob, N*bTransGamViscBlob, N*bTransGamViscBlob);
	static final Pt3D blobRotGam = new Pt3D(N*bRotGamViscBlob, N*bRotGamViscBlob, N*bRotGamViscBlob);
	static final double sideRotRestrictFactor = 1; //if 1 then no rotation restriction, otherwise restrict rotation for filaments along cylinder
	
	static final double numMonomersAtGoalHighDrag = 200;	//number of monomers at which want function to reach goalHighDrag
	static double goalHighDrag = 100*dragToFix;		//Env.dragToFix is changed in Listeria to bTransGam.y, (max drag of Listeria); here scaled by a factor
	static final boolean useOldDrag = true;		//if true then use DragModifications as pre 110105 (ie for runs testing polarity)
	static final boolean adhesionsByLin = false;	//if true then use LinearFunction to calculate drags y=ax+b
	static final boolean adhesionsByQuad = false;	//if true then use QuadraticFunction to calculate drags y=ax^2 +b
	static final boolean adhesionsByExp = false;	//if true then use ExponentialFunction to calculate drags y=ae^bx
	
	
	// **** DEAD DISTANCE ****
	static final double deadDist = Env.listeriaCapRad;	// distance (microns) from listeria coord at which something is dead

	// **** COLLISION DETECTORS ***
	static final double XYGridScale = 10;				// scale for Rough_Grid (XY detector)
	static final double YZGridScale = 10;				// scale for Rough_Grid_Lite (YZ detector)
	
	// **** SPHERICAL DIFFUSION GRID***	
	static final int molarGridScale = 30;		// # of radial elements in spherical grid that keeps track of implicit players
	static final boolean reactionsChangeGridConc = false; // whether to have reactions alter grid concentrations
	static final boolean useOnlyAmbient = true;		// if true then don't use spherical grid
	
	// **** THREAD NUMBERS ****
	static final int numberOfDiffReacThreads = 4; 	// # of threads for diffusion/reaction calculations
	static final int numberOfPlayerThreads = 4;		// # of threads to use in moving players
	static final int numberOfInteractionThreads = 4;	//  # of threads to use in interaction simulations
	static final int numberOfMaintenanceThreads = 1;// # of threads in player maintenance ***Must be 1***
	static final int numberOfCollisionThreads = 4;   // # of threads for distributing collision forces
	static final int numberOfLinkThreads = 4;	
	static final int numberOfFrictionThreads = 4;	
	
	// **** THREAD START AND STOP POINTS ****
	static final int numberOfWaves = 10;
	static final int diffStart = 0;					static final int diffStop = 0;
	static final int listeriaColStart = 1;			static final int listeriaColStop = 1;
	static final int colSum1Start = 2;				static final int colSum1Stop = 2;
	static final int linkSum1Start = 3;				static final int linkSum1Stop = 3;
	static final int fricStart = 4;					static final int fricStop = 4;
	static final int moveStart = 5;					static final int moveStop = 5;
	static final int interactStart = 6;				static final int interactStop = 6;
	static final int playerCleanupStart = 7;		static final int playerCleanupStop = 7;
	static final int linkUpdateStart = 8;			static final int linkUpdateStop = 8;
	static final int frictionCleanupStart = 8;		static final int frictionCleanupStop = 8;
	static final int linkerCleanupStart = 9;		static final int linkerCleanupStop = 9;
	static final int reacStart = 10;					static final int reacStop = 10;
	
	// **** RANDOM NUMBERS ****
	static final int numOfRdmNumbers = 2000000;	// # of random numbers to generate
	static final int rdmNumUse = 50;				// # of times to use a rdm number
	
	// **** TIME STEPS ****
	static double deltaT = 1e-4;					// time step in seconds
	static final int timeInt = 10;					// number of deltaTs in deltaT2
	static double deltaT2 = timeInt*deltaT;			// longer time step in seconds
	static final double runTime = 360;//600.0;				// final simulation time
	
	// **** RUNTIME PARAMETERS .... 
	static boolean remote = false;					// flag for running without GUI
	static final Pt3D dimensions = new Pt3D(20,20,20);	// dimensions in x-y-z, m.
	static final int maxPlayers = 10000;			// max number of players
	static final int maxLinkers = 100000;			// max number of linkers
	static final int maxCollisions = 10000;			// max number of collisions each time-step
	static final int maxSuperstructs = 5000;		// max number of superstructures
	static int counter = 0;							// keeps track of integration step
	static double simulationTime = 0;				// the current sim time
	static int updatePerSec = 500;					//(500 updates/sec)
	static int updateFrequency = (int)((1.0/deltaT)*(1.0/Env.updatePerSec)); // (updates/sec) # of integration steps for certain costly updating methods
	static final int numberOfInitialRods = 0; 		// # of initial rods to create for testing
	static boolean listeriaPresent = true;			// flag to indicate whether or not a listeria exists in this run
	static boolean brownianJiggle = true;			// flag for Brownian jostling
	static boolean shakeTransListeria = true;		// flag to Brownian translational force the listeria
	static boolean shakeRotListeria = true;			// flag to Brownian rotational force the listeria
	static boolean shakeActin = true;				// flag to Brownian force the actin and actin branches
	static double lisShakeAttenFactor = 0.2;		// scale Brownian forces by this factor
	static boolean actANucleation = true;			// does ActA nucleate new filaments
	//static boolean actANucleation = false;			// does ActA nucleate new filaments
	static final boolean modifyPolymProb = true;	// tipClearance modifies polymerization probability for actin filaments
	static final boolean collisionForcesCapOff = false; // flag indicates if a rod tip collision with Listeria will uncap
	static final boolean actAProtectsCapping = true;	// does binding to actA force + end cap off
	static final boolean loadedRodsUncapped = true;	// when we load a file, consider all filaments as uncapped
	static String icFile = "testfile.data";				// file that contains initial conditions
	static String icPath = null;					// path to ic file
	static final int collArrayStep = 100;			// step for increasing collision array size	
	
	// **** EVENT VARIABLES ***
	static int newTetherCt = 0;				// number of new tethers formed
	static int cutTetherCt = 0;				// number of tethers cut
	static int newArpCt = 0;				// number of new Arp2/3s
	static int actinAddFlexCt = 0;			// number of actin monomers added to filaments impinging.... should be zero always
	static int actinAdd1Ct = 0;				// number of actin monomers added < dist1 from listeria surface
	static int actinAdd2Ct = 0;				// number of actin monomers added < dist2 and > dist1 from listeria surface
	static int actinAdd3Ct = 0;				// number of actin monomers added < dist3 and > dist2 from listeria surface
	static double actinAddFlexCtCos = 0;		// cos of listeria orientation with polymerizing rod below level 1
	static double actinAdd1CtCos = 0;		// 	ditto level1
	static double actinAdd2CtCos = 0;		//	ditto level2
	static double actinAdd3CtCos = 0;		//  ditto level3
	static int closeTipCt = 0;				// number of plus-ends with tip clearance < dist1
	static int tipsOnLisSphere = 0;			// ditto, but also just on spherical cap
	static double aveColForceInx = 0;		// used with tipsOnLisSphere to figure relation between barbed-ends and propulsive force
	static double aveFricForceInx = 0;		// used with closeTipCt to compare fric. relations with PDE model
	static double aveNormForce = 0;			// sums normal forces over a reporting interval
	static int failedTether = 0;			// number of tether binding events that fail because no binding site
	static final double actinAdd1Dist = 1*gActinDiameter;
	static final double actinAdd2Dist = 2*gActinDiameter;
	static final double actinAdd3Dist = 4*gActinDiameter;
	//variables for tracking number of tethers etc. in regions A to D
	static boolean reportRegionInfo = true;
	static int newTetherCtInA = 0;				// number of new tethers formed in region A
	static int newTetherCtInB = 0;				// number of new tethers formed in region B
	static int newTetherCtInC = 0;				// number of new tethers formed in region C
	static int newTetherCtInD = 0;				// number of new tethers formed in region D
	static int newTetherCtInE = 0;				// number of new tethers formed in region E
	static int cutTetherCtInA = 0;				// number of tethers cut in region A
	static int cutTetherCtInB = 0;				// number of tethers cut in region B
	static int cutTetherCtInC = 0;				// number of tethers cut in region C
	static int cutTetherCtInD = 0;				// number of tethers cut in region D
	static int cutTetherCtInE = 0;				// number of tethers cut in region E
	static int strainedTetherCtInA = 0;
	static int strainedTetherCtInB = 0;
	static int strainedTetherCtInC = 0;
	static int strainedTetherCtInD = 0;
	static int strainedTetherCtInE = 0;
	static int monCtInA = 0;
	static int monCtInB = 0;
	static int monCtInC = 0;
	static int monCtInD = 0;
	static int monCtInE = 0;
	static int arpBranchCtInA = 0;
	static int arpBranchCtInB = 0;
	static int arpBranchCtInC = 0;
	static int arpBranchCtInD = 0;
	static int arpBranchCtInE = 0;
	static double arpBranchRatioInA = 0;
	static double arpBranchRatioInB = 0;
	static double arpBranchRatioInC = 0;
	static double arpBranchRatioInD = 0;
	static double arpBranchRatioInE = 0;
	
	//variables for tracking number of monomers and number of arps in a region of the tail
	static int monCt = 0;
	static int arpCt = 0;
	static int arpBranchCt = 0;
	static double arpBranchActMonRatio = 0;
	static final double frontDistFromLis = 0.2;
	static final double backDistFromLis = 0.5;
	static final double frontBorder = (-1*(Env.listeriaCapRad + Env.frontDistFromLis));
	static final double backBorder = (-1*(Env.listeriaCapRad + Env.backDistFromLis));
	
	//**** PAINTING VARIABLES ****
		// ***** text formatting
	static DecimalFormat scaleBarFormat = new DecimalFormat("#0.000;#0.000#");
		// ***** window sizing ***
	static final int zoomWindowSize = 800;			// size of zoomPanel
	static boolean usePinCam = false;			// show with pinhole camera
		// ***** painting options ***
	static final boolean showLinkers = false;			// flag for drawing all Linkers
	static final boolean showUntetheredActAs = true;	// flag for drawing untethered ActAs
	//static final boolean showUntetheredActAs = false;	// flag for drawing untethered ActAs
	static final boolean plotActAHist = true;			// plot the distribution of ActAs
	static final boolean showProbsOnScreen = true;		// for debugging, show event probabilities on screen
	static final boolean drawScaleBar = true;		// flag for drawing scale bar
	static boolean drawActinHelix = true;			// flag to indicate fancy helix drawing
	static boolean drawEachMonomer = true;		// flag to draw each monomer as a circle
	static boolean drawHelicalLines = false;			// flag to draw helix lines
	static double helixScaleCutOff = 250;			// value of scale at which to switch to simple line drawing
	static double helixBlobDiam = gActinDiameter;			// diameter to use in drawing blobs
	static final double helixAngleInc = Math.PI + helixAngPerActin;  // angle increment in radians
	static final double helixXInc = lengthPerActin;	// x increment per line segment
	static final double helicalBlobOffset = (fActinDiameter-helixBlobDiam)/2;	// offset from center line
	static final int elementToShow = -1;			// what element to show concentration of (-1 = none)
	static final boolean showDragEllipses = false;	// flag to show, or not, the equivalent ellipses used for drag
	static final boolean showHydrolysisStages = true;	// flag to paint hydrolyzed sections of actin
	static final boolean showARPLocations = true;		// flag to paint ARP2/3 locations on the filament
	static final boolean showADFLocations = true;		// flag to paint ADF/Cofilin locations on the filament
	static boolean rotateRocketBugsAngleC = false;		// flag for rotation of zoomPanel dispAng
	static boolean rotateRocketBugsAngleCC = false;		// flag for rotation the other way
	static final double viewAngleStep = 2*degToRad;	// increment in dispAng
	static boolean chgZIn = false;					// zoom in
	static boolean chgZOut = false;					// zoom out
	static final double zoomInc = 0.1;				// zoom in/out by this amount
	static int courseToShow = 0;					// course of spherical grid to show
	static final boolean showSurfaceGrids = false; 	// flag for showing surface grids involved in processes... good testing option
		// ***** colors *****
	//static final Color ATPActinColor = Color.yellow;
	static final Color ATPActinColor = new Color ((float)1.0,(float)1.0,(float)100.0/255);
	static final Color ADPPiActinColor = Color.orange;
	static final Color ADPActinColor = Color.red;
	static final Color cappedActinColor = Color.gray;
	//static final Color activeARPColor = new Color ((float)0.5,(float)0.5,(float)0.75);
	static final Color activeARPColor = new Color ((float)150.0/255,(float)200.0/255,(float)100.0/255);
	static final Color inactiveARPColor = Color.gray;
	static final Color adfColor = Color.darkGray;
	static final Color plusCapColor = Color.lightGray;
	static final Color minusCapColor = new Color ((float)0.5,(float)0.5,(float)0.75);
	static final Color showGridDefaultColor = Color.white;
	static final Color unboundActAColor = new Color ((float)0.50,(float)1.0,(float)0.25);
	static final Color boundActAColor = new Color ((float)0.45,(float)0.40,(float)0.85);
	static final Color listeriaColor = Color.white;
	static final Color backColor = Color.black; //new Color((float)254.0/255,(float)240.0/255,(float)200.0/255);
	static final Color zoomBoxColor = Color.lightGray;
	static final Color actinOutlineColor = Color.black;
	
	// **** REACTION RATES ****		_On (M^-1 s^-1)   _Off (s^-1)
		// capping protein
	static final double kCap_On = 3.0;		// cap association rate 
	static final double kCap_Off = 4.0e-4;		// cap dissociation rate 
		// barbed end dynamics
	static final double kPlusATP_On = 11.6;		// plus-end association rate of ATP-actin 
	static final double kPlusATP_Off = 1.4;		// plus-end dissociation rate of ATP-actin 
	static final double kPlusADP_On = 3.8;		// plus-end association rate of ADP-actin 
	static final double kPlusADP_Off = 7.2;		// plus-end dissociation rate of ADP-actin
	static final double kPlusProfilin_On = 11.6;	// plus-end association rate of Profilin-ATP-actin 
	static final double kPlusProfilin_Off = 1.4;	// plus-end dissociation rate of Profilin-ATP-actin 
		// pointed end dynamics
	static final double kMinusATP_On = 1.3;		// minus-end association rate of ATP-actin 
	static final double kMinusATP_Off = 0.8;	// minus-end dissociation rate of ATP-actin 
	static final double kMinusADP_On = 0.16;	// minus-end association rate of ADP-actin 
	//static final double kMinusADP_Off =2.7;//8; //2.7;//0.27;	// minus-end dissociation rate of ADP-actin 
		// profilin 
	static final double kProfilinATP_On = 5.0;	// association rate of profilin and ATP-actin 
	static final double kProfilinATP_Off = 5.0;	// dissociation rate of profilin and ATP-actin 
	static final double kProfilinADP_On = 5.0;	// association rate of profilin and ADP-actin 
	static final double kProfilinADP_Off = 10.0;	// dissociation rate of profilin and ADP-actin 
		// dimers and trimers
	static final double kDimer_On = 2.0e-9;	// association rate of actin-actin to form a dimer 
	static final double kDimer_Off = 5.0e-4;	// dissociation rate of dimer back to actin monomers 
		// arp2/3
	static final double kBoundActAArp_Off = 0.6;	// (/s) same as kActAActinArp_Off below
	static final double kArpFilamentNuc = 0.1;	// *** Check... rate of new filament nucleation by active ARP2/3
	static final double kActAActin_On = 43.0;	// association rate of ActA with actin monomer 
	static final double kActAActin_Off = 30.0;	// dissociation rate of ActA-actin
	static final double kActAActinArp_On = 0.8;	// association rate of ActA-actin and Arp2/3
	static final double kActAActinArp_Off = 0.6;	// dissociation rate of ActA-actin-Arp2/3
	static final double kActAFactinArp_On = 0.8;	// association rate of ActA and Factin-Arp2/3
	static final double kActAFactinArp_Off = 0.8;	// dissociation rate of ActA-Factin-Arp2/3
	static final double kATPFactinArp_On = 0;//2.0;		// association rate of Arp2/3 with F-actin (ATP)
	static final double kATPFactinArp_Off = 10;//0.1;		// dissociation rate of Arp2/3-Factin (ATP)
	static final double kADPFactinArp_On = 0;//2.0;		// association rate of Arp2/3 with F-actin (ADP)
	static final double kADPFactinArp_Off = 10;//2.0;	// dissociation rate of Arp2/3 with F-actin (ADP)
		// hydrolysis
	// **** vectorial model Carlier et al. 1987 and Melki et al. 1996 ****
	//static final double kATPtoADPPi = 12.3;		// hydrolysis rate of ATP to ADP-Pi (s^-1)
	//static final double kADPPitoADP = 0.1; //0.01;//0.0026;		// dissociation of Pi from ADP-Pi  (s^-1)
		// adf/cofilin
	static final double kATPFactinCofilin_On = 0.01;	// association rate of ATP-Factin with ADF/Cofilin
	static final double kATPFactinCofilin_Off = 0.2;	// dissociation rate of ATP-Factin with ADF/Cofilin
	static final double kADPFactinCofilin_On = 0.01;//0.1;	// association rate of ADP-Factin with ADF/Cofilin
	static final double kADPFactinCofilin_Off = 0.05;	// dissociation rate of ADP-Factin with ADF/Cofilin
	static final double kATPGactinCofilin_On = 0.01;	// association rate of ATP-Gactin with ADF/Cofilin
	static final double kATPGactinCofilin_Off = 0.006; // dissociation rate of ATP-Gactin with ADF/Cofilin
	static final double kADPGactinCofilin_On = 0.1;	// association rate of ADP-Gactin with ADF/Cofilin
	static final double kADPGactinCofilin_Off = 0.012; // dissociation rate of ADP-Gactin with ADF/Cofilin
	static final double kCofilinSever = 0.01;			// sever rate by Cofilin 
	
	// **** MOVIE MAKING AND OTHER INFO SAVING *****
	static final int lisInfoSavesPerSec = 10; //3300; 	// number of listeria info saves each second
	static int lisInfoFreq = (int)(Math.ceil((1.0/deltaT)*(1.0/lisInfoSavesPerSec))); 	// number of integration steps between lis info saves
	static final int histSavesPerSec = 10; 	// number of histogram saves each second
	static int histFreq = (int)(Math.ceil((1.0/deltaT)*(1.0/histSavesPerSec))); 	// int steps between saves
	static final int rawMovieFrameInt = 5;		// number of integration steps per raw movie frame
	//static final int qkMovieFrameInt = 200;
	static int qkFramesPerSec = 2;				// (60 frames/sec)
	static int qkMovieFrameInt = (int)((1.0/deltaT)*(1.0/qkFramesPerSec));	// (frames/sec) number of integration steps per qk movie frame
	static boolean qkMovie = false;				// flag indicates whether or not we're making qkMovie
	static boolean rawMovie = false;				// flag indicates whether or not we're making rawMovie
	static String rawMovieFileName = null;			// movie name
	static String rawMoviePath = null;				// movie path
	static String qkMovieFileName = null;
	static String qkMoviePath = null;
	static double saveConfPerSec = 0.1;		// (.1 saves/sec)
	static int saveConfFrequency = (int)((1.0/deltaT)*(1.0/Env.saveConfPerSec));	// (saves/sec) # of integration steps between configuration saves
	static String datFilePath = null;				// path for .dat config files
	static String srcFilePath = null;
	static final double rotateStepPerMovieFrame = 4.0*Math.PI/180;	// radians to rotate view after each movie frame
	static boolean epsOut = false; 		// flag for eps grab of image
	


	public static void recalculateDependents () {
		Env.actATetherLength = Env.tetherInGActin*Env.gActinDiameter;		// (m) distance from Listeria surface in which tethering occurs/length of ActA strand outside bacteria
		Env.actAProtectsCappingDist = Env.tetherInGActin*Env.gActinDiameter;	// distance from end2 of filament to which a bound ActA protects from capping
		Env.maxTetherStretch = Env.maxTetherStrain*Env.actATetherLength;	// need to reset this value
		Env.maxStretchDistFromActA = Env.maxTetherStretch - Env.actATetherLength;
		Env.tetherSpringK = Env.maxTetherForce/Env.maxStretchDistFromActA;
		Env.deltaT2 = Env.timeInt*Env.deltaT;
		Env.saveConfFrequency = (int)((1.0/Env.deltaT)*(1.0/Env.saveConfPerSec));	// (saves/sec) # of integration steps between configuration saves
		Env.updateFrequency = (int)((1.0/Env.deltaT)*(1.0/Env.updatePerSec)); // (updates/sec) # of integration steps for certain costly updating methods
		Env.qkMovieFrameInt = (int)((1.0/Env.deltaT)*(1.0/Env.qkFramesPerSec));	// (frames/sec) number of integration steps per qk movie frame
		Env.lisInfoFreq = (int)(Math.ceil((1.0/Env.deltaT)*(1.0/Env.lisInfoSavesPerSec))); // lis info saves
	}
	
	public static void resetEventCounters () {
		newTetherCt = 0;				// number of new tethers formed
		cutTetherCt = 0;				// number of tethers cut
		newArpCt = 0;					// number of new Arp2/3s
		actinAddFlexCt = 0;				// number of actin monomers added to filaments impinging.... should be zero always
		actinAdd1Ct = 0;				// number of actin monomers added < dist1 from listeria surface
		actinAdd2Ct = 0;				// number of actin monomers added < dist2 and > dist1 from listeria surface
		actinAdd3Ct = 0;				// number of actin monomers added < dist3 and > dist2 from listeria surface
		actinAddFlexCtCos = 0;			// cos of listeria orientation with polymerizing rod below level 1
		actinAdd1CtCos = 0;				// 	ditto level1
		actinAdd2CtCos = 0;				//	ditto level2
		actinAdd3CtCos = 0;				//  ditto level3
		closeTipCt = 0;					// number of plus ends with tip clearance < dist1 
		tipsOnLisSphere = 0;			// number of plus ends with tip clearance < dist1 and on spherical cap of bug
		aveColForceInx = 0;				// average collision force in path direction
		aveFricForceInx = 0;			// ave. fric force
		aveNormForce = 0;
		failedTether = 0;
		newTetherCtInA = 0;				// number of new tethers formed in region A
		newTetherCtInB = 0;				// number of new tethers formed in region B
		newTetherCtInC = 0;				// number of new tethers formed in region C
		newTetherCtInD = 0;				// number of new tethers formed in region D
		cutTetherCtInA = 0;				// number of tethers cut in region A
		cutTetherCtInB = 0;				// number of tethers cut in region B
		cutTetherCtInC = 0;				// number of tethers cut in region C
		cutTetherCtInD = 0;				// number of tethers cut in region D
	}
	
	public static void registerArp(A_Rod rod) {
		//if (rod.tipClearance < Env.actinAdd3Dist) { Env.newArpCt ++; }
		Env.newArpCt ++;
	}
	
	public static void registerPlusMon(A_Rod rod) {
		if (rod.tipClearance < Env.lengthPerActin) { 
			Env.actinAddFlexCt ++;  
			Env.actinAddFlexCtCos += Pt3D.Dot(Player.listeria.uVec,rod.uVec);
			return;
		} 
		if (rod.tipClearance < Env.actinAdd1Dist) { 
			Env.actinAdd1Ct ++; 
			Env.actinAdd1CtCos += Pt3D.Dot(Player.listeria.uVec,rod.uVec);
			return;
		}
		if (rod.tipClearance < Env.actinAdd2Dist) { 
			Env.actinAdd2Ct ++; 
			Env.actinAdd2CtCos += Pt3D.Dot(Player.listeria.uVec,rod.uVec);
			return;
		}
		if (rod.tipClearance < Env.actinAdd3Dist) { 
			Env.actinAdd3Ct ++; 
			Env.actinAdd3CtCos += Pt3D.Dot(Player.listeria.uVec,rod.uVec);
			return;
		}
	}
	
	public static void registerNewTether(Pt3D locOnLis) {
		Listeria.tethersFormed.addValue(locOnLis.x-Listeria.halfCylLength);
		Env.newTetherCt ++;
	}
	
	public static void registerCutTether(Pt3D locOnLis) {
		Listeria.tethersBroken.addValue(locOnLis.x-Listeria.halfCylLength);	// subtract half-cylinder length to use coord not rec1 as origin
		Env.cutTetherCt ++;
	}
	
	public static void registerCloseTip() {
		Env.closeTipCt ++;
	}
	
	public static void registerTipOnLisSphere() {
		Env.tipsOnLisSphere ++;
	}
	
	public static void addColForceInx (double val) {
		Env.aveColForceInx+=val;
	}
	
	public static void addFricForceInx (double val) {
		Env.aveFricForceInx+=val;
	}
	
	public static void addNormForce (double val) {
		Env.aveNormForce+=val;
	}
	
	public static void registerFailedTether() {
		Env.failedTether++;
	}
	
	public static void setNewToonDist () {
		double xDelta = listeriaLength/numMeasures;
		double offset1 = -0.2;
		double offset2 = -0.3;
		double curX;
		double normSum = 0;
		for (int i=0;i<numMeasures;i++) {
			curX = i*xDelta;
			double f1 = rdmAmp*Math.sin(0.9*Math.PI*(curX-offset1)/listeriaLength);
			if (f1 < 0) { f1 = 0; }
			double f2 = (1-rdmAmp)*Math.sin(1.8*Math.PI*(curX-offset2)/listeriaLength);
			if (f2 < 0) { f2 = 0; }
			if (i > 12) { f2 = 0; }
			
			actAMeasure[i] = f1 + f2;
			normSum += actAMeasure[i];
		}
		
		for (int i=0;i<numMeasures;i++) {
			actAMeasure[i] = actAMeasure[i]/normSum;
			actAProb[i] = AF*actAMeasure[i];
			if (actAProb[i] > 1) { actAProb[i] = 1; }
		}
	}
	/*public static void setNewToonDist () {
		double xDelta = listeriaLength/numMeasures;
		double curX;
		double normSum = 0;
		for (int i=0;i<numMeasures;i++) {
			curX = i*xDelta;
			actAMeasure[i] = Math.sin(0.5*Math.PI*(curX)/listeriaLength) + 2*Math.cos(Math.PI*(curX-offSet)/listeriaLength);
			if (actAMeasure[i] < 0) { actAMeasure[i] = 0; }
			normSum += actAMeasure[i];
		}
		
		for (int i=0;i<numMeasures;i++) {
			actAMeasure[i] = actAMeasure[i]/normSum;
			actAProb[i] = AF*actAMeasure[i];
			if (actAProb[i] > 1) { actAProb[i] = 1; }
		}
	}*/
	
	
	
	
	
}
