package rocketBugs;
/*  Linker ... this class defines the general kind of thing that links two Players
	together, e.g.  an ActA protein links together actin filaments and the listeria 
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class Linker {
	static Linker [] theLinkers = new Linker [Env.maxLinkers]; 
	static int linkerCt = 0;
	static LinkThreads linkTs = new LinkThreads();
	static final Pt3D zeroVec = new Pt3D();	// just a zero vector
	Pt3D locOnLis = new Pt3D();		// location relative to listeria.rec1 in listeria's local (x) coordinate system
	Player player1;
	Player player2;
	int idnum;										// defines position in array of Linkers
	boolean remove = false;							// flag for linker removal
	boolean removable = false;						// permanent or temporary linker
	boolean linked = false;							// flag to indicate that two players are linked by this Linker
	double linkMag;									// magnitude of link force
	Pt3D linkForce = new Pt3D();					// force vector for link, player2 on player1
	Pt3D linkUVec = new Pt3D();						// force unit vector from 2 to 1
	Pt3D linkPt1 = new Pt3D();						// link point on player1 in X
	Pt3D linkPt2 = new Pt3D();						// link point on player2 in X
	double attenVal;
	static final Object linkerLock = new Object();	// generic object for synchronizing new linker creation
	// each linker has its own rdm index
	int uRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*Env.numOfRdmNumbers));
	int cRdmIndex = (int)(Math.floor(Env.mtRNG.raw()*Env.numOfRdmNumbers));

	// time-step averaging
	static int stepsToTrackAndAverage = 3;
	ValueTracker forceMagTrack = new ValueTracker(stepsToTrackAndAverage);
	ValueTracker stretchTrack = new ValueTracker(stepsToTrackAndAverage);

	public Linker () {
	
	}
	
	static class LinkThreads extends ThreadSet {
		public LinkThreads () {
			super (Env.numberOfLinkThreads, "Link Threads");
		}
		
		public void divideAndConquer(int jobId) {
			this.jobId = jobId;
			switch (jobId) {
				case Env.linkSum1Start:
					for (int i=0; i <= numThreads; i++) {
						jobDiv[i] = i*linkerCt/numThreads;	// divide the job amongst threads
					}
					spawn(); break;
				case Env.linkUpdateStart:
					for (int i=0; i <= numThreads; i++) {
						jobDiv[i] = i*linkerCt/numThreads;	// divide the job amongst threads
					}
					spawn(); break;
			}
		}
		
		public void regroup (int jobId) {
			switch (jobId) {
				case Env.linkSum1Stop:
					gather(); break;
				case Env.linkUpdateStop:
					gather(); break;
			}
		}
		
		public void execute (int threadId) {
			switch (jobId) {
				case Env.linkSum1Start:
					for (int i = jobDiv[threadId]; i < jobDiv[threadId+1]; i++) {
						theLinkers[i].enforceLink();
					}
					break;
				case Env.linkUpdateStart:
					for (int i = jobDiv[threadId]; i < jobDiv[threadId+1]; i++) {
						theLinkers[i].updateState();
					}
					break;
			}
				
		}
	}
	
	public void sepaku () {
		player1 = null;
		player2 = null;	
		linkForce = null;
		linkUVec = null;
		linkPt1 = null;
		linkPt2 = null;
	}

	
	// some methods to be overwritten by specific linkers
	public void updateState () { talkln ("Linker updateState() should never be called by " + String.valueOf(this)); }
	public void calculateLinkForce () { talkln ("Linker calculateLinkForce() should never be called by "+ String.valueOf(this)); }
	public void checkPlayers() { talkln ("Linker checkPlayers() should never be called by "+ String.valueOf(this)); }
	public void showYourself (Graphics g, double scale, PinCam cam) { talkln ("Linker showYourself() should never be called by "+ String.valueOf(this)); }
	
	public static void reportRunStats() {
		int linkedCt = 0;
		for (int i=0; i<linkerCt; i++) {
			if (theLinkers[i].linked) { linkedCt++; }
		}
		talkln ("Linker Stats: #Linkers = " + String.valueOf(linkerCt) + " with " + String.valueOf(linkedCt) + " linked. " );
	}
	
	public static int countStrainedTethers1 () {
		int strainCt = 0;
		for (int i=0;i<linkerCt;i++) {
			if (theLinkers[i].linked) {
				double stretch = theLinkers[i].stretchTrack.averageVal();
				if ((stretch < Env.maxTetherStretch/2) & (stretch > 0)) { strainCt++; }
			}
		}
		return strainCt;
	}
	
	public static int countStrainedTethers2 () {
		int strainCt = 0;
		for (int i=0;i<linkerCt;i++) {
			if (theLinkers[i].linked) {
				double stretch = theLinkers[i].stretchTrack.averageVal();
				if (stretch > Env.maxTetherStretch/2) { strainCt++; }
			}
		}
		return strainCt;
	}
	
	
	public static void linkerCleanup () {
		int j = 0;
		while (j < linkerCt) {
			if (theLinkers[j].remove) {
				removeLinker(theLinkers[j]);
			} else {
				j++;	// move on to next in list
			}
		}
	}
	
	public static void addLinker (Linker addMe) {
		synchronized (linkerLock) {
			addMe.idnum = linkerCt;
			theLinkers[linkerCt] = addMe;
			linkerCt++;
		}
	}
	
	public static void removeLinker (Linker rmMe) {
		synchronized (linkerLock) {
			int swapId = rmMe.idnum;						// store swap location
			theLinkers[swapId] = theLinkers[linkerCt-1];		// swap last in list into this spot
			theLinkers[swapId].idnum = swapId;				// give new Id
			rmMe.sepaku();									// null out all links in the linker to remove
			linkerCt --;									// decrement linker counter
		}
	}
	
	public void enforceLink () {
		if (linked) {
			player1.link(linkPt1, linkForce);
			player2.link(linkPt2, Pt3D.Reverse(linkForce));
			if(Env.reportRegionInfo){player1.regionLink(linkPt1, linkForce);}
		}
	}
	
	public static void enforceAllLinks () {
		for (int i=0; i < linkerCt; i++) {
			theLinkers[i].enforceLink();
		} 
	}
	
	public static void showAllLinkers (Graphics g, double scale, PinCam cam) {
		for (int i=0; i<linkerCt; i++) {
			theLinkers[i].showYourself(g, scale, cam);
		}
	}
	
	public static void talkln (String info) {
		FileOps.talkln(info);
	}
	
	public static void talk (String info) {
		FileOps.talk(info);
	}
	

}