package rocketBugs;
/*	The camera object... definition and methods for a pinhole camera */

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import java.lang.Math;

public class PinCam {
	double focalLength;
	Pt3D zoomCen, pinPos, offset;	// pinhole pinPos and offsets for top-left film plane corner
	Pt3D e, v, h;		// unit vectors... e is along pinhole sight, v is vertical, h is horizontal
	Pixel pixOffset = new Pixel ();  	// store any drawing offset for this camera
	Pixel pixLimits = new Pixel ();	// define the maximum x and y to draw for this camera
	
	public PinCam (Pt3D zoomCen, Pt3D offset, Pt3D e, double focalLength) {
		this.zoomCen = zoomCen;
		pinPos = Pt3D.Add(zoomCen, focalLength, e);
		this.offset = offset;
		this.e = e;
		this.focalLength = focalLength;
		v = new Pt3D (0,1,0);
		updateUnitVecs ();
	}
	
	public Pixel ptToPix (Pt3D pt, double scale) {
		Pixel retPix = new Pixel();
		// calculate distance from pinhole location to pt
		double d1 = Pt3D.ptDist(pinPos, pt);
		// find the cosine of the angle between this ray and the unit vector for the camera
		Pt3D ptRay = Pt3D.Add(1/d1, pt, -1/d1, pinPos);
		double cosAng = Pt3D.Dot(e, ptRay);
		// if the object is behind or more than 90 degree to side, return flagged Pixel
		if (cosAng < 0) { retPix.show = false; }
		// use the similar right triangle to find distance to film plane along -ptRay
		double d2 = focalLength/cosAng;
		// location of projected pt is just pinhole location minus d2 in -ptRay direction
		Pt3D mirrorPt = Pt3D.Add(pinPos, -d2, ptRay);
		Pt3D projPt = Pt3D.Add(2, zoomCen, -1, mirrorPt);
		// next find the components in h and v of the vector to this point, minus offset components
		double hComp = Pt3D.Dot(projPt, h) - Pt3D.Dot(offset,h);
		double vComp = Pt3D.Dot(projPt, v) - Pt3D.Dot(offset,v);
		// then define the pixel
		int hInt = (int) Math.rint(hComp*scale);
		int vInt = (int) Math.rint(vComp*scale);
		if ((hInt > pixLimits.x) | (hInt < 0) | (vInt > pixLimits.y) | (vInt < 0)) { retPix.show = false; } 
		retPix.x = hInt + pixOffset.x;
		retPix.y = vInt + pixOffset.y;
		retPix.viewDist = d1;
		return retPix;
	}
	
	public void changePos (Pt3D zoomCen, Pt3D offset) {
		this.zoomCen = zoomCen;
		pinPos = Pt3D.Add (zoomCen, focalLength, e);
		this.offset = offset;
	}
	
	public void changeDir (double angle) {
		e = new Pt3D (Math.sin(angle),0,Math.cos(angle));
		updateUnitVecs ();
	}
	
	public void changeFocalLength (double focalLength) {
		this.focalLength = focalLength;
	}
	
	private void updateUnitVecs () {
		h = Pt3D.Cross(v,e);
	}
	
	private void talk (String info) {
		System.out.println(info);
	}
	
}
		
		
		