package rocketBugs;
/* 
	RocketBugs.... the main linked file for the cellular protrusion mechanism study
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.text.*;
import java.io.*;
import java.util.*;

public class RocketBugs {
	static RBFrame theFrame;
	static PlayerTools playerTools;
	static FileOps fileOps;
	static SphImpPlayers sphImp;
	static Rdm rdm;						// class defining an array of random numbers for later use
	static ThreadSet [] tSets;
	TimeLoop timeLoop = new TimeLoop();
	Object flagGUI = new Object();	// just a generic object on which to synchronize GUI changes
	boolean paused = true;
	boolean waitOnGUI = false;	// boolean flag to let iteration loop know that a GUI change is waiting to implement itself
	boolean readICs =false;	// flag for reading initial conditions from a file
	boolean verboseMode = false; 	// flag to signal printing of periodic system info
	// booleans and strings for movie making
	static boolean startRawMovieNow = false;		
	static boolean startQkMovieNow = false;
	DecimalFormat fileIdFormat = new DecimalFormat ("#000000000.#;#000000000.#");
	DecimalFormat timeOutFormat = new DecimalFormat ("#0.00#; #0.00#");
	Date timeNow;
	
	// some timing counters
	Timer diffTimer = new Timer("Diffusion calculations");
	Timer reacTimer = new Timer("Reaction calculations");
	Timer collDR1Timer = new Timer("Collision rough detection 1st plane");
	Timer collDR2Timer = new Timer("Collision rough detection 2nd plane");
	Timer collDFTimer = new Timer("Collision fine detection");
	Timer collEf1Timer = new Timer("Collision effects");
	Timer collEf2Timer = new Timer("Collision effects");
	Timer moveTimer = new Timer("Moving players");
	Timer interactionTimer = new Timer("Interactions Simulation");
	Timer maintainTimer = new Timer("Maintenance of players");
	Timer linkUpdateTimer = new Timer("Update of ActAs");
	Timer attenTimer = new Timer("Attenuation of Forces");

	
	public RocketBugs (String [] inputParams) {
		if (inputParams.length != 0) { parseParams(inputParams); }	// figure out what options have been specified
		
		
		// create instance of file operations class ... once this is created then "talk" and "talkln" can be used
		fileOps = new FileOps (this);
		
		// report input parameters
		reportParams();
		
		// create an array of random numbers for later access
		rdm = new Rdm (Env.numOfRdmNumbers, 10, Env.deltaT);
		Player.rdm = rdm;		// set pointer 
		
		// create instance of structure tools class
		playerTools = new PlayerTools (this);
		
		// set Players reference to this
		Player.MC = this;
		
		// add the Listeria bacteria (if requested)
		if ((Env.listeriaPresent)) {
			Pt3D listeriaPos = new Pt3D(10,18,10);
			Pt3D listeriaUVec = new Pt3D(0, -1, 0);
			Player.listeria = new Listeria (listeriaPos,listeriaUVec,sphImp);
		}  
		
		// create instance of the spherical implicit player grid
		sphImp = new SphImpPlayers (this, Env.molarGridScale);
		Player.impP = sphImp;
		
		// initialize collision array
		Collision.initializeCollisionArray();
		
		// add a circular membrane
		//playerTools.makeMembraneNodes ();
		
		// read in initial conditions if specified
		if (readICs) { 
			fileOps.loadConfig(true); 
		} else {
			for (int i = 0; i < Env.numberOfInitialRods; i++) { playerTools.makeRandomRod(new Pt3D(1,1,1),Env.dimensions,A_Rod.minMonomers); }
		}
		
		// if we aren't running remotely then create the GUI...
		if (!Env.remote) { theFrame = new RBFrame (Player.thePlayers, this); }
		
		// start the time loop 
		timeLoop.start();
	}
	
	class TimeLoop extends Thread {
		double starttime,finishtime;
		int saveConfCounter = 0;	// counter for saving config during run
		int updateCounter = 0;
		int rawMovieCounter = 0;
		int qkMovieCounter = 0;
		int lisInfoCounter = 0;
		int histCounter = 0;
		int timesCleared = 0;
		int secs,tenthsecs,hunsecs;
		
		public void run() {
			loadAllThreadSets();	//**** create all these threads 'cause we're running multithreaded
				
			talkln("Starting integration steps.....");
			starttime = (double)java.lang.System.currentTimeMillis();
			
			while (Env.simulationTime <= Env.runTime) { doLoop(); }	// **** the run loop
			
			finishtime = (double)java.lang.System.currentTimeMillis();
			reportThreadTimes(starttime, finishtime);
			try { this.sleep(50000); } catch (Exception e) { }
			System.exit(0);
		}
		
		public void reportRates () {
			talk ("arpOnProb = " + String.valueOf(ActA.arpOnProb));
			talk (", eff k+ = " + String.valueOf(ActA.arpOnProb/(Env.ARPConc*Env.deltaT)));
			talk (", eff k- = " + String.valueOf(Env.kBoundActAArp_Off));
			talkln (", eff KD = " + String.valueOf(Env.kBoundActAArp_Off*Env.ARPConc*Env.deltaT/ActA.arpOnProb));
			talk ("actinOnProb = " + String.valueOf(ActA.actinOnProb));
			talk (", eff k+ = " + String.valueOf(ActA.actinOnProb/(Env.PROFILIN_ATP_ACTINConc*Env.deltaT)));
			talk (", eff k- = " + String.valueOf(Env.kActAActin_Off));
			talkln (", eff KD = " + String.valueOf(Env.kActAActin_Off*Env.PROFILIN_ATP_ACTINConc*Env.deltaT/ActA.actinOnProb));
		}
		
		public void doLoop() {
			//reportRates();
			if (Player.listeria.haywire) { System.exit(0); }	// quit run if the listeria has gone "blip"
			
			if (! Env.remote) { 
				theFrame.showAll(); 
			}
			synchronized (flagGUI) {  // make GUI and File ops thread safe
				if (! paused) {
					mThreadRun();

					Env.counter++;
					saveConfCounter++;
					updateCounter++;
					rawMovieCounter++;
					qkMovieCounter++;
					lisInfoCounter++;
					histCounter++;
					Env.simulationTime += Env.deltaT;
				} 
				//System.out.println("waitOnGUI is " + String.valueOf(waitOnGUI));
				flagGUI.notifyAll();
				while (waitOnGUI) { try { flagGUI.wait(10); } catch(Exception e) { talkln (" Exception waiting on GUI"); }}
			}
			if (! Env.remote) { try { this.sleep(1); } catch (Exception e) {e.printStackTrace();} }
			
			if (updateCounter >= Env.updateFrequency) {
				playerTools.superStructureMaintenance();
				Player.listeria.checkViscDragIncrease();
				updateCounter = 0;
				if (! Env.remote) {
					theFrame.updateDisplay(Env.simulationTime, Player.playerCt);
				} else {
					if (verboseMode) {
						timeNow = new Date();
						talk ("Simulation Time = " + String.valueOf(timeOutFormat.format(Env.simulationTime)));
						talk (", Number of Players =  " + String.valueOf(Player.playerCt));
						talk (", Number of Tethers = " + String.valueOf(Player.listeria.linkerCt));
						talkln (", Clock = " + timeNow.toString());
					}
				}
			} 
			
			// random number refreshing
			Rdm.ckURdmRefresh();
			Rdm.ckCRdmRefresh();
				
			if ((saveConfCounter == Env.saveConfFrequency) & (Env.qkMovie)) {
				String confName = Env.datFilePath + "/" + Env.qkMovieFileName + String.valueOf(fileIdFormat.format(timesCleared*32700+Env.counter)) + ".data";
				fileOps.saveConfig(confName);
				saveConfCounter = 0;
			}

			if (((rawMovieCounter >= Env.rawMovieFrameInt) & (Env.rawMovie)) | (startRawMovieNow)) { 
				String fileName = Env.rawMoviePath + "/" + Env.rawMovieFileName + String.valueOf(fileIdFormat.format(timesCleared*32700+Env.counter));
				theFrame.zoomFrame.showRocketBugs (sphImp);  // paint before RAW save
				fileOps.makeMovieFrame(fileName,"raw"); 
				startRawMovieNow = false;
				rawMovieCounter = 0;
				//counter++;
				//theFrame.zoomFrame.rotateRocketBugsAngle(Env.rotateStepPerMovieFrame);		  // rotate view angle
			}
			if (((qkMovieCounter >= Env.qkMovieFrameInt) & (Env.qkMovie)) | (startQkMovieNow)) {
				String fileName = Env.qkMoviePath + "/" + Env.qkMovieFileName + String.valueOf(fileIdFormat.format(timesCleared*32700+Env.counter));
				fileOps.makeMovieFrame(fileName,"qk");
				startQkMovieNow = false;
				qkMovieCounter = 0;
				// report on run
				Player.reportRunStats(); 
				Linker.reportRunStats();
			}
			
			if ((lisInfoCounter >= Env.lisInfoFreq) & (Env.qkMovie)) {
				fileOps.saveListeriaPositionalInfo(Player.listeria);
				if(Env.reportRegionInfo){fileOps.saveListeriaRegionalInfo(Player.listeria); fileOps.saveListeriaForceInfo(Player.listeria);}
				Env.resetEventCounters();
				lisInfoCounter = 0;
			}
			
			if ((histCounter >= Env.histFreq) & (Env.qkMovie)) {
				//histograms
				Listeria.updateHistos();
				HistogramPlus.writeAllHistos();
				histCounter = 0;
				Listeria.resetHistos();
			}
				
			if (Env.counter == 32700) {
				talk ("clearing counters....");
				Env.counter = 0;
				timesCleared++;
				talkln ("done.");
			}
			
		}	
	}
	
	public void waitOnTimeLoop() {
		synchronized (flagGUI) {
			waitOnGUI = true;
			try { flagGUI.wait(); } catch (Exception e) { talkln ("Exception waiting in waitOnTimeLoop"); }
		}
	}
	
	public void notifyTimeLoop() {
		synchronized (flagGUI) {
			waitOnGUI = false;
			flagGUI.notifyAll();
		}
	}
	
	private void loadAllThreadSets () {
		tSets = new ThreadSet [5];
		tSets[0] = Player.pTs;
		tSets[1] = Collision.colTs;
		tSets[2] = Linker.linkTs;
		tSets[3] = SphImpPlayers.diffTs;
		tSets[4] = FrictionEvent.fricTs;
	}
	
	private void startAllThreadSets (int waveNum) {
		for (int i=0; i < tSets.length; i++) {
			tSets[i].divideAndConquer(waveNum);
		}
	}
	
	private void waitOnAllThreadSets(int waveNum) {
		for (int i=0; i < tSets.length; i++) {
			tSets[i].regroup(waveNum);
		}
	}
	
	
	private void mThreadRun() {
		/* waveNum commands
			see Env.java for wavenum definitions
		*/
		for (int waveNum = 0; waveNum < Env.numberOfWaves; waveNum++) {
			startAllThreadSets(waveNum);
			waitOnAllThreadSets(waveNum);
			if (waveNum == Env.playerCleanupStart) { playerCleanup(); }
			if (waveNum == Env.frictionCleanupStart) { frictionCleanup(); }
			if (waveNum == Env.linkerCleanupStart) { linkerCleanup(); }
		}
	}
	
	public void playerCleanup() {
		Player.playerCleanup();   // players
		Collision.reset();		// reset collision ct to zero
	}
	
	public void linkerCleanup() {
		Linker.linkerCleanup();
	}
	
	public void frictionCleanup() {
		FrictionEvent.reset();
	}
	
	public void collisionAndLinkSteps () {
		Collision.enforceAllCollisions();
		Linker.enforceAllLinks();
	}
	
	public void bioReactions () {
		sphImp.toggleArray();
		reacTimer.start();
		sphImp.proteinReactions (0, sphImp.rDim); 
		reacTimer.stop();
		reacTimer.inc();
	}
			
	public void implicitDiffusion () {
		diffTimer.start();
		sphImp.resetDiffusibles();
		int diffuser = sphImp.getDiffusible();
		while (diffuser != -1) {
			sphImp.diffusionCalc (diffuser);
			diffuser = sphImp.getDiffusible();
		}
		diffTimer.stop(); diffTimer.inc();
	}
	
	public void moveAllPlayers () {
		moveTimer.start();
		for (int i = 0; i < Player.playerCt; i++) {
			if (!Player.thePlayers[i].structureMember) {		// players that are part of a superstructure are moved by their subsumer
				Player.thePlayers[i].movePlayer();
			}
		}
		moveTimer.stop(); moveTimer.inc();
	}
	
	
	public void reportThreadTimes (double starttime, double finishtime) {
		talkln ("");
		talkln ("That run took " + String.valueOf((finishtime-starttime)/1000) + " seconds with a final player count at " + String.valueOf(Player.playerCt));
		talkln ("Time breakdown: ");
		diffTimer.reportTime();
		reacTimer.reportTime();
		collDR1Timer.reportTime();
		collDR2Timer.reportTime();
		collDFTimer.reportTime();
		collEf1Timer.reportTime();
		moveTimer.reportTime();
	}
	
	private void parseParams(String [] inputParams) {
		for (int i = 0; i < inputParams.length; i++) {
			if ((inputParams[i].equals("-help")) | (inputParams[i].equals("?"))) {
				System.out.println ("The following command line arguments are accepted.....");
				System.out.println ("	-help	 prints this help message");
				System.out.println ("	-r remote start (no local GUI) with default initial conditions");
				System.out.println ("	-v verbose mode... program prints out periodic run info");
				System.out.println ("	-ic [filename]	read initial conditions from the file given");
				System.out.println ("	-dt [double] use this value for the fixed time-step");
				System.out.println ("	-it [double] start with this given simulation time");
				System.out.println ("	-mf [filename]	write a movie to the named file");
				System.out.println ("	-mt ...sets multi-threading flag to true- use MT version of code");
				System.out.println ("	-ld [double] use this factor to scale Listeria diffusivity");
				System.out.println ("	-lm [double] set the link force multiplier relative to collision force");
				System.out.println ("	-tl [double] set the tether length in multiples of G-Actin diameters");
				System.out.println ("	-ts [double] set the maximum tether strain");
				System.exit(0);
			}
			if (inputParams[i].equals("-r")) {
				Env.remote = true;
				paused = false;
			}
			if (inputParams[i].equals("-dt")) {
				Double dt = Double.valueOf(inputParams[i+1]);
				Env.deltaT = dt.doubleValue();	
			}
			
			if (inputParams[i].equals("-ic")) {
				File icFile = new File(inputParams[i+1]);
				if (icFile.exists()) {
					readICs = true;
					Env.icFile = icFile.getName();
					Env.icPath = icFile.getPath();
				} else {
					System.out.println ("Error with initial conditions file... try -help");
					System.exit(0);
				}
			}
	
			if (inputParams[i].equals("-v")) { 
				verboseMode = true; 
			}
			
			if (inputParams[i].equals("-ts")) {
				Double ts = Double.valueOf(inputParams[i+1]);
				Env.maxTetherStrain = ts.doubleValue();
			}
		
			if (inputParams[i].equals("-tl")) {
				Double tl = Double.valueOf(inputParams[i+1]);
				Env.tetherInGActin = tl.doubleValue();
			}
			
			if (inputParams[i].equals("-it")) {
				Double dd = Double.valueOf(inputParams[i+1]);
				Env.simulationTime = dd.doubleValue();
				Env.counter = (int)(Env.simulationTime/Env.deltaT);
			}
			
			if (inputParams[i].equals("-mf")) {
				File mvdir = new File(inputParams[i+1]);
				File altdir = new File (inputParams[i+1]);
				int j=1;
				while (altdir.isDirectory()) {
					System.out.println (altdir.getName() + " exists.... keeping file name BUT changing directory name");
					altdir = new File(inputParams[i+1] + "." + String.valueOf(j));
					j++;
				}
				altdir.mkdir();
				File qkdir = new File (altdir.getAbsolutePath() + File.separator + mvdir.getName() + ".QK");
				qkdir.mkdir();
				Env.qkMovieFileName = mvdir.getName();
				Env.qkMoviePath = qkdir.getAbsolutePath();
				// make a directory for .dat files
				File datdir = new File (altdir.getAbsolutePath() + File.separator + mvdir.getName() + ".DATA");
				datdir.mkdir();
				Env.datFilePath = datdir.getAbsolutePath();
				// make a directory for source files
				File srcdir = new File (altdir.getAbsolutePath() + File.separator + mvdir.getName() + ".SRC");
				srcdir.mkdir();
				Env.srcFilePath = srcdir.getAbsolutePath();
				//*******
				Env.qkMovie = true;
				startQkMovieNow = true;
			}
		}
		Env.recalculateDependents();
	}
	
	public void reportParams () {
		timeNow = new Date();
		if (Env.qkMovie) { 
			talkln (" ******* " + Env.qkMovieFileName + " : " + timeNow.toString() + " *******");
			talkln ("Quick format movie files will be stored in the directory" + Env.qkMoviePath); 
		}
		if (readICs) { talkln ("Initial conditions file is '" + String.valueOf(Env.icFile) + "'"); }
		if (Env.remote) { talkln ("Running remotely... no GUI"); }
		talkln ("	Multithreading is always ON now");
		talkln ("	Initial time is " + String.valueOf(Env.simulationTime));
		talkln ("	Time-step is " + String.valueOf(Env.deltaT));
		talkln ("	Maximum tether strain is " + String.valueOf(Env.maxTetherStrain) + " which corresponds to " + String.valueOf(Env.maxTetherStretch) + " / " + String.valueOf(Env.actATetherLength) + " microns ");
		talkln ("	Tether length is " + String.valueOf(Env.tetherInGActin) + "x G-Actin diameter");
		talkln ("	Collision attenuation is " + Env.collisionAtten);
		talkln ("	Link attenuation is " + Env.linkAtten);
		talkln (" ***************************************************************************");
	}
	
	public void clearAll() {
		for (int i=0; i < Player.playerCt; i++) { Player.thePlayers[i] = null; }
		Player.playerCt = 0;
		Env.simulationTime = 0;
	}
			
	private void talkln (String info) {
		FileOps.talkln(info);
	}
	
	private void talk (String info) {
		FileOps.talk(info);
	}

}
		
		
		
		
		
		
		
		
		
		
		