package rocketBugs;
/* 
	StereoPanel.... the class defining the panel for stereo vision (3D) display
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;

public class StereoPanel extends JPanel {
	StereoFrame pFrame;		// a reference back to the parent JFrame of this panel
	Image theImage;
	Graphics thePalate;
	int winSize;
	double scale,windowfact,winScale;
	double halfWindow;		// dimension of half the zoom window width
	double dispAng = 0;
	double [] trigDispAng = {0,1};  	// store {sin(dispAng), cos(dispAng)}
	Pt3D offset = new Pt3D ();	// store {x,y,z} offsets for the zoom window
	Pt3D stereoCen = new Pt3D ();	// store zoom center
	int [] cursorPos = {0,0};	// store initial cursor position for dispAng and Z changes
	PinCam stereoCamL, stereoCamR;
	double eyeSep, eyeAng;	// stereo view variables
	
	public StereoPanel (int winSize, double [] cenPt, double scale, StereoFrame pFrame) {
		this.pFrame = pFrame;
		this.winSize = winSize;
		this.scale = scale;
		windowfact = winSize/Env.dimensions.x;
		winScale = scale*windowfact;
		halfWindow = winSize/(2*winScale);
		stereoCamL = new PinCam (stereoCen, offset, new Pt3D (0,0,1), .8);
		stereoCamR = new PinCam (stereoCen, offset, new Pt3D (0,0,1), .8);
		stereoCamL.pixOffset = new Pixel (0,0);
		stereoCamR.pixOffset = new Pixel (winSize,0);
		stereoCamL.pixLimits = new Pixel (winSize,winSize);
		stereoCamR.pixLimits = new Pixel (winSize,winSize);
		newStereoCenter(cenPt);
		setSize(winSize,winSize);
	}
	
	public void initialize() {
		theImage = createImage(2*winSize,winSize);
		thePalate = theImage.getGraphics();
	}
	
	public void newStereoCenter (Pt3D stereoCen) {
		this.stereoCen = stereoCen;
		offsetsFromStereoCen ();
		updatePinCams ();
	}
	
	public void newStereoCenter (double [] cenPt) {
		stereoCen.x = cenPt[0];
		stereoCen.y = cenPt[1];
		stereoCen.z = cenPt[2];
		offsetsFromStereoCen ();
		updatePinCams ();
	}
	
	public void offsetsFromStereoCen () {
		offset.y = stereoCen.y - halfWindow;
		offset.x = stereoCen.x - halfWindow*trigDispAng[1];
		offset.z = stereoCen.z + halfWindow*trigDispAng[0];
	}
	
	public void newScale (double newS) {
		scale = newS;
		winScale = scale*windowfact;
		halfWindow = winSize/(2*winScale);
		offsetsFromStereoCen();
		updatePinCams ();
	}
	
	public void newDispAng (double newAng) {
		dispAng = newAng;
		trigDispAng[0] = Math.sin(dispAng);
		trigDispAng[1] = Math.cos(dispAng);
		offsetsFromStereoCen();
		updatePinCams ();
	}
	
	public void chgZValue (double deltaZ) {
		stereoCen.x += deltaZ*trigDispAng[0];
		stereoCen.z += deltaZ*trigDispAng[1];
		offsetsFromStereoCen();
		updatePinCams ();
	}
	
	public void setCursorPos (int x, int y) {
		cursorPos[0] = x;
		cursorPos[1] = y;
	}	
	
	public void chgRocketBugsAng (int x, int y) {
		double newAng = dispAng + (Math.PI/2)*(cursorPos[0]-x)/(winSize/2);
		newDispAng (newAng);
		pFrame.updateInfo();	
	}
	
	public void chgRocketBugsZ (int x, int y) {
		double deltaZ = Env.dimensions.z*(cursorPos[1]-y)/(winSize);
		chgZValue (deltaZ);
		pFrame.updateInfo();	
	}
	
	public void newFocalLength (double focalLen) {
		stereoCamL.changeFocalLength (focalLen);
		stereoCamR.changeFocalLength (focalLen);
	}
	
	public void newEyeSep (double eyeSep) {
		this.eyeSep = eyeSep;
		updatePinCams ();
	}
	
	public void newEyeAng (double eyeAng) {
		this.eyeAng = eyeAng*Math.PI/180;
		updatePinCams ();
	}
	
	public void updatePinCams () {
		stereoCamL.changePos (stereoCen, offset);
		stereoCamL.changeDir(dispAng);
		Pt3D rightShift = Pt3D.Scale(eyeSep, stereoCamL.h);
		stereoCamR.changePos(Pt3D.Add(stereoCen,rightShift), Pt3D.Add(offset, rightShift));
		stereoCamL.changeDir(dispAng-eyeAng/2);
		stereoCamR.changeDir(dispAng+eyeAng/2);
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint (Graphics g) {
		g.drawImage(theImage,0,0,null);
	}
	
	public void showPlayers (Player [] thePlayers, int numOfPlayers) {
		for (int i = 0; i < numOfPlayers; i++) {
			thePlayers[i].stereoVision(thePalate,winScale, stereoCamL, stereoCamR);
		}
	}
	
	public void showCellInside (SphImpPlayers sphImp) {
		//sphImp.showMemGrid(thePalate,winScale,offset);
	}
	
	public void showElementConc (SphImpPlayers sphImp, int element) {
		//sphImp.showElementConc (thePalate,winScale,offset,element);
	}
	
	public double [] convertPixelToPoint (int xPix, int yPix) {
		double [] point = new double [3];
		point[0] = offset.x + trigDispAng[1]*(double)(xPix)/winScale;
		point[1] = offset.y + (double)(yPix)/winScale;
		point[2] = offset.z - trigDispAng[0]*(double)(xPix)/winScale;
		return point;
	}

	public void clearImage() {
		thePalate.setColor(Color.black);
		thePalate.fillRect(0,0,2*winSize,winSize);
	}
	
	private void talk (String info) {
		System.out.println(info);
	}
}