package rocketBugs;
/*
	SuperStructure... this class, which extends Player, provides some common methods and variables
	used by SuperStructures, which are Players who contain (subsume) other Players
*/

/*  <RocketBugs - for the simulation of actin-based Listeria motility>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math.*;
import java.awt.*;

public class SuperStructure extends Player {
	static SuperStructure [] structures = new SuperStructure [Env.maxSuperstructs];
	static int structureCt = 0;
	static Object structureLock = new Object (); //generic object for creation/removal of superstructs
	short strucId;		// array position of this superstructure in the complete superstructure list
	Player [] members = new Player [2000];
	int [] memIds = new int [2000];		// array of ints to hold data on members during read from file only
	int numMems = 0;
	double majorAxis,minorAxis;
	double [] fVeloc = new double [3]; 	// array to store angular velocities in fixed-frame
	Pt3D tempPt = new Pt3D();			// blank Pt3D to use whenever
	Pt3D tetherForceVec, tetherTorqueVec;  // recycled in tether effects calculations
	Pt3D coordInLMFrame = new Pt3D();
	// some variables for debugging
	int oldNumMems = -1;
	
	// special flags
	boolean linked = false;		// linked to Listeria?
	
	
	public SuperStructure () {
		super();
		structure = true;
		SuperStructure.addStructure(this);
		
	}
	
	public static void addStructure (SuperStructure addMe) {
		synchronized (structureLock) {
			addMe.strucId = (short)structureCt;
			structures[structureCt] = addMe;
			structureCt++;
		}
	}
			
	public static void removeStructure (SuperStructure rmMe) {
		// check if any members that need to be dissociated
		for (int i=0; i<rmMe.numMems; i++) {
			if (i>0) {
				talkln ("Removing member " + String.valueOf(i) + " from a structure about to be removed... why are we here?");
			}
			rmMe.removeMember(rmMe.members[i]);
		}
		synchronized (structureLock) {
			short swapId = rmMe.strucId;						// store swap location
			structures[swapId] = structures[structureCt-1];		// swap last in list into this spot
			structures[swapId].strucId = swapId;				// give new Id
			rmMe.sepaku();										// null out all links in the structure to remove
			structureCt += -1;									// decrement structure counter
		}
	}
	
	public void addMember (Player player) {
		// first check to see if this player is already a structureMember.. remove if so
		if (player.structureMember) {
			player.myStructure.removeMember(player);
		}
		//  add to members of superstructure
		
		synchronized (structureLock) {
			player.structureMember = true;
			player.myStructure = this;
			player.structureId = numMems;
			members[numMems] = player;
			numMems++;	
		}
	}
	
	public void removeMember (Player player) {
		// remove from superstructure
		synchronized (structureLock) {
			if (player.structureId < 0) {
				talkln ("*** Error: tried to removeMember " + String.valueOf(player) + " from Structure " + String.valueOf(player.myStructure));
				return;
			} else {
				player.structureMember = false;
				player.myStructure = null;
				numMems += -1;
				int swapIdNum = player.structureId;
				player.structureId = -1;
				members[swapIdNum] = members[numMems];
				members[swapIdNum].structureId = swapIdNum;
				members[numMems] = null;
				if (numMems <= 1) { remove = true; } else { remove = false; }
			}
		}
	}
	
	// Some methods to be overridden by subclasses
	public void addSubBranch (A_Rod baseP) { talkln ("SuperStructure addSubBranch() should never be called"); }
	public void removeSubBranch (A_Rod baseP) { talkln ("SuperStructure removeSubBranch() should never be called"); }
	public void calculateProperties () { talkln ("SuperStructure calculateProperties() should never be called"); }
	
	public void initialize () {
		// this method assumes the uVec has been set, or it unchanged
		// euler angles from uVec
		angle.eulerAnglesFromUVec(uVec);
		// calculate cos and sin of Euler angles only once during each time step
		cosAng.cosine(angle);
		sinAng.sine(angle);
		// find the transformation matrices at this time step
		transMat ();
		coordInLMFrame.sub(coord,Player.listeria.coord);
		coordInLMFrame.XTox(Player.listeria);
		
		
	}
	
	public void movePlayer() {
		age += Env.deltaT;
		if (linkerCt > 0) { linked = true; }
		checkIfFixed();
		
		sumForces();
		
		// first store the position of each member relative to the structure
		Pt3D [][] relPos = new Pt3D[numMems][3];	// an array of Pt3D to store CM, unit, and tangent vector in body-fixed axis
		for (int i=0; i < numMems; i++) {
			tempPt.sub(members[i].coord, coord);
			relPos[i][0] = Pt3D.XToNewx(this,tempPt);
			relPos[i][1] = Pt3D.XToNewx(this,members[i].uVec);
			relPos[i][2] = Pt3D.XToNewx(this,members[i].t1Vec);
		}
	
		// explicit Euler approximation to ODE solution
		// first work in coordinates aligned with the rod....
		// transform forces and torques into body-fixed axis....
		bForceSum.XTox(this, forceSum);
		bTorqueSum.XTox(this, torqueSum);
		// add random forces, given in body-fixed frame
		if (Env.shakeActin) { 
			calcRandomForces();
			if (linked) {
				double scaleFac = linkerCt*((Env.tetherBrownianScale/majorAxis)-((Env.tetherBrownianScale/Env.cutoffLength)-(1.0)));
				//double scaleFac = linkerCt*Env.tetherBrownianScale/majorAxis;		//old (pre050106) way
				if (majorAxis > linkerCt * Env.cutoffLength) { scaleFac = 1.0; }
				//if (scaleFac < 1.0) { scaleFac = 1.0; }
				randForces.scale(1.0/scaleFac);
				randTorques.scale(1.0/scaleFac);
			}
			bForceSum.inc(randForces);
			bTorqueSum.inc(randTorques);
		}
		// now that the forces and torques are in the body fixed frame, we apply the eoms....
		//NEW CODE: if filaments at the sides, then rotation is restricted. If want original rotation, jus tmake Env.sideRotRestrictFactor = 1
		if((coordInLMFrame.x > -Listeria.halfCylLength) & (coordInLMFrame.x <Listeria.halfCylLength)){
			bRestrictRotGam.scale(Env.sideRotRestrictFactor, bRotGam);
			bVeloc.div(1.0e6, bForceSum, bTransGam);		// in micron/sec
			bAngVeloc.div(bTorqueSum, bRestrictRotGam); 	// in radians/sec
		}else {
			bVeloc.div(1.0e6, bForceSum, bTransGam);		// in micron/sec
			bAngVeloc.div(bTorqueSum, bRotGam);			// in radians/sec
		}
		
		
		// ....then transform back into fixed coordinate frame
		veloc.xToX(this, bVeloc);
		
		// to apply the body-fixed angular velocities, approximate new unit vector from arc of rotations
		double endTransInZ = -bAngVeloc.y * Env.deltaT;	// arclength out at 1 micron
		double endTransInY = bAngVeloc.z * Env.deltaT;
		uVec.setVals(1,endTransInY,endTransInZ);	// in body-fixed, not a unit vector yet
		uVec.xToX(this);	// make in fixed-frame, not a unit vector yet
		uVec.unitVec();		// make a unit vector
		
		// and finally move everything
		coord.inc(Env.deltaT, veloc);		// just position = velocity*time
		
		// body-fixed x angular velocity is same as euler angle phi rotation
		angle.z += bAngVeloc.x*Env.deltaT;
	
		// update transformation matrices, unit vector, etc for this new position
		initialize();
	
		// now update positions of all the members of this superstructure... same position relative to structure
		for (int i = 0; i < numMems; i++) {
			members[i].subsumedMove(coord, Pt3D.xToNewX(this,relPos[i][0]), Pt3D.xToNewX(this,relPos[i][1]), Pt3D.xToNewX(this,relPos[i][2]));
		}
		
		// reset counters
		resetCounters();
	}
	
	public void resetCounters () {
		super.resetCounters();
		linked = false;		
	}
	
	public void checkIfFixed() {
		for (int i=0;i<numMems;i++) {
			if (members[i].fixedInSpace) {
				fixedInSpace = true;
				return;
			}
		}
	}
	
	public static void structureProperties () {
		for (int i = 0; i < structureCt; i++) {
			structures[i].calculateProperties();
		}
	}
	
	public void report() {
		talkln ("SuperStructure " + String.valueOf(this) + " status:");
		talkln ("  numMems = " + String.valueOf(numMems));
		for (int i=0; i<numMems; i++) {
			talkln ("      member[" + String.valueOf(i) + "] = " + String.valueOf(members[i]));
			talkln ("          id = " + String.valueOf(members[i].structureId));
			talkln ("          structureMember = " + String.valueOf(members[i].structureMember));
			talkln ("          justRemoved = " + String.valueOf(members[i].justRemoved));
		}
	}
	
	public void tweezedMove(Pt3D mvPt) {
		Pt3D deltaPos = Pt3D.Sub(mvPt,coord);
		coord = mvPt;
	
		Player curP;
		for (int i = 0; i < numMems; i++) {
			curP = members[i];
			curP.coord.inc(deltaPos);
			curP.initialize();
		}
	}
	
	public void showCentroid (Graphics g, Pixel centroid, int cenRad) {
		g.setColor(Color.white);
		g.fillArc(centroid.x,centroid.y,cenRad,cenRad,45,90);
		g.setColor(Color.darkGray);
		g.fillArc(centroid.x,centroid.y,cenRad,cenRad,135,90);
		g.setColor(Color.white);
		g.fillArc(centroid.x,centroid.y,cenRad,cenRad,225,90);
		g.setColor(Color.darkGray);
		g.fillArc(centroid.x,centroid.y,cenRad,cenRad,315,90);
	}
	
	
}






